import { __decorate, __metadata, __param } from 'tslib';
import { InjectionToken, Injectable, Inject, ViewChild, ElementRef, Input, HostBinding, Component, Renderer2, HostListener, EventEmitter, Output, ContentChildren, QueryList, ChangeDetectorRef, Directive, Self, Optional, ViewChildren, ComponentFactoryResolver, ViewContainerRef, NgModule } from '@angular/core';
import { Subject, fromEvent, merge } from 'rxjs';
import { Easing } from '@tweenjs/tween.js';
import { delay, distinctUntilChanged, filter, debounceTime, tap } from 'rxjs/operators';
import { NG_VALUE_ACCESSOR, NG_VALIDATORS, FormControl, FormGroup, NgForm, FormsModule } from '@angular/forms';
import BScroll from '@better-scroll/core';
import Wheel from '@better-scroll/wheel';
import { CubicBezier } from 'tanbo-bezier';
import { DOCUMENT, Location, CommonModule } from '@angular/common';
import { NavigationStart, Router, PRIMARY_OUTLET, ChildrenOutletContexts, ActivatedRoute, RouteReuseStrategy } from '@angular/router';
import { trigger, transition, animate, keyframes, style, state } from '@angular/animations';

/**
 * 当前运行环境的枚举量
 */
var BrowserENV;
(function (BrowserENV) {
    BrowserENV["iphone"] = "iphone";
    BrowserENV["iphoneX"] = "iphoneX";
    BrowserENV["android"] = "android";
    BrowserENV["default"] = "default";
})(BrowserENV || (BrowserENV = {}));
/**
 * @whatItDoes 给组件、指令或服务注入当前运行环境的 token
 * 一般由框架自动检测，你可以在组件注入，并根据不同的环境，编写后续逻辑
 *
 * @howToUse
 * ```typescript
 * import { Inject } from '@angular/core';
 * import { UI_BROWSER_ENV, BrowserENV } from '@tanbo/ui-native';
 *
 * @Component({
 *   templateUrl: './demo.html'
 * })
 * export class DemoComponent {
 *   constructor(@Inject(UI_BROWSER_ENV) private env: BrowserENV) {
 *     console.log(env); // 当前环境
 *   }
 * }
 *
 * ```
 */
const UI_BROWSER_ENV = new InjectionToken('UI_BROWSER_ENV');
/**
 * @whatItDoes 给组件、指令或服务注入当前缩放系数的 token
 * @howToUse
 * ```typescript
 * import { Inject } from '@angular/core';
 * import { UI_SCREEN_SCALE } from '@tanbo/ui-native';
 *
 * @Component({
 *   templateUrl: './demo.html'
 * })
 * export class DemoComponent {
 *   constructor(@Inject(UI_SCREEN_SCALE) private env: number) {
 *     console.log(env); // 当前环境
 *   }
 * }
 *
 * ```
 */
const UI_SCREEN_SCALE = new InjectionToken('UI_SCREEN_SCALE');

let AppController = class AppController {
    constructor() {
        this.quitEvent = new Subject();
        this.resizeEvent = new Subject();
        this.transitionEvent = new Subject();
        this.onQuit = this.quitEvent.asObservable();
        this.onResize = this.resizeEvent.asObservable();
        this.transition = this.transitionEvent.asObservable();
    }
    quit() {
        this.quitEvent.next();
    }
    resize(env) {
        this.resizeEvent.next(env);
    }
    transitionStart() {
        this.transitionEvent.next(true);
    }
    transitionEnd() {
        this.transitionEvent.next(false);
    }
};
AppController = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], AppController);
function getDeviceType() {
    const u = navigator.userAgent;
    // android终端
    const isAndroid = u.indexOf('Android') > -1 || u.indexOf('Adr') > -1;
    // ios终端
    const isIOS = /\(i[^;]+;( U;)? CPU.+Mac OS X/.test(u);
    if (isAndroid) {
        return BrowserENV.android;
    }
    if (isIOS) {
        const screen = window.screen;
        if (screen.width === 375 && screen.height === 812 ||
            screen.height === 375 && screen.width === 812 ||
            screen.width === 414 && screen.height === 896 ||
            screen.height === 414 && screen.width === 896) {
            return BrowserENV.iphoneX;
        }
        return BrowserENV.iphone;
    }
    return BrowserENV.default;
}

const UI_DO_REFRESH_DISTANCE = new InjectionToken('UI_DO_REFRESH_DISTANCE');
let PullDownRefreshController = class PullDownRefreshController {
    constructor(doRefreshDistance) {
        this.doRefreshDistance = doRefreshDistance;
        this.dragSource = new Subject();
        this.dragEndSource = new Subject();
        this.refreshSource = new Subject();
        this.refreshEndSource = new Subject();
        this.distance = 0;
        this.timer = null;
        this.onStateChange = this.dragSource.asObservable();
        this.onDragEnd = this.dragEndSource.asObservable();
        this.onRefresh = this.refreshSource.asObservable();
        this.onRefreshEnd = this.refreshEndSource.asObservable();
    }
    refreshEnd() {
        this.refreshEndSource.next();
        if (this.distance === 0) {
            this.dragSource.next(0);
            return;
        }
        this.timer = setTimeout(() => {
            this.animationTo(0);
        }, 1000);
    }
    drag(n) {
        clearTimeout(this.timer);
        cancelAnimationFrame(this.animationId);
        this.distance = n;
        if (this.distance < 0) {
            this.distance = 0;
        }
        this.dragSource.next(this.distance);
    }
    dragEnd() {
        if (this.distance < this.doRefreshDistance) {
            this.animationTo(0);
        }
        else {
            this.refresh();
        }
        this.dragEndSource.next();
    }
    refresh() {
        cancelAnimationFrame(this.animationId);
        this.animationTo(this.doRefreshDistance, () => {
            this.refreshSource.next();
        });
    }
    animationTo(target, callback) {
        const oldDistance = this.distance;
        const interval = target - oldDistance;
        if (interval === 0) {
            return;
        }
        const max = 20;
        let step = 0;
        const animationFn = () => {
            step++;
            this.distance = Easing.Cubic.Out(step / max) * interval + oldDistance;
            this.dragSource.next(this.distance);
            if (step < max) {
                this.animationId = requestAnimationFrame(animationFn);
            }
            else {
                if (callback) {
                    callback();
                }
            }
        };
        this.animationId = requestAnimationFrame(animationFn);
    }
};
PullDownRefreshController.ctorParameters = () => [
    { type: Number, decorators: [{ type: Inject, args: [UI_DO_REFRESH_DISTANCE,] }] }
];
PullDownRefreshController = __decorate([
    Injectable(),
    __param(0, Inject(UI_DO_REFRESH_DISTANCE)),
    __metadata("design:paramtypes", [Number])
], PullDownRefreshController);

const UI_DO_LOAD_DISTANCE = new InjectionToken('UI_DO_LOAD_DISTANCE');
let PullUpLoadController = class PullUpLoadController {
    constructor() {
        this.infiniteSource = new Subject();
        this.loadedSource = new Subject();
        this.isPrevLoaded = true;
        this.onLoading = this.infiniteSource.asObservable();
        this.onLoaded = this.loadedSource.asObservable();
    }
    loading() {
        if (this.isPrevLoaded) {
            this.isPrevLoaded = false;
            this.infiniteSource.next();
        }
    }
    loaded() {
        this.isPrevLoaded = true;
        this.loadedSource.next();
    }
};
PullUpLoadController = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], PullUpLoadController);

let RefresherComponent = class RefresherComponent {
    constructor(pullDownRefreshController, scale, doRefreshDistance) {
        this.pullDownRefreshController = pullDownRefreshController;
        this.scale = scale;
        this.doRefreshDistance = doRefreshDistance;
        this.color = '#80848f';
        this.opacity = 1;
        this.progress = 0;
        this.isLoaded = false;
        this.subs = [];
    }
    get height() {
        return this.progress + 'px';
    }
    ngOnInit() {
        this.subs.push(this.pullDownRefreshController.onStateChange.subscribe(n => {
            cancelAnimationFrame(this.animationId);
            this.progress = n;
            if (this.isLoaded && n < this.doRefreshDistance) {
                this.outStage();
                if (n === 0) {
                    this.isLoaded = false;
                }
            }
            else {
                this.opacity = 1;
                this.inStage();
            }
        }));
        this.subs.push(this.pullDownRefreshController.onRefresh.subscribe(() => {
            this.animateYoYo();
        }));
        this.subs.push(this.pullDownRefreshController.onRefreshEnd.subscribe(() => {
            this.opacity = 0.5;
            this.isLoaded = true;
        }));
    }
    ngAfterViewInit() {
        this.canvasElement = this.canvas.nativeElement;
        this.containerWidth = this.canvasElement.offsetWidth;
        this.canvasElement.width = this.containerWidth * 2;
        this.context = this.canvasElement.getContext('2d');
    }
    ngOnDestroy() {
        this.subs.forEach(item => item.unsubscribe());
        cancelAnimationFrame(this.animationId);
    }
    inStage() {
        if (!this.canvasElement) {
            return;
        }
        this.canvasElement.height = this.progress * 2;
        this.context.translate(this.containerWidth, this.progress);
        this.drawProgress(this.progress);
    }
    outStage() {
        if (!this.canvasElement) {
            return;
        }
        this.canvasElement.height = this.progress * 2;
        const context = this.context;
        context.translate(this.containerWidth, this.progress);
        context.clearRect(-this.containerWidth, -this.progress, this.containerWidth * 2, this.progress * 2);
        context.fillStyle = this.color;
        const r = this.doRefreshDistance - this.progress;
        context.arc(0, 0, r * 10 * this.scale, 0, Math.PI * 2);
        context.fill();
        this.opacity = Easing.Cubic.Out(this.progress / this.doRefreshDistance) / 2;
    }
    animateYoYo() {
        let progress = this.progress;
        const max = this.doRefreshDistance;
        const yoyo = () => {
            progress += 2 * this.scale;
            const isSingle = Math.floor(progress / max) % 2 === 1;
            let result;
            if (isSingle) {
                result = max - progress % max;
            }
            else {
                result = progress % max;
            }
            this.drawProgress(Easing.Cubic.In(result / max) * max);
            if (this.isLoaded && result === this.doRefreshDistance) {
                return;
            }
            this.animationId = requestAnimationFrame(yoyo);
        };
        this.animationId = requestAnimationFrame(yoyo);
    }
    drawProgress(progress) {
        const context = this.context;
        context.clearRect(-this.containerWidth, -this.progress, this.containerWidth * 2, this.progress * 2);
        this.context.fillStyle = this.color;
        let r1 = this.progress / 4;
        let r2 = this.progress / 5;
        if (r1 > 12 * this.scale) {
            r1 = 12 * this.scale;
        }
        if (r2 > 8 * this.scale) {
            r2 = 8 * this.scale;
        }
        context.beginPath();
        context.arc(0, 0, r1, 0, Math.PI * 2);
        context.closePath();
        context.fill();
        const distance = (this.doRefreshDistance - progress) / this.doRefreshDistance * 40 * this.scale;
        context.beginPath();
        context.arc(-distance, 0, r2, 0, Math.PI * 2);
        context.closePath();
        context.fill();
        context.beginPath();
        context.arc(distance, 0, r2, 0, Math.PI * 2);
        context.closePath();
        context.fill();
    }
};
RefresherComponent.ctorParameters = () => [
    { type: PullDownRefreshController },
    { type: Number, decorators: [{ type: Inject, args: [UI_SCREEN_SCALE,] }] },
    { type: Number, decorators: [{ type: Inject, args: [UI_DO_REFRESH_DISTANCE,] }] }
];
__decorate([
    ViewChild('canvas', { static: true }),
    __metadata("design:type", ElementRef)
], RefresherComponent.prototype, "canvas", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], RefresherComponent.prototype, "color", void 0);
__decorate([
    HostBinding('style.height'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], RefresherComponent.prototype, "height", null);
RefresherComponent = __decorate([
    Component({
        selector: 'ui-refresher',
        template: "<canvas [ngStyle]=\"{opacity: opacity}\" class=\"ui-progress-bar\" #canvas></canvas>\n<div class=\"ui-refresher-content\" [ngClass]=\"{'ui-refresher-loaded': isLoaded}\">\n  <ng-content></ng-content>\n</div>"
    }),
    __param(1, Inject(UI_SCREEN_SCALE)),
    __param(2, Inject(UI_DO_REFRESH_DISTANCE)),
    __metadata("design:paramtypes", [PullDownRefreshController, Number, Number])
], RefresherComponent);

let ScrollComponent = class ScrollComponent {
    constructor(renderer, elementRef, doLoadDistance, pullUpLoadController, pullDownRefreshController) {
        this.renderer = renderer;
        this.elementRef = elementRef;
        this.doLoadDistance = doLoadDistance;
        this.pullUpLoadController = pullUpLoadController;
        this.pullDownRefreshController = pullDownRefreshController;
        this.distanceY = 0;
        this.unBindFnList = [];
        this._openRefresh = false;
        this._openInfinite = false;
        this.isBindRefresh = false;
        this.isBindInfinite = false;
    }
    // 是否开启下拉刷新
    set openRefresh(value) {
        this._openRefresh = value;
        if (value && !this.isBindRefresh) {
            this.bindingRefresher();
            this.isBindRefresh = true;
        }
    }
    get openRefresh() {
        return this._openRefresh;
    }
    // 是否开启下拉刷新
    set openInfinite(value) {
        this._openInfinite = value;
        if (value && !this.isBindInfinite) {
            this.bindingInfinite();
            this.isBindInfinite = true;
        }
    }
    get openInfinite() {
        return this._openInfinite;
    }
    ngOnInit() {
        this.sub = this.pullDownRefreshController.onStateChange.subscribe(n => {
            this.distanceY = n;
            this.paddingTop = `${n}px`;
        });
    }
    ngOnDestroy() {
        this.sub.unsubscribe();
        this.unBindFnList.forEach(item => item());
    }
    bindingInfinite() {
        const element = this.elementRef.nativeElement;
        this.unBindFnList.push(this.renderer.listen(element, 'scroll', () => {
            if (!this.openInfinite) {
                return;
            }
            // 计算最大滚动距离
            const maxScrollY = Math.max(element.scrollHeight, element.offsetHeight) - element.offsetHeight;
            // 如果当前滚动距离小于上拉刷新临界值，则记录相应值，并就广播相应事件
            if (maxScrollY - element.scrollTop < this.doLoadDistance) {
                this.pullUpLoadController.loading();
            }
        }));
    }
    bindingRefresher() {
        const element = this.elementRef.nativeElement;
        const fn = this.renderer.listen(element, 'touchstart', (ev) => {
            if (!this.openRefresh) {
                return;
            }
            const startPoint = ev.touches[0];
            const startX = startPoint.pageX;
            const startY = startPoint.pageY;
            let oldY = startY;
            let unBindTouchMoveFn;
            let unBindTouchEndFn;
            let unBindTouchCancelFn;
            const unBindFn = () => {
                unBindTouchMoveFn();
                unBindTouchEndFn();
                unBindTouchCancelFn();
                this.pullDownRefreshController.dragEnd();
            };
            let isFirstTouching = true;
            unBindTouchMoveFn = this.renderer.listen(element, 'touchmove', (moveEvent) => {
                const movePoint = moveEvent.touches[0];
                const moveX = movePoint.pageX;
                const moveY = movePoint.pageY;
                if (Math.abs(moveX - startX) > Math.abs(moveY - startY) && isFirstTouching) {
                    unBindFn();
                    return;
                }
                isFirstTouching = false;
                const n = moveY - oldY;
                oldY = moveY;
                if (n < 0) {
                    // 上拉
                    if (this.distanceY > 0) {
                        this.pullDownRefreshController.drag(n / 2 + this.distanceY);
                        return false;
                    }
                }
                else if (element.scrollTop <= 0) {
                    // 下拉
                    this.pullDownRefreshController.drag(n / 2 + this.distanceY);
                }
            });
            unBindTouchEndFn = this.renderer.listen(element, 'touchend', unBindFn);
            unBindTouchCancelFn = this.renderer.listen(element, 'touchcancel', unBindFn);
        });
        this.unBindFnList.push(fn);
    }
};
ScrollComponent.ctorParameters = () => [
    { type: Renderer2 },
    { type: ElementRef },
    { type: Number, decorators: [{ type: Inject, args: [UI_DO_LOAD_DISTANCE,] }] },
    { type: PullUpLoadController },
    { type: PullDownRefreshController }
];
__decorate([
    HostBinding('style.paddingTop'),
    __metadata("design:type", String)
], ScrollComponent.prototype, "paddingTop", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [Boolean])
], ScrollComponent.prototype, "openRefresh", null);
__decorate([
    Input(),
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [Boolean])
], ScrollComponent.prototype, "openInfinite", null);
ScrollComponent = __decorate([
    Component({
        selector: 'ui-scroll',
        template: "<ng-content></ng-content>\n"
    }),
    __param(2, Inject(UI_DO_LOAD_DISTANCE)),
    __metadata("design:paramtypes", [Renderer2,
        ElementRef, Number, PullUpLoadController,
        PullDownRefreshController])
], ScrollComponent);

let ScrollTabService = class ScrollTabService {
    constructor() {
        this.selectedEvent = new Subject();
        this.onSelected = this.selectedEvent.asObservable();
    }
    selected(c) {
        this.selectedEvent.next(c);
    }
};
ScrollTabService = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], ScrollTabService);

let ScrollTabButtonComponent = class ScrollTabButtonComponent {
    constructor(elementRef, scrollTabService) {
        this.elementRef = elementRef;
        this.scrollTabService = scrollTabService;
        this.selected = false;
    }
    click() {
        this.scrollTabService.selected(this);
    }
};
ScrollTabButtonComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: ScrollTabService }
];
__decorate([
    Input(),
    HostBinding('class.ui-selected'),
    __metadata("design:type", Boolean)
], ScrollTabButtonComponent.prototype, "selected", void 0);
__decorate([
    HostListener('click'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ScrollTabButtonComponent.prototype, "click", null);
ScrollTabButtonComponent = __decorate([
    Component({
        selector: 'ui-scroll-tab-button',
        template: "<ng-content></ng-content>"
    }),
    __metadata("design:paramtypes", [ElementRef,
        ScrollTabService])
], ScrollTabButtonComponent);

let ScrollTabComponent = class ScrollTabComponent {
    constructor(scrollTabService) {
        this.scrollTabService = scrollTabService;
        this.uiChange = new EventEmitter();
        this.left = 0;
        this.lineWidth = 0;
        this._index = 0;
        this.subs = [];
    }
    set index(value) {
        if (this.children) {
            this.autoUpdateStyle(value);
        }
        this._index = value;
    }
    get index() {
        return this._index;
    }
    ngAfterContentInit() {
        setTimeout(() => {
            this.initStyle();
        });
        this.subs.push(this.children.changes.pipe(delay(0)).subscribe(() => {
            this.initStyle();
        }));
        this.subs.push(this.scrollTabService.onSelected.subscribe(c => {
            this.children.forEach((item, i) => {
                if (item === c) {
                    this.uiChange.emit(i);
                }
            });
        }));
    }
    ngOnDestroy() {
        this.subs.forEach(item => item.unsubscribe());
    }
    initStyle() {
        if (this.children.length === 0) {
            return;
        }
        const btns = this.children.toArray().map((item, index) => {
            item.selected = index === this.index;
            return item.elementRef.nativeElement;
        });
        const element = this.wrap.nativeElement;
        this.lineWidth = btns[this.index].offsetWidth;
        this.left = btns[this.index].offsetLeft;
        element.scrollLeft = this.left + this.lineWidth / 2 - element.offsetWidth / 2;
    }
    autoUpdateStyle(index) {
        cancelAnimationFrame(this.animationId);
        if (index === this.index) {
            return;
        }
        const btns = this.children.toArray().map((item, ii) => {
            item.selected = ii === index;
            return item.elementRef.nativeElement;
        });
        if (!btns[this.index] || !btns[index]) {
            return;
        }
        const element = this.wrap.nativeElement;
        const oldLineWidth = btns[this.index].offsetWidth;
        const oldScrollLeft = element.scrollLeft;
        const oldLeft = this.left;
        const targetLineWidth = btns[index].offsetWidth;
        const targetScrollLeft = btns[index].offsetLeft + btns[index].offsetWidth / 2 - element.offsetWidth / 2;
        const targetLeft = btns[index].offsetLeft;
        let i = 0;
        const fn = () => {
            i++;
            const offset = Easing.Cubic.Out(i / 20);
            const scrollLeft = offset * (targetScrollLeft - oldScrollLeft);
            const width = offset * (targetLineWidth - oldLineWidth);
            const left = offset * (targetLeft - oldLeft);
            element.scrollLeft = oldScrollLeft + scrollLeft;
            this.lineWidth = oldLineWidth + width;
            this.left = oldLeft + left;
            if (i < 20) {
                this.animationId = requestAnimationFrame(fn);
            }
        };
        this.animationId = requestAnimationFrame(fn);
    }
};
ScrollTabComponent.ctorParameters = () => [
    { type: ScrollTabService }
];
__decorate([
    Output(),
    __metadata("design:type", Object)
], ScrollTabComponent.prototype, "uiChange", void 0);
__decorate([
    ContentChildren(ScrollTabButtonComponent),
    __metadata("design:type", QueryList)
], ScrollTabComponent.prototype, "children", void 0);
__decorate([
    ViewChild('wrap', { static: true }),
    __metadata("design:type", ElementRef)
], ScrollTabComponent.prototype, "wrap", void 0);
__decorate([
    Input(),
    __metadata("design:type", Number),
    __metadata("design:paramtypes", [Number])
], ScrollTabComponent.prototype, "index", null);
ScrollTabComponent = __decorate([
    Component({
        selector: 'ui-scroll-tab',
        template: "<div class=\"ui-scroll-tab\" #wrap>\n  <div class=\"ui-scroll-tab-wrap\">\n    <ng-content></ng-content>\n  </div>\n  <div class=\"ui-scroll-tab-line-wrap\" [ngStyle]=\"{width: lineWidth + 'px', transform: 'translateX('+left+'px)'}\">\n    <div class=\"ui-scroll-tab-line\"></div>\n  </div>\n</div>",
        providers: [
            ScrollTabService
        ]
    }),
    __metadata("design:paramtypes", [ScrollTabService])
], ScrollTabComponent);

let CollectionItemComponent = class CollectionItemComponent {
};
CollectionItemComponent = __decorate([
    Component({
        selector: 'ui-collection-item',
        template: "<ng-content></ng-content>",
        providers: [
            PullDownRefreshController,
            PullUpLoadController
        ]
    })
], CollectionItemComponent);

let CollectionComponent = class CollectionComponent {
    constructor(scale, elementRef) {
        this.scale = scale;
        this.elementRef = elementRef;
        /**
         * 拖动事件 $event 为当前拖动的进度
         */
        this.uiSliding = new EventEmitter();
        /**
         * 当完成整屏切换时，触发事件， $event 为当前拖动的进度
         */
        this.uiSlidingFinish = new EventEmitter();
        /**
         * 是否为垂直翻页，默认为 false，即为水平翻页
         */
        this.vertical = false;
        /**
         * 是否填满整个容器，默认不填充，如设置为 true，那么会填充满第一个有定位的父级元素
         */
        this.fill = true;
        this.transform = '';
        this.max = 0;
        // 记录已拖动的距离
        this.distance = 0;
        this.slidingEventSource = new Subject();
        this.subs = [];
        this._index = 0;
        this.oldDistance = 0;
        this.slidingEvent$ = this.slidingEventSource.asObservable();
    }
    /**
     * 显示第几屏的索引
     */
    set index(value) {
        this._index = value;
        if (this.items) {
            cancelAnimationFrame(this.animationId);
            this.setStyle();
        }
    }
    get index() {
        return this._index;
    }
    // 通过子级的多少，计算自身的盒子大小
    get width() {
        return this.vertical ? 'auto' : this.childrenCount * 100 + '%';
    }
    get height() {
        return this.vertical ? this.childrenCount * 100 + '%' : 'auto';
    }
    get childrenCount() {
        if (this.items) {
            return this.items.length;
        }
        return 0;
    }
    get stepDistance() {
        return this.vertical ? this.element.offsetHeight : this.element.offsetWidth;
    }
    get min() {
        return -this.stepDistance * (this.childrenCount - 1);
    }
    ngAfterContentInit() {
        this.element = this.elementRef.nativeElement;
        this.subs.push(this.slidingEvent$.pipe(distinctUntilChanged()).subscribe((n) => {
            this.uiSliding.emit(n);
        }));
        this.setStyle();
    }
    ngOnDestroy() {
        this.subs.forEach(item => item.unsubscribe());
    }
    drag(event) {
        if (event.first) {
            this.oldDistance = this.distance;
        }
        if ((this.vertical && event.firstDirection !== 'up' && event.firstDirection !== 'down') ||
            (!this.vertical && event.firstDirection !== 'left' && event.firstDirection !== 'right')) {
            this.touchAction = undefined;
            event.stop();
            return;
        }
        if (event.type === 'touchmove') {
            this.touchAction = this.vertical ? 'pan-y' : 'pan-x';
            let distance = this.oldDistance + (this.vertical ? event.distanceY : event.distanceX);
            if (distance > this.max) {
                distance /= 3;
            }
            else if (distance < this.min) {
                distance = this.min + (distance - this.min) / 3;
            }
            this.distance = distance;
            this.transform = `translate${this.vertical ? 'Y' : 'X'}(${distance}px)`;
            // 发送事件，并传出当前已滑动到第几屏的进度
            this.slidingEventSource.next(distance / this.stepDistance * -1);
            // event.srcEvent.preventDefault();
        }
        else if (event.type === 'touchend') {
            this.touchAction = undefined;
            if (this.distance > this.max) {
                this.autoUpdateStyle(this.max);
                return;
            }
            else if (this.distance < this.min) {
                this.autoUpdateStyle(this.min);
                return;
            }
            let translateDistance;
            const targetIndex = Math.ceil(this.distance / this.stepDistance);
            const offset = Math.abs(this.distance % this.stepDistance);
            if (event.durationTime < 200 && Math.abs(this.vertical ? event.distanceY : event.distanceX) > 50 * this.scale) {
                // swipe
                if (this.oldDistance < this.distance) {
                    translateDistance = targetIndex * this.stepDistance;
                }
                else {
                    translateDistance = targetIndex * this.stepDistance - this.stepDistance;
                }
            }
            else {
                if (offset < (this.stepDistance / 2)) {
                    translateDistance = targetIndex * this.stepDistance;
                }
                else {
                    translateDistance = targetIndex * this.stepDistance - this.stepDistance;
                }
            }
            this.autoUpdateStyle(translateDistance);
        }
    }
    setStyle() {
        this.distance = this.index * -1 * this.stepDistance;
        this.transform = `translate${this.vertical ? 'Y' : 'X'}(${this.distance}px)`;
    }
    autoUpdateStyle(translateDistance) {
        const max = 20;
        let step = 0;
        const distance = translateDistance - this.distance;
        if (distance === 0) {
            return;
        }
        const rawDistance = this.distance;
        const moveToTarget = () => {
            step++;
            const translate = rawDistance + Easing.Cubic.Out(step / max) * distance;
            this.distance = translate;
            this.transform = `translate${this.vertical ? 'Y' : 'X'}(${translate}px)`;
            this.slidingEventSource.next(translate / this.stepDistance * -1);
            if (step < max) {
                this.animationId = requestAnimationFrame(moveToTarget);
            }
            else {
                // 发送事件，并传出当前滑动到了第几屏
                this.uiSlidingFinish.emit(this.distance / this.stepDistance * -1);
            }
        };
        this.animationId = requestAnimationFrame(moveToTarget);
    }
};
CollectionComponent.ctorParameters = () => [
    { type: Number, decorators: [{ type: Inject, args: [UI_SCREEN_SCALE,] }] },
    { type: ElementRef }
];
__decorate([
    ContentChildren(CollectionItemComponent),
    __metadata("design:type", QueryList)
], CollectionComponent.prototype, "items", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], CollectionComponent.prototype, "uiSliding", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], CollectionComponent.prototype, "uiSlidingFinish", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], CollectionComponent.prototype, "vertical", void 0);
__decorate([
    Input(),
    HostBinding('class.ui-fill'),
    __metadata("design:type", Boolean)
], CollectionComponent.prototype, "fill", void 0);
__decorate([
    Input(),
    __metadata("design:type", Number),
    __metadata("design:paramtypes", [Number])
], CollectionComponent.prototype, "index", null);
CollectionComponent = __decorate([
    Component({
        selector: 'ui-collection',
        template: "<div class=\"ui-collection\"\n     (uiPan)=\"drag($event)\"\n     [ngClass]=\"{'ui-vertical': vertical}\"\n     [ngStyle]=\"{width: width, height: height, transform: transform, webkitTransform: transform, touchAction: touchAction}\">\n  <ng-content></ng-content>\n</div>"
    }),
    __param(0, Inject(UI_SCREEN_SCALE)),
    __metadata("design:paramtypes", [Number, ElementRef])
], CollectionComponent);

let FlexComponent = class FlexComponent {
};
FlexComponent = __decorate([
    Component({
        selector: 'ui-flex',
        template: "<ng-content></ng-content>"
    })
], FlexComponent);

let FlexItemComponent = class FlexItemComponent {
};
FlexItemComponent = __decorate([
    Component({
        selector: 'ui-flex-item',
        template: "<ng-content></ng-content>"
    })
], FlexItemComponent);

/**
 * @whatItDoes 给 `ui-select`、`ui-input[type=date]` 组件的下拉箭头的 html 元素 class 的名字的 token
 *
 * 你可以通过 provide 定义自己的下拉箭头 class
 *
 * @howToUse
 * ```typescript
 * import { NgModule } from '@angular/core';
 * import { UI_SELECT_ARROW_CLASSNAME } from '@tanbo/ui-native';
 *
 * @NgModule({
 *   ...
 *   providers: [{
 *     provide: UI_SELECT_ARROW_CLASSNAME,
 *     useValue: 'custom-classname'
 *   }]
 * })
 * export class AppModule {
 * }
 * ```
 */
const UI_SELECT_ARROW_CLASSNAME = new InjectionToken('UI_SELECT_ARROW_CLASSNAME');
/**
 * 把表单元素的输入属性转成 boolean 类型
 * '' => true
 * 0 => false
 * null => false
 * undefined => false
 *
 * @param value
 * @returns boolean
 */
function inputAttrToBoolean(value) {
    return value === '' || !!value;
}

let PickerService = class PickerService {
    constructor() {
        this.showEvent = new Subject();
        this.scrollEvent = new Subject();
        this.onShow = this.showEvent.asObservable();
        this.onScroll = this.scrollEvent.asObservable();
    }
    show() {
        this.showEvent.next();
    }
    scroll(b) {
        this.scrollEvent.next(b);
    }
};
PickerService = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], PickerService);

let ButtonComponent = class ButtonComponent {
    constructor(renderer, elementRef) {
        this.renderer = renderer;
        this.elementRef = elementRef;
        this._loading = false;
    }
    set loading(v) {
        this._loading = inputAttrToBoolean(v);
        this.updateState();
    }
    get loading() {
        return this._loading;
    }
    updateState() {
        this.renderer.setProperty(this.elementRef.nativeElement, 'disabled', this._loading);
    }
};
ButtonComponent.ctorParameters = () => [
    { type: Renderer2 },
    { type: ElementRef }
];
__decorate([
    Input(),
    HostBinding('class.ui-loading'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], ButtonComponent.prototype, "loading", null);
ButtonComponent = __decorate([
    Component({
        /* tslint:disable */
        selector: '[ui-button]',
        /* tslint:enable */
        template: "<span class=\"ui-loading-bar\"></span>\n<ng-content></ng-content>"
    }),
    __metadata("design:paramtypes", [Renderer2,
        ElementRef])
], ButtonComponent);

var CheckboxComponent_1;
let CheckboxComponent = CheckboxComponent_1 = class CheckboxComponent {
    constructor() {
        this.uiChange = new EventEmitter();
        this.checkedIcon = 'ui-icon-checkbox-checked';
        this.uncheckedIcon = 'ui-icon-checkbox-unchecked';
        this.text = '';
        this.value = '';
        this._disabled = false;
        this._readonly = false;
        this._checked = false;
    }
    set disabled(isDisabled) {
        this._disabled = inputAttrToBoolean(isDisabled);
    }
    get disabled() {
        return this._disabled;
    }
    set readonly(isReadonly) {
        this._readonly = inputAttrToBoolean(isReadonly);
    }
    get readonly() {
        return this._readonly;
    }
    set checked(isChecked) {
        this._checked = inputAttrToBoolean(isChecked);
    }
    get checked() {
        return this._checked;
    }
    click() {
        if (this.disabled || this.readonly) {
            return;
        }
        this.checked = !this.checked;
        if (this.onChange) {
            this.onChange(this.checked);
        }
        if (this.onTouched) {
            this.onTouched();
        }
        this.uiChange.emit(this.checked);
    }
    writeValue(value) {
        this.checked = !!value;
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
};
__decorate([
    Output(),
    __metadata("design:type", Object)
], CheckboxComponent.prototype, "uiChange", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], CheckboxComponent.prototype, "checkedIcon", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], CheckboxComponent.prototype, "uncheckedIcon", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], CheckboxComponent.prototype, "name", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], CheckboxComponent.prototype, "text", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], CheckboxComponent.prototype, "value", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], CheckboxComponent.prototype, "forId", void 0);
__decorate([
    Input(),
    HostBinding('class.ui-disabled'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], CheckboxComponent.prototype, "disabled", null);
__decorate([
    Input(),
    HostBinding('class.ui-readonly'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], CheckboxComponent.prototype, "readonly", null);
__decorate([
    Input(),
    HostBinding('class.ui-checked'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], CheckboxComponent.prototype, "checked", null);
CheckboxComponent = CheckboxComponent_1 = __decorate([
    Component({
        selector: 'ui-input[type=checkbox]',
        template: "<div class=\"ui-checkbox\">\n  <span *ngIf=\"checked\" [ngClass]=\"checkedIcon\">{{ text }}</span>\n  <span *ngIf=\"!checked\" [ngClass]=\"uncheckedIcon\">{{ text }}</span>\n</div>\n<input class=\"ui-input-checkbox\"\n       type=\"button\"\n       [value]=\"value\"\n       (click)=\"click()\"\n       [attr.id]=\"forId\"\n       [attr.name]=\"name\"\n       [disabled]=\"disabled\"\n       [checked]=\"checked\"\n       [readonly]=\"readonly\">\n",
        providers: [{
                provide: NG_VALUE_ACCESSOR,
                useExisting: CheckboxComponent_1,
                multi: true
            }]
    })
], CheckboxComponent);

class UIDate {
    constructor(d) {
        this.d = d;
        if (typeof d === 'number') {
            this.date = new Date(d);
        }
        else if (typeof d === 'string') {
            this.date = this.stringToDate(d);
        }
        else if (d instanceof Date) {
            this.date = d;
        }
        else {
            this.date = new Date();
        }
    }
    get timestamp() {
        return this.date.getTime();
    }
    set year(v) {
        this.date.setFullYear(v);
    }
    get year() {
        return this.date.getFullYear();
    }
    set month(v) {
        const date = new Date(this.timestamp);
        date.setMonth(v + 1, 0);
        const day = date.getDate();
        if (day < this.day) {
            this.date.setMonth(v, day);
            return;
        }
        this.date.setMonth(v);
    }
    get month() {
        return this.date.getMonth();
    }
    set day(v) {
        this.date.setDate(v);
    }
    get day() {
        return this.date.getDate();
    }
    set hours(v) {
        this.date.setHours(v);
    }
    get hours() {
        return this.date.getHours();
    }
    set minutes(v) {
        this.date.setMinutes(v);
    }
    get minutes() {
        return this.date.getMinutes();
    }
    set seconds(v) {
        this.date.setSeconds(v);
    }
    get seconds() {
        return this.date.getSeconds();
    }
    toStringByFormatString(s) {
        const date = new Date();
        date.setTime(this.timestamp);
        const year = date.getFullYear();
        const month = date.getMonth() + 1;
        const day = date.getDate();
        const hours = date.getHours();
        const minutes = date.getMinutes();
        const seconds = date.getSeconds();
        return s.replace(/[yMdhms]+/g, (str) => {
            switch (str) {
                case 'yy':
                    return year ? toDouble(year % 100) : '';
                case 'yyyy':
                    return year + '';
                case 'M':
                    return month + '';
                case 'MM':
                    return toDouble(month);
                case 'd':
                    return day + '';
                case 'dd':
                    return toDouble(day);
                case 'h':
                    return hours + '';
                case 'hh':
                    return toDouble(hours);
                case 'm':
                    return minutes + '';
                case 'mm':
                    return toDouble(minutes);
                case 's':
                    return seconds + '';
                case 'ss':
                    return toDouble(seconds);
                default:
                    return str;
            }
        });
    }
    stringToDate(s) {
        let dateArr = (s.match(/\d+/g) || []).map(item => {
            return +item;
        });
        const arr = [0, 0, 1, 0, 0, 0];
        dateArr = dateArr.concat(arr.slice(dateArr.length));
        const date = new Date();
        date.setFullYear(dateArr[0]);
        date.setMonth(dateArr[1] - 1);
        date.setDate(dateArr[2]);
        date.setHours(dateArr[3]);
        date.setMinutes(dateArr[4]);
        date.setSeconds(dateArr[5]);
        date.setMilliseconds(0);
        return date;
    }
}
/**
 * 给0-9的数字补零，转为00、01……
 * @param n 0-9的数字
 */
function toDouble(n) {
    return n > 9 ? n + '' : '0' + n;
}

var DateComponent_1;
let DateComponent = DateComponent_1 = class DateComponent {
    constructor(arrowIcon, pickerService) {
        this.pickerService = pickerService;
        this.uiChange = new EventEmitter();
        this.title = '';
        this.format = 'yyyy-MM-dd';
        this.displayFormat = '';
        this.placeholder = '';
        this.arrowIconClassName = '';
        this.focus = false;
        this.years = [];
        this.months = [];
        this.days = [];
        this.hours = [];
        this.minutes = [];
        this.seconds = [];
        this.displayValue = '';
        this._disabled = false;
        this._readonly = false;
        this.timer = null;
        this.isScrolling = false;
        this.arrowIconClassName = arrowIcon;
        this.currentDate = new UIDate();
        let d = new Date(this.currentDate.timestamp);
        d.setFullYear(d.getFullYear() + 20);
        this._maxDate = new UIDate(d);
        d = new Date(this.currentDate.timestamp);
        d.setFullYear(d.getFullYear() - 80);
        this._minDate = new UIDate(d);
    }
    set value(value) {
        this._value = value;
        if (value === '' || value === null || value === undefined) {
            this.displayValue = '';
        }
        else {
            this.currentDate = new UIDate(value);
            this.displayValue = this.currentDate.toStringByFormatString(this.displayFormat || this.format);
            this.initYears();
        }
    }
    get value() {
        return this._value;
    }
    set maxDate(value) {
        this._maxDate = new UIDate(value);
        this.initYears();
    }
    set minDate(value) {
        this._minDate = new UIDate(value);
        this.initYears();
    }
    set disabled(isDisabled) {
        this._disabled = inputAttrToBoolean(isDisabled);
    }
    get disabled() {
        return this._disabled;
    }
    set readonly(isReadonly) {
        this._readonly = inputAttrToBoolean(isReadonly);
    }
    get readonly() {
        return this._readonly;
    }
    ngOnInit() {
        this.sub = this.pickerService.onScroll.subscribe(b => {
            this.isScrolling = b;
        });
        this.initYears();
    }
    ngOnDestroy() {
        this.sub.unsubscribe();
    }
    initYears() {
        if (/yy|yyyy/.test(this.format)) {
            this.years.length = 0;
            for (let i = this._minDate.year; i <= this._maxDate.year; i++) {
                this.years.push({
                    value: i,
                    text: i + '年'
                });
            }
        }
        this.initMonths();
    }
    initMonths() {
        if (/M|MM/.test(this.format)) {
            this.months.length = 0;
            let startMonth = 0;
            let endMonth = 11;
            if (this.currentDate.year <= this._minDate.year) {
                startMonth = this._minDate.month;
                if (this.currentDate.month < startMonth) {
                    this.currentDate.month = startMonth;
                }
            }
            if (this.currentDate.year >= this._maxDate.year) {
                endMonth = this._maxDate.month;
                if (this.currentDate.month > endMonth) {
                    this.currentDate.month = endMonth;
                }
            }
            for (; startMonth <= endMonth; startMonth++) {
                this.months.push({
                    value: startMonth,
                    text: startMonth + 1 + '月'
                });
            }
            this.months = this.months.slice(); // 数组复制触发视图变更
        }
        this.initDays();
    }
    initDays() {
        if (/d|dd/.test(this.format)) {
            this.days.length = 0;
            // 获取当前选中月总共有多少天
            const date = new Date();
            // 设置为当月1日，防止溢出
            date.setDate(1);
            date.setFullYear(this.currentDate.year);
            date.setMonth(this.currentDate.month + 1);
            date.setDate(0);
            let startDay = 1;
            let endDay = date.getDate();
            if (this.currentDate.year <= this._minDate.year) {
                if (this.currentDate.month <= this._minDate.month) {
                    startDay = this._minDate.day;
                }
            }
            if (this.currentDate.year >= this._maxDate.year) {
                if (this.currentDate.month >= this._maxDate.month) {
                    endDay = this._maxDate.day;
                }
            }
            for (; startDay <= endDay; startDay++) {
                this.days.push({
                    text: startDay + '日',
                    value: startDay
                });
            }
            this.days = this.days.slice(); // 数组复制触发视图变更
        }
    }
    show() {
        clearTimeout(this.timer);
        if (this.disabled || this.readonly) {
            return;
        }
        this.focus = true;
        this.timer = setTimeout(() => {
            this.pickerService.show();
        });
    }
    selected() {
        if (!this.focus || this.isScrolling) {
            return;
        }
        let value;
        if (this.format) {
            value = this.currentDate.toStringByFormatString(this.format);
        }
        else {
            value = this.currentDate.timestamp;
        }
        this.displayValue = this.currentDate.toStringByFormatString(this.displayFormat || this.format);
        if (this.onChange) {
            this.onChange(value);
        }
        if (this.onTouched) {
            this.onTouched();
        }
        this._value = value;
        this.uiChange.emit(value);
        this.hide();
    }
    cellSelected(cell, type) {
        switch (type) {
            case 'year':
                this.currentDate.year = +cell.value;
                this.initMonths();
                break;
            case 'month':
                this.currentDate.month = +cell.value;
                this.initDays();
                break;
            case 'day':
                this.currentDate.day = +cell.value;
                break;
            case 'hours':
                this.currentDate.hours = +cell.value;
                break;
            case 'minutes':
                this.currentDate.minutes = +cell.value;
                break;
            case 'seconds':
                this.currentDate.seconds = +cell.value;
                break;
        }
    }
    hide() {
        clearTimeout(this.timer);
        this.focus = false;
    }
    writeValue(value) {
        this.value = value;
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
};
DateComponent.ctorParameters = () => [
    { type: String, decorators: [{ type: Inject, args: [UI_SELECT_ARROW_CLASSNAME,] }] },
    { type: PickerService }
];
__decorate([
    Output(),
    __metadata("design:type", Object)
], DateComponent.prototype, "uiChange", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], DateComponent.prototype, "title", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], DateComponent.prototype, "name", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], DateComponent.prototype, "forId", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], DateComponent.prototype, "format", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], DateComponent.prototype, "displayFormat", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], DateComponent.prototype, "placeholder", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], DateComponent.prototype, "arrowIconClassName", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], DateComponent.prototype, "value", null);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], DateComponent.prototype, "maxDate", null);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], DateComponent.prototype, "minDate", null);
__decorate([
    Input(),
    HostBinding('class.ui-disabled'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], DateComponent.prototype, "disabled", null);
__decorate([
    Input(),
    HostBinding('class.ui-readonly'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], DateComponent.prototype, "readonly", null);
DateComponent = DateComponent_1 = __decorate([
    Component({
        selector: 'ui-input[type=date]',
        template: "<div class=\"ui-date-fixed\" (click)=\"show()\">\n  <div class=\"ui-date-input-wrap\">\n    <input class=\"ui-date-input\" [disabled]=\"disabled\"\n           [attr.name]=\"name\"\n           [attr.id]=\"forId\"\n           [placeholder]=\"placeholder\"\n           readonly\n           type=\"text\"\n           [value]=\"displayValue\">\n    <div class=\"ui-date-input-mask\"></div>\n  </div>\n  <div class=\"ui-date-arrow\" *ngIf=\"arrowIconClassName\">\n    <span [ngClass]=\"arrowIconClassName\"></span>\n  </div>\n</div>\n<ui-action-sheet [show]=\"focus\" (click)=\"hide()\">\n  <div class=\"ui-picker\" uiStopPropagation>\n    <ui-flex class=\"ui-picker-header\">\n      <button class=\"ui-picker-action\" type=\"button\" (click)=\"hide()\">\u53D6\u6D88</button>\n      <ui-flex-item class=\"ui-picker-title\">{{ title }}</ui-flex-item>\n      <button class=\"ui-picker-action\" type=\"button\" (click)=\"selected()\">\u786E\u8BA4</button>\n    </ui-flex>\n\n    <!-- *ngIf=\"focus\" \u89E3\u51B3dom\u9020\u6210\u7684\u5361\u987F\u95EE\u9898 -->\n    <div *ngIf=\"focus\" class=\"ui-picker-selector\">\n      <div class=\"ui-picker-list\">\n        <ui-picker-column *ngIf=\"years.length\" (uiSelected)=\"cellSelected($event, 'year')\"\n                          [cells]=\"years\" [value]=\"{value: currentDate.year}\"></ui-picker-column>\n        <ui-picker-column *ngIf=\"months.length\" (uiSelected)=\"cellSelected($event, 'month')\"\n                          [cells]=\"months\" [value]=\"{value: currentDate.month}\"></ui-picker-column>\n        <ui-picker-column *ngIf=\"days.length\" (uiSelected)=\"cellSelected($event, 'day')\"\n                          [cells]=\"days\" [value]=\"{value: currentDate.day}\"></ui-picker-column>\n        <ui-picker-column *ngIf=\"hours.length\" (uiSelected)=\"cellSelected($event, 'hours')\"\n                          [cells]=\"hours\" [value]=\"{value: currentDate.hours}\"></ui-picker-column>\n        <ui-picker-column *ngIf=\"minutes.length\" (uiSelected)=\"cellSelected($event, 'minutes')\"\n                          [cells]=\"minutes\" [value]=\"{value: currentDate.minutes}\"></ui-picker-column>\n        <ui-picker-column *ngIf=\"seconds.length\" (uiSelected)=\"cellSelected($event, 'seconds')\"\n                          [cells]=\"seconds\" [value]=\"{value: currentDate.seconds}\"></ui-picker-column>\n      </div>\n    </div>\n  </div>\n</ui-action-sheet>\n",
        providers: [{
                provide: NG_VALUE_ACCESSOR,
                useExisting: DateComponent_1,
                multi: true
            }, PickerService]
    }),
    __param(0, Inject(UI_SELECT_ARROW_CLASSNAME)),
    __metadata("design:paramtypes", [String, PickerService])
], DateComponent);

let SelectService = class SelectService {
    constructor() {
        this.checkedSource = new Subject();
        this.onChecked = this.checkedSource.asObservable();
    }
    checked(option) {
        this.checkedSource.next(option);
    }
};
SelectService = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], SelectService);

let OptionComponent = class OptionComponent {
    constructor(elementRef, selectService) {
        this.elementRef = elementRef;
        this.selectService = selectService;
        this.uiChecked = new EventEmitter();
        this.value = '';
        this._disabled = false;
        this._selected = false;
    }
    set disabled(isDisabled) {
        this._disabled = inputAttrToBoolean(isDisabled);
    }
    get disabled() {
        return this._disabled;
    }
    set selected(isSelected) {
        this._selected = inputAttrToBoolean(isSelected);
    }
    get selected() {
        return this._selected;
    }
    click() {
        if (!this.disabled) {
            this.selectService.checked(this);
            this.uiChecked.emit(this);
        }
    }
    ngAfterViewInit() {
        this.nativeElement = this.elementRef.nativeElement;
    }
};
OptionComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: SelectService }
];
__decorate([
    Output(),
    __metadata("design:type", Object)
], OptionComponent.prototype, "uiChecked", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], OptionComponent.prototype, "value", void 0);
__decorate([
    Input(),
    HostBinding('class.ui-disabled'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], OptionComponent.prototype, "disabled", null);
__decorate([
    Input(),
    HostBinding('class.ui-selected'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], OptionComponent.prototype, "selected", null);
__decorate([
    HostListener('click'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], OptionComponent.prototype, "click", null);
OptionComponent = __decorate([
    Component({
        selector: 'ui-option',
        template: "<ng-content></ng-content>"
    }),
    __metadata("design:paramtypes", [ElementRef,
        SelectService])
], OptionComponent);

var PickerComponent_1;
let PickerComponent = PickerComponent_1 = class PickerComponent {
    constructor(arrowIcon, pickerService) {
        this.pickerService = pickerService;
        this.uiChange = new EventEmitter();
        this.arrowIconClassName = '';
        this.columnSize = 3;
        this.placeholder = '';
        this.value = [];
        this.title = '';
        this.focus = false;
        this.list = [];
        this._value = [];
        this._disabled = false;
        this._readonly = false;
        this.timer = null;
        this.isScrolling = false;
        this.arrowIconClassName = arrowIcon;
    }
    set disabled(isDisabled) {
        this._disabled = inputAttrToBoolean(isDisabled);
    }
    get disabled() {
        return this._disabled;
    }
    set readonly(isReadonly) {
        this._readonly = inputAttrToBoolean(isReadonly);
    }
    get readonly() {
        return this._readonly;
    }
    set data(list) {
        this._data = list;
        if (this.value.length >= this.columnSize) {
            this.value.forEach(item => this._value.push(item));
            this.makeList(0, list);
        }
        else {
            this.list = [list];
            if (list.length) {
                this.cellSelected(list[0], 0);
            }
        }
    }
    get data() {
        return this._data;
    }
    ngOnInit() {
        this.sub = this.pickerService.onScroll.subscribe(b => {
            this.isScrolling = b;
        });
    }
    ngOnDestroy() {
        this.sub.unsubscribe();
    }
    click() {
        clearTimeout(this.timer);
        if (this.disabled || this.readonly) {
            return;
        }
        this.focus = true;
        this.timer = setTimeout(() => {
            this.pickerService.show();
        });
    }
    hide() {
        clearTimeout(this.timer);
        this.focus = false;
    }
    cellSelected(cell, index) {
        this._value[index] = cell;
        this._value.length = index + 1;
        let children = cell.children;
        let i = index;
        while (children) {
            this._value[++i] = children[0];
            children = children[0].children;
        }
        this.makeList(index + 1, cell.children);
    }
    selected() {
        if (!this.focus || this.isScrolling) {
            return;
        }
        if (this.onChange) {
            this.onChange(this._value);
        }
        if (this.onTouched) {
            this.onTouched(this._value);
        }
        this.value = this._value;
        this.uiChange.emit(this._value);
        this.hide();
    }
    writeValue(value) {
        this.value = value;
        if (value && Array.isArray(value)) {
            this.value.forEach(item => this._value.push(item));
            this.makeList(0, this._data);
        }
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
    makeList(startIndex, list) {
        this.list.length = startIndex;
        if (list) {
            const fn = (parentList, child, index) => {
                this.list.push(child);
                parentList.forEach(item => {
                    if (this._value[index] && item.value === this._value[index].value && item.children) {
                        const nextChild = [];
                        fn(item.children, nextChild, index + 1);
                    }
                    child.push(item);
                });
            };
            fn(list, [], startIndex);
        }
        this.completionListLength();
    }
    completionListLength() {
        for (let i = this.list.length; i < this.columnSize; i++) {
            this.list.push([]);
        }
    }
};
PickerComponent.ctorParameters = () => [
    { type: String, decorators: [{ type: Inject, args: [UI_SELECT_ARROW_CLASSNAME,] }] },
    { type: PickerService }
];
__decorate([
    Output(),
    __metadata("design:type", Object)
], PickerComponent.prototype, "uiChange", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], PickerComponent.prototype, "name", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], PickerComponent.prototype, "arrowIconClassName", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], PickerComponent.prototype, "columnSize", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], PickerComponent.prototype, "placeholder", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], PickerComponent.prototype, "forId", void 0);
__decorate([
    Input(),
    __metadata("design:type", Array)
], PickerComponent.prototype, "value", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], PickerComponent.prototype, "title", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], PickerComponent.prototype, "disabled", null);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], PickerComponent.prototype, "readonly", null);
__decorate([
    Input(),
    __metadata("design:type", Array),
    __metadata("design:paramtypes", [Array])
], PickerComponent.prototype, "data", null);
__decorate([
    HostListener('click'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PickerComponent.prototype, "click", null);
PickerComponent = PickerComponent_1 = __decorate([
    Component({
        selector: 'ui-picker',
        template: "<div class=\"ui-picker-fixed\">\n  <div class=\"ui-picker-input-wrap\">\n    <div class=\"ui-picker-content\" [ngClass]=\"{'ui-show': value && value.length}\">\n      <ng-content></ng-content>\n    </div>\n    <input [ngClass]=\"{'ui-show': !value || !value.length}\"\n           type=\"text\" class=\"ui-picker-input\"\n           [attr.name]=\"name\" [attr.id]=\"forId\"\n           [disabled]=\"disabled\" readonly\n           [placeholder]=\"placeholder\">\n    <div class=\"ui-picker-input-mask\"></div>\n  </div>\n  <div class=\"ui-picker-arrow\" *ngIf=\"arrowIconClassName\">\n    <span [ngClass]=\"arrowIconClassName\"></span>\n  </div>\n</div>\n<ui-action-sheet [show]=\"focus\" (click)=\"hide()\" uiStopPropagation>\n  <div class=\"ui-picker\" uiStopPropagation>\n    <ui-flex class=\"ui-picker-header\">\n      <button class=\"ui-picker-action\" type=\"button\" (click)=\"hide()\">\u53D6\u6D88</button>\n      <ui-flex-item class=\"ui-picker-title\">{{ title }}</ui-flex-item>\n      <button class=\"ui-picker-action\" type=\"button\" (click)=\"selected()\">\u786E\u8BA4</button>\n    </ui-flex>\n    <div class=\"ui-picker-selector\" *ngIf=\"focus\">\n      <div class=\"ui-picker-list\">\n        <!--\n        <ui-picker-column *ngFor=\"let column of list; let i = index\"\n                          (uiSelected)=\"cellSelected($event, i)\"\n                          [cells]=\"column\" [value]=\"value[i]\"></ui-picker-column>\n        -->\n        <!-- ngFor \u4F1A\u5BFC\u81F4\u56DE\u586B\u9009\u4E2Dbug -->\n        <ui-picker-column *ngIf=\"list.length && list[0]\" (uiSelected)=\"cellSelected($event, 0)\"\n                          [cells]=\"list[0]\" [value]=\"value[0] || list[0][0]\"></ui-picker-column>\n\n        <ui-picker-column *ngIf=\"list.length && list[1]\" (uiSelected)=\"cellSelected($event, 1)\"\n                          [cells]=\"list[1]\" [value]=\"value[1] || list[1][0]\"></ui-picker-column>\n\n        <ui-picker-column *ngIf=\"list.length && list[2]\" (uiSelected)=\"cellSelected($event, 2)\"\n                          [cells]=\"list[2]\" [value]=\"value[2] || list[2][0]\"></ui-picker-column>\n\n        <ui-picker-column *ngIf=\"list.length && list[3]\" (uiSelected)=\"cellSelected($event, 3)\"\n                          [cells]=\"list[3]\" [value]=\"value[3] || list[3][0]\"></ui-picker-column>\n\n        <ui-picker-column *ngIf=\"list.length && list[4]\" (uiSelected)=\"cellSelected($event, 4)\"\n                          [cells]=\"list[4]\" [value]=\"value[4] || list[4][0]\"></ui-picker-column>\n\n        <ui-picker-column *ngIf=\"list.length && list[5]\" (uiSelected)=\"cellSelected($event, 5)\"\n                          [cells]=\"list[5]\" [value]=\"value[5] || list[5][0]\"></ui-picker-column>\n\n        <ui-picker-column *ngIf=\"list.length && list[6]\" (uiSelected)=\"cellSelected($event, 6)\"\n                          [cells]=\"list[6]\" [value]=\"value[6] || list[6][0]\"></ui-picker-column>\n      </div>\n    </div>\n  </div>\n</ui-action-sheet>\n",
        providers: [{
                provide: NG_VALUE_ACCESSOR,
                useExisting: PickerComponent_1,
                multi: true
            },
            PickerService
        ]
    }),
    __param(0, Inject(UI_SELECT_ARROW_CLASSNAME)),
    __metadata("design:paramtypes", [String, PickerService])
], PickerComponent);

BScroll.use(Wheel);
let PickerColumnComponent = class PickerColumnComponent {
    constructor(elementRef, scale, pickerService) {
        this.elementRef = elementRef;
        this.scale = scale;
        this.pickerService = pickerService;
        this._cells = [];
        this.uiSelected = new EventEmitter();
        this.cellHeight = 36;
        this.distanceTop = 0;
        this.subs = [];
    }
    set cells(v) {
        this._cells = v || [];
        /** 视图更新，重新计算 BetterScroll */
        if (this.scrollInstance) {
            setTimeout(() => {
                console.log(v);
                this.scrollInstance.refresh();
            }, 500);
        }
    }
    get cells() {
        return this._cells;
    }
    ngAfterViewInit() {
        this.initScroll();
        this.subs.push(this.pickerService.onShow.subscribe(() => {
            if (this.scrollInstance) {
                this.scrollInstance.refresh();
                let selectedIndex = 0;
                for (let i = 0; i < this.cells.length; i++) {
                    if (this.cells[i].value === this.value.value) {
                        selectedIndex = i;
                        break;
                    }
                }
                this.scrollInstance.scrollTo(0, -selectedIndex * (this.cellHeight * this.scale));
                return;
            }
            this.initScroll();
        }));
    }
    initScroll() {
        this.scrollInstance = new BScroll(this.elementRef.nativeElement, {
            wheel: {
                selectedIndex: 0,
                rotate: 0,
                wheelWrapperClass: 'ui-picker-column',
                wheelItemClass: 'ui-picker-cell'
            },
            probeType: 1
        });
        this.scrollInstance.on('scrollStart', () => {
            this.pickerService.scroll(true);
        });
        this.scrollInstance.on('scrollEnd', (position) => {
            this.pickerService.scroll(false);
            const value = this.cells[Math.floor(Math.abs(position.y / (this.cellHeight * this.scale)))];
            if (value.disabled) {
                return;
            }
            this.uiSelected.emit(value);
        });
    }
    ngOnDestroy() {
        if (this.scrollInstance) {
            this.scrollInstance.destroy();
        }
        this.subs.forEach(item => item.unsubscribe());
    }
};
PickerColumnComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: Number, decorators: [{ type: Inject, args: [UI_SCREEN_SCALE,] }] },
    { type: PickerService }
];
__decorate([
    Output(),
    __metadata("design:type", Object)
], PickerColumnComponent.prototype, "uiSelected", void 0);
__decorate([
    Input(),
    __metadata("design:type", Array),
    __metadata("design:paramtypes", [Array])
], PickerColumnComponent.prototype, "cells", null);
__decorate([
    Input(),
    __metadata("design:type", Number)
], PickerColumnComponent.prototype, "cellHeight", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], PickerColumnComponent.prototype, "value", void 0);
PickerColumnComponent = __decorate([
    Component({
        selector: 'ui-picker-column',
        template: "<div class=\"ui-picker-column\"\n     [ngStyle]=\"{transform: 'translateY(' + distanceTop + 'px)'}\">\n  <button type=\"button\"\n          *ngFor=\"let cell of cells\"\n          [disabled]=\"cell.disabled\"\n          class=\"ui-picker-cell\">{{ cell.text }}\n  </button>\n</div>\n"
    }),
    __param(1, Inject(UI_SCREEN_SCALE)),
    __metadata("design:paramtypes", [ElementRef, Number, PickerService])
], PickerColumnComponent);

let RadioStateService = class RadioStateService {
    constructor() {
        this.stateSource = new Subject();
        this.state = this.stateSource.asObservable();
    }
    publishEvent() {
        this.stateSource.next();
    }
};
RadioStateService = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], RadioStateService);

var RadioComponent_1;
let RadioComponent = RadioComponent_1 = class RadioComponent {
    constructor(radioStateService) {
        this.radioStateService = radioStateService;
        this.uiChange = new EventEmitter();
        this.checkedIcon = 'ui-icon-radio-checked';
        this.uncheckedIcon = 'ui-icon-radio-unchecked';
        this.value = '';
        this.text = '';
        this._disabled = false;
        this._readonly = false;
        this._checked = false;
    }
    set disabled(isDisabled) {
        this._disabled = inputAttrToBoolean(isDisabled);
    }
    get disabled() {
        return this._disabled;
    }
    set readonly(isReadonly) {
        this._readonly = inputAttrToBoolean(isReadonly);
    }
    get readonly() {
        return this._readonly;
    }
    set checked(isChecked) {
        this._checked = inputAttrToBoolean(isChecked);
    }
    get checked() {
        return this._checked;
    }
    click() {
        if (this.disabled || this.readonly) {
            return;
        }
        this.rawInput.nativeElement.checked = true;
        if (this.onChange) {
            this.onChange(this.value);
        }
        if (this.onTouched) {
            this.onTouched(this.value);
        }
        // 当自身被点击时，发布事件，更新其它radio状态
        this.radioStateService.publishEvent();
        this.uiChange.emit(this.value);
    }
    ngOnInit() {
        // 当某一个radio被点击时，更新其它radio状态
        this.sub = this.radioStateService.state.subscribe(() => {
            this.checked = this.rawInput.nativeElement.checked;
        });
    }
    ngOnDestroy() {
        this.sub.unsubscribe();
    }
    writeValue(value) {
        if (typeof value === 'number') {
            this.checked = this.value === ('' + value);
        }
        else {
            this.checked = this.value === value;
        }
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
};
RadioComponent.ctorParameters = () => [
    { type: RadioStateService }
];
__decorate([
    ViewChild('rawInput', { static: true }),
    __metadata("design:type", ElementRef)
], RadioComponent.prototype, "rawInput", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], RadioComponent.prototype, "uiChange", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], RadioComponent.prototype, "checkedIcon", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], RadioComponent.prototype, "uncheckedIcon", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], RadioComponent.prototype, "name", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], RadioComponent.prototype, "value", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], RadioComponent.prototype, "forId", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], RadioComponent.prototype, "text", void 0);
__decorate([
    Input(),
    HostBinding('class.ui-disabled'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], RadioComponent.prototype, "disabled", null);
__decorate([
    Input(),
    HostBinding('class.ui-readonly'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], RadioComponent.prototype, "readonly", null);
__decorate([
    Input(),
    HostBinding('class.ui-checked'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], RadioComponent.prototype, "checked", null);
__decorate([
    HostListener('click'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], RadioComponent.prototype, "click", null);
RadioComponent = RadioComponent_1 = __decorate([
    Component({
        selector: 'ui-input[type=radio]',
        template: "<div class=\"ui-radio\">\n  <span *ngIf=\"checked\" [ngClass]=\"checkedIcon\">{{ text }}</span>\n  <span *ngIf=\"!checked\" [ngClass]=\"uncheckedIcon\">{{ text }}</span>\n</div>\n<input #rawInput class=\"ui-input-radio\"\n       type=\"radio\"\n       [value]=\"value\"\n       [attr.id]=\"forId\"\n       [attr.name]=\"name\"\n       [disabled]=\"disabled\"\n       [checked]=\"checked\"\n       [readonly]=\"readonly\">\n",
        providers: [{
                provide: NG_VALUE_ACCESSOR,
                useExisting: RadioComponent_1,
                multi: true
            }]
    }),
    __metadata("design:paramtypes", [RadioStateService])
], RadioComponent);

var RangeComponent_1;
let RangeComponent = RangeComponent_1 = class RangeComponent {
    constructor(elementRef, renderer) {
        this.elementRef = elementRef;
        this.renderer = renderer;
        this.uiChange = new EventEmitter();
        this.showProgress = false;
        this.position = 50;
        this.isTouching = false;
        this._disabled = false;
        this._readonly = false;
        this._min = 0;
        this._max = 100;
        this._step = 1;
        this._value = 50;
    }
    set disabled(isDisabled) {
        this._disabled = inputAttrToBoolean(isDisabled);
    }
    get disabled() {
        return this._readonly;
    }
    set readonly(isReadonly) {
        this._readonly = inputAttrToBoolean(isReadonly);
    }
    get readonly() {
        return this._readonly;
    }
    set min(min) {
        const v = RangeComponent_1.toNumber(min);
        if (!isNaN(v)) {
            this._min = v;
        }
    }
    get min() {
        return this._min;
    }
    set max(max) {
        const v = RangeComponent_1.toNumber(max);
        if (!isNaN(v)) {
            this._max = v;
        }
    }
    get max() {
        return this._max;
    }
    set step(step) {
        const v = RangeComponent_1.toNumber(step);
        if (!isNaN(v)) {
            this._step = v;
        }
    }
    get step() {
        return this._step;
    }
    set value(value) {
        let v = RangeComponent_1.toNumber(value);
        if (!isNaN(v)) {
            this._value = v;
            if (this.min <= this.max) {
                if (v < this.min) {
                    v = this.min;
                }
                else if (v > this.max) {
                    v = this.max;
                }
                this.position = (v - this.min) / (this.max - this.min) * 100;
            }
        }
    }
    get value() {
        return this._value;
    }
    static toNumber(value) {
        if (typeof value === 'number') {
            return value;
        }
        return Number(value);
    }
    drag(event) {
        if (this.readonly || this.disabled) {
            return;
        }
        this.isTouching = true;
        let unbindTouchEndFn;
        let unbindTouchCancelFn;
        let unbindTouchMoveFn;
        if (this.min >= this.max) {
            return;
        }
        const element = event.target;
        unbindTouchEndFn = this.renderer.listen(element, 'touchend', () => {
            this.isTouching = false;
            unbindTouchMoveFn();
            unbindTouchEndFn();
            unbindTouchCancelFn();
        });
        unbindTouchCancelFn = this.renderer.listen(element, 'touchcancel', () => {
            this.isTouching = false;
            unbindTouchMoveFn();
            unbindTouchEndFn();
            unbindTouchCancelFn();
        });
        const section = this.max - this.min;
        const maxWidth = this.elementRef.nativeElement.offsetWidth;
        const nowWidth = this.rangeBar.nativeElement.offsetWidth;
        const oldX = event.touches[0].clientX;
        let oldValue = this.value;
        unbindTouchMoveFn = this.renderer.listen(element, 'touchmove', (ev) => {
            const dragDistance = ev.touches[0].clientX - oldX;
            const proportion = (nowWidth + dragDistance) / maxWidth;
            const temporaryValue = Math.floor(section * proportion / this.step) * this.step;
            let value = this.min + temporaryValue;
            if (value < this.min) {
                value = this.min;
            }
            else if (value > this.max) {
                value = this.max - (this.max - this.min) % this.step;
            }
            if (value !== oldValue) {
                this.value = value;
                oldValue = value;
                if (this.onChange) {
                    this.onChange(value);
                }
                if (this.onTouched) {
                    this.onTouched(value);
                }
                this.uiChange.emit(value);
            }
            ev.stopPropagation();
        });
    }
    writeValue(value) {
        this.value = value;
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
};
RangeComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
__decorate([
    ViewChild('rangeBar', { static: true }),
    __metadata("design:type", ElementRef)
], RangeComponent.prototype, "rangeBar", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], RangeComponent.prototype, "uiChange", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], RangeComponent.prototype, "name", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], RangeComponent.prototype, "forId", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], RangeComponent.prototype, "showProgress", void 0);
__decorate([
    Input(),
    HostBinding('class.ui-disabled'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], RangeComponent.prototype, "disabled", null);
__decorate([
    Input(),
    HostBinding('class.ui-readonly'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], RangeComponent.prototype, "readonly", null);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], RangeComponent.prototype, "min", null);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], RangeComponent.prototype, "max", null);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], RangeComponent.prototype, "step", null);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], RangeComponent.prototype, "value", null);
RangeComponent = RangeComponent_1 = __decorate([
    Component({
        selector: 'ui-input[type=range]',
        template: "<div class=\"ui-range\">\n  <div class=\"ui-range-progress-bar\" #rangeBar [ngStyle]=\"{width: position + '%'}\">\n    <div class=\"ui-range-btn\">\n      <span class=\"ui-range-progress\" *ngIf=\"showProgress\" [ngClass]=\"{'ui-touched': isTouching}\">{{ value }}</span>\n      <button class=\"ui-range-btn-raw\" type=\"button\" (touchstart)=\"drag($event)\">\n      </button>\n    </div>\n  </div>\n</div>\n<input class=\"ui-input-range-raw\"\n       type=\"range\"\n       [value]=\"value\"\n       [attr.id]=\"forId\"\n       [attr.name]=\"name\"\n       [disabled]=\"disabled\"\n       [max]=\"max\"\n       [min]=\"min\"\n       [step]=\"step\"\n       [readonly]=\"readonly\">",
        providers: [{
                provide: NG_VALUE_ACCESSOR,
                useExisting: RangeComponent_1,
                multi: true
            }]
    }),
    __metadata("design:paramtypes", [ElementRef,
        Renderer2])
], RangeComponent);

var SelectComponent_1;
let SelectComponent = SelectComponent_1 = class SelectComponent {
    constructor(arrowIcon, selectService, changeDetectorRef) {
        this.selectService = selectService;
        this.changeDetectorRef = changeDetectorRef;
        this.uiChange = new EventEmitter();
        this.placeholder = '';
        this.selectedIndex = 0;
        this.cancelText = '取消';
        this.arrowIconClassName = '';
        this.focus = false;
        this.text = '';
        this._disabled = false;
        this._readonly = false;
        this.value = '';
        this.subs = [];
        this.isWrite = false;
        this.arrowIconClassName = arrowIcon;
    }
    set disabled(isDisabled) {
        this._disabled = inputAttrToBoolean(isDisabled);
    }
    get disabled() {
        return this._disabled;
    }
    set readonly(isReadonly) {
        this._readonly = inputAttrToBoolean(isReadonly);
    }
    get readonly() {
        return this._readonly;
    }
    static getTextByElement(element) {
        if (element) {
            return element.innerText.replace(/^[\s\n\t\r]+|[\s\n\t\r]+$/g, '');
        }
        return '';
    }
    ngAfterContentInit() {
        if (!this.isWrite) {
            this.updateBySelf();
        }
        this.subs.push(this.selectService.onChecked.subscribe((option) => {
            this.options.forEach((op, index) => {
                if (op === option) {
                    if (op !== this.selectedOption) {
                        this.selectedOption = op;
                        op.selected = true;
                        this.value = option.value;
                        this.text = SelectComponent_1.getTextByElement(option.nativeElement);
                        this.selectedIndex = index;
                        if (this.onChange) {
                            this.onChange(this.value);
                        }
                        if (this.onTouched) {
                            this.onTouched();
                        }
                        this.uiChange.emit(this.value);
                    }
                }
                else {
                    op.selected = false;
                }
            });
        }));
        this.subs.push(this.options.changes.pipe(delay(0)).subscribe(() => {
            this.updateByNgModel();
        }));
    }
    ngAfterViewInit() {
        if (this.selectedOption) {
            this.text = SelectComponent_1.getTextByElement(this.selectedOption.nativeElement);
            this.changeDetectorRef.detectChanges();
        }
    }
    ngOnDestroy() {
        this.subs.forEach(item => {
            item.unsubscribe();
        });
    }
    showOptions() {
        if (!this.disabled && !this.readonly) {
            this.focus = true;
        }
    }
    hideOptions() {
        this.focus = false;
    }
    writeValue(value) {
        this.isWrite = true;
        this.value = value;
        if (this.options) {
            let selectedOption;
            this.options.forEach((item, index) => {
                item.selected = false;
                if (item.value === value || `${item.value}` === value || item.value === `${value}`) {
                    selectedOption = item;
                    this.selectedIndex = index;
                }
            });
            if (selectedOption) {
                this.text = SelectComponent_1.getTextByElement(selectedOption.nativeElement);
                selectedOption.selected = true;
            }
            else {
                this.selectedOption = null;
                this.text = '';
            }
        }
        else {
            this.selectedOption = null;
            this.text = '';
        }
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
    updateByNgModel() {
        let selectedOption;
        this.options.forEach((item, index) => {
            item.selected = false;
            if (item.value === this.value || `${item.value}` === this.value || item.value === `${this.value}`) {
                selectedOption = item;
                this.selectedIndex = index;
            }
        });
        if (selectedOption) {
            this.text = SelectComponent_1.getTextByElement(selectedOption.nativeElement);
            selectedOption.selected = true;
            this.selectedOption = selectedOption;
        }
        else {
            this.selectedIndex = -1;
            this.text = '';
            this.selectedOption = null;
        }
    }
    updateBySelf() {
        let defaultOption;
        this.options.forEach((option, index) => {
            if (option.selected) {
                defaultOption = option;
                this.selectedIndex = index;
            }
        });
        if (!defaultOption) {
            defaultOption = this.options.toArray()[this.selectedIndex];
        }
        if (!defaultOption) {
            defaultOption = this.options.first;
            this.selectedIndex = 0;
        }
        if (defaultOption) {
            this.value = defaultOption.value;
            setTimeout(() => {
                if (!this.isWrite) {
                    defaultOption.selected = true;
                }
            });
            this.selectedOption = defaultOption;
        }
    }
};
SelectComponent.ctorParameters = () => [
    { type: String, decorators: [{ type: Inject, args: [UI_SELECT_ARROW_CLASSNAME,] }] },
    { type: SelectService },
    { type: ChangeDetectorRef }
];
__decorate([
    ContentChildren(OptionComponent),
    __metadata("design:type", QueryList)
], SelectComponent.prototype, "options", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], SelectComponent.prototype, "uiChange", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], SelectComponent.prototype, "forId", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], SelectComponent.prototype, "name", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], SelectComponent.prototype, "placeholder", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], SelectComponent.prototype, "selectedIndex", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], SelectComponent.prototype, "cancelText", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], SelectComponent.prototype, "arrowIconClassName", void 0);
__decorate([
    Input(),
    HostBinding('class.ui-disabled'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], SelectComponent.prototype, "disabled", null);
__decorate([
    Input(),
    HostBinding('class.ui-readonly'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], SelectComponent.prototype, "readonly", null);
SelectComponent = SelectComponent_1 = __decorate([
    Component({
        selector: 'ui-select',
        template: "<div class=\"ui-select-fixed\" (click)=\"showOptions()\">\n  <div class=\"ui-raw-input-wrap\">\n    <input class=\"ui-raw-input\" [disabled]=\"disabled\"\n           [attr.name]=\"name\"\n           [placeholder]=\"placeholder\"\n           readonly\n           [attr.id]=\"forId\"\n           type=\"text\"\n           [value]=\"text\">\n    <div class=\"ui-raw-input-mask\"></div>\n  </div>\n  <div class=\"ui-select-arrow\" *ngIf=\"arrowIconClassName\">\n    <span [ngClass]=\"arrowIconClassName\"></span>\n  </div>\n</div>\n<ui-action-sheet [show]=\"focus\" (click)=\"hideOptions()\" uiStopPropagation=\"touchmove\">\n  <div class=\"ui-select-modal\">\n    <div class=\"ui-select-options\">\n      <ng-content></ng-content>\n    </div>\n    <div class=\"ui-select-btn-wrap\" *ngIf=\"cancelText\">\n      <button type=\"button\" class=\"ui-select-btn\">{{ cancelText }}</button>\n    </div>\n  </div>\n</ui-action-sheet>\n",
        providers: [{
                provide: NG_VALUE_ACCESSOR,
                useExisting: SelectComponent_1,
                multi: true
            }, SelectService
        ]
    }),
    __param(0, Inject(UI_SELECT_ARROW_CLASSNAME)),
    __metadata("design:paramtypes", [String, SelectService,
        ChangeDetectorRef])
], SelectComponent);

var SwitchComponent_1;
let SwitchComponent = SwitchComponent_1 = class SwitchComponent {
    constructor() {
        this.uiChange = new EventEmitter();
        this.value = '';
        this.name = '';
        this._disabled = false;
        this._readonly = false;
        this._checked = false;
    }
    set disabled(isDisabled) {
        this._disabled = inputAttrToBoolean(isDisabled);
    }
    get disabled() {
        return this._disabled;
    }
    set readonly(isReadonly) {
        this._readonly = inputAttrToBoolean(isReadonly);
    }
    get readonly() {
        return this._readonly;
    }
    set checked(isChecked) {
        this._checked = inputAttrToBoolean(isChecked);
    }
    get checked() {
        return this._checked;
    }
    click() {
        if (this.disabled || this.readonly) {
            return;
        }
        this.checked = !this.checked;
        if (this.onChange) {
            this.onChange(this.checked);
        }
        if (this.onTouched) {
            this.onTouched(this.checked);
        }
        this.uiChange.emit(this.checked);
    }
    writeValue(value) {
        this.checked = !!value;
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
};
__decorate([
    Output(),
    __metadata("design:type", Object)
], SwitchComponent.prototype, "uiChange", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], SwitchComponent.prototype, "forId", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], SwitchComponent.prototype, "value", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], SwitchComponent.prototype, "name", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], SwitchComponent.prototype, "disabled", null);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], SwitchComponent.prototype, "readonly", null);
__decorate([
    Input(),
    HostBinding('class.ui-checked'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], SwitchComponent.prototype, "checked", null);
__decorate([
    HostListener('click'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SwitchComponent.prototype, "click", null);
SwitchComponent = SwitchComponent_1 = __decorate([
    Component({
        selector: 'ui-switch',
        template: "<button type=\"button\" class=\"ui-switch\">\n  <div class=\"ui-switch-btn\"></div>\n</button>\n<input class=\"ui-input-raw\"\n       type=\"radio\"\n       [value]=\"value\"\n       [attr.id]=\"forId\"\n       [attr.name]=\"name\"\n       [disabled]=\"disabled\"\n       [checked]=\"checked\"\n       [readonly]=\"readonly\">",
        providers: [{
                provide: NG_VALUE_ACCESSOR,
                useExisting: SwitchComponent_1,
                multi: true
            }]
    })
], SwitchComponent);

var CheckboxRequiredValidator_1, SwitchRequiredValidator_1, RadioRequiredValidator_1, SelectRequiredValidator_1, DateRequiredValidator_1;
let CheckboxRequiredValidator = CheckboxRequiredValidator_1 = class CheckboxRequiredValidator {
    get required() {
        return this._required;
    }
    set required(value) {
        this._required = value !== false && value !== null;
        if (this._onChange) {
            this._onChange();
        }
    }
    validate(c) {
        if (this.required) {
            return c.value === true ? null : { 'required': true };
        }
        return null;
    }
    registerOnValidatorChange(fn) {
        this._onChange = fn;
    }
};
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], CheckboxRequiredValidator.prototype, "required", null);
CheckboxRequiredValidator = CheckboxRequiredValidator_1 = __decorate([
    Directive({
        /* tslint:disable */
        selector: 'ui-input[required][type=checkbox]',
        /* tslint:enable */
        providers: [{
                provide: NG_VALIDATORS,
                useExisting: CheckboxRequiredValidator_1,
                multi: true
            }]
    })
], CheckboxRequiredValidator);
let SwitchRequiredValidator = SwitchRequiredValidator_1 = class SwitchRequiredValidator {
    get required() {
        return this._required;
    }
    set required(value) {
        this._required = value !== false && value !== null;
        if (this._onChange) {
            this._onChange();
        }
    }
    validate(c) {
        if (this.required) {
            return c.value === true ? null : { 'required': true };
        }
        return null;
    }
    registerOnValidatorChange(fn) {
        this._onChange = fn;
    }
};
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], SwitchRequiredValidator.prototype, "required", null);
SwitchRequiredValidator = SwitchRequiredValidator_1 = __decorate([
    Directive({
        /* tslint:disable */
        selector: 'ui-switch[required]',
        /* tslint:enable */
        providers: [{
                provide: NG_VALIDATORS,
                useExisting: SwitchRequiredValidator_1,
                multi: true
            }]
    })
], SwitchRequiredValidator);
let RadioRequiredValidator = RadioRequiredValidator_1 = class RadioRequiredValidator {
    get required() {
        return this._required;
    }
    set required(value) {
        this._required = value !== false && value !== null;
        if (this._onChange) {
            this._onChange();
        }
    }
    validate(c) {
        if (this.required) {
            return c.value !== '' && c.value !== undefined && c.value !== null ? null : { 'required': true };
        }
        return null;
    }
    registerOnValidatorChange(fn) {
        this._onChange = fn;
    }
};
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], RadioRequiredValidator.prototype, "required", null);
RadioRequiredValidator = RadioRequiredValidator_1 = __decorate([
    Directive({
        /* tslint:disable */
        selector: 'ui-input[required][type=radio]',
        /* tslint:enable */
        providers: [{
                provide: NG_VALIDATORS,
                useExisting: RadioRequiredValidator_1,
                multi: true
            }]
    })
], RadioRequiredValidator);
let SelectRequiredValidator = SelectRequiredValidator_1 = class SelectRequiredValidator {
    get required() {
        return this._required;
    }
    set required(value) {
        this._required = value !== false && value !== null;
        if (this._onChange) {
            this._onChange();
        }
    }
    validate(c) {
        if (this.required) {
            return c.value !== '' && c.value !== undefined && c.value !== null ? null : { 'required': true };
        }
        return null;
    }
    registerOnValidatorChange(fn) {
        this._onChange = fn;
    }
};
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], SelectRequiredValidator.prototype, "required", null);
SelectRequiredValidator = SelectRequiredValidator_1 = __decorate([
    Directive({
        /* tslint:disable */
        selector: 'ui-select[required]',
        /* tslint:enable */
        providers: [{
                provide: NG_VALIDATORS,
                useExisting: SelectRequiredValidator_1,
                multi: true
            }]
    })
], SelectRequiredValidator);
let DateRequiredValidator = DateRequiredValidator_1 = class DateRequiredValidator {
    get required() {
        return this._required;
    }
    set required(value) {
        this._required = value !== false && value !== null;
        if (this._onChange) {
            this._onChange();
        }
    }
    validate(c) {
        if (this.required) {
            return c.value !== '' && c.value !== undefined && c.value !== null ? null : { 'required': true };
        }
        return null;
    }
    registerOnValidatorChange(fn) {
        this._onChange = fn;
    }
};
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], DateRequiredValidator.prototype, "required", null);
DateRequiredValidator = DateRequiredValidator_1 = __decorate([
    Directive({
        /* tslint:disable */
        selector: 'ui-input[type=date][required]',
        /* tslint:enable */
        providers: [{
                provide: NG_VALIDATORS,
                useExisting: DateRequiredValidator_1,
                multi: true
            }]
    })
], DateRequiredValidator);

let SubmitDirective = class SubmitDirective {
    constructor(ngForm) {
        this.ngForm = ngForm;
        this.uiSubmit = new EventEmitter();
    }
    ngOnInit() {
        const group = this.ngForm.form;
        this.sub = this.ngForm.ngSubmit.subscribe((ev) => {
            if (!this.ngForm.valid) {
                this.markAsTouched(group);
            }
            else {
                this.uiSubmit.emit(ev);
            }
        });
    }
    ngOnDestroy() {
        this.sub.unsubscribe();
    }
    markAsTouched(group) {
        Object.keys(group.controls).map((field) => {
            const control = group.get(field);
            if (control instanceof FormControl) {
                control.markAsDirty({ onlySelf: true });
                control.markAsTouched({ onlySelf: true });
            }
            else if (control instanceof FormGroup) {
                this.markAsTouched(control);
            }
        });
    }
};
SubmitDirective.ctorParameters = () => [
    { type: NgForm, decorators: [{ type: Self }, { type: Optional }] }
];
__decorate([
    Output(),
    __metadata("design:type", Object)
], SubmitDirective.prototype, "uiSubmit", void 0);
SubmitDirective = __decorate([
    Directive({
        /*tslint:disable*/
        selector: '[uiSubmit]'
        /*tslint:enable*/
    }),
    __param(0, Self()), __param(0, Optional()),
    __metadata("design:paramtypes", [NgForm])
], SubmitDirective);

let ImageViewerController = class ImageViewerController {
    constructor() {
        this.imageSource = new Subject();
        this.displayEvent = new Subject();
        this.imageList = [];
        this.images = this.imageSource.asObservable();
        this.display = this.displayEvent.asObservable();
    }
    addImage(src, srcElement) {
        if (this.findIndex(src) === -1) {
            this.imageList.push({
                srcElement,
                src,
                scale: 1
            });
            this.imageSource.next(this.imageList);
        }
    }
    removeImage(src) {
        const index = this.findIndex(src);
        if (index > -1) {
            this.imageList.splice(index, 1);
            this.imageSource.next(this.imageList);
        }
    }
    show(firstSrc) {
        this.displayEvent.next({
            firstSrc,
            show: true
        });
    }
    hide() {
        this.displayEvent.next({
            show: false
        });
    }
    findIndex(src) {
        for (let i = 0; i < this.imageList.length; i++) {
            if (this.imageList[i].src === src) {
                return i;
            }
        }
        return -1;
    }
};
ImageViewerController = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], ImageViewerController);

let ImageViewableDirective = class ImageViewableDirective {
    constructor(imageViewerService, elementRef) {
        this.imageViewerService = imageViewerService;
        this.elementRef = elementRef;
    }
    set src(v) {
        if (v && typeof v === 'string') {
            this.imageViewerService.removeImage(this._src);
            this._src = v;
            this.imageViewerService.addImage(v, this.elementRef.nativeElement);
        }
    }
    get src() {
        return this._src;
    }
    ngOnDestroy() {
        this.imageViewerService.removeImage(this.src);
    }
    click() {
        this.imageViewerService.show(this.src);
    }
};
ImageViewableDirective.ctorParameters = () => [
    { type: ImageViewerController },
    { type: ElementRef }
];
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], ImageViewableDirective.prototype, "src", null);
__decorate([
    HostListener('click'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ImageViewableDirective.prototype, "click", null);
ImageViewableDirective = __decorate([
    Directive({
        selector: '[uiImageViewable]'
    }),
    __metadata("design:paramtypes", [ImageViewerController,
        ElementRef])
], ImageViewableDirective);

let ImageViewerComponent = class ImageViewerComponent {
    constructor(imageViewerService, renderer2, elementRef) {
        this.imageViewerService = imageViewerService;
        this.renderer2 = renderer2;
        this.elementRef = elementRef;
        this.isShow = false;
        this.bgColor = '';
        this.isShowSlide = false;
        this.images = [];
        this.viewIndex = 0;
        this.opacity = 0;
        this.subs = [];
    }
    get transition() {
        return this.bgColor ? 'none' : '';
    }
    ngOnInit() {
        this.subs.push(this.imageViewerService.images.subscribe(images => {
            this.images = images;
        }));
        this.subs.push(this.imageViewerService.display.subscribe(config => {
            this.isShow = config.show;
            if (config.show) {
                this.isShowSlide = true;
                const index = this.findIndex(config.firstSrc);
                this.viewIndex = index > -1 ? index : 0;
                this.setupInitStyles();
            }
        }));
    }
    ngOnDestroy() {
        this.subs.forEach(item => item.unsubscribe());
    }
    click() {
        const current = this.images[this.viewIndex];
        if (current.srcElement) {
            const rect = current.srcElement.getBoundingClientRect();
            this.renderer2.setStyle(current.srcElement, 'opacity', 0);
            const fn = this.renderer2.listen(this.imgElements.toArray()[this.viewIndex].nativeElement, 'transitionend', () => {
                this.opacity = 0;
                this.isShowSlide = false;
                this.renderer2.setStyle(current.srcElement, 'opacity', 1);
                fn();
            });
            current.styles = {
                left: rect.left,
                top: rect.top,
                width: rect.width,
                height: rect.height,
                transition: 'all .3s'
            };
        }
        this.imageViewerService.hide();
    }
    pinch(ev, imageView) {
        if (ev.first) {
            const imgRect = this.imgElements.toArray()[this.viewIndex].nativeElement.getBoundingClientRect();
            imageView.styles.transformOrigin = `${ev.moveX - imgRect.left}px ${ev.moveY - imgRect.top}px`;
        }
        if (ev.type === 'touchmove') {
            const transforms = [
                `scale(${ev.scale})`,
                `translate(${ev.distanceX / ev.scale}px, ${ev.distanceY / ev.scale}px)`
            ];
            imageView.styles.transition = 'none';
            imageView.styles.transform = transforms.join(' ');
        }
        else if (ev.type === 'touchend') {
            const imgRect = this.imgElements.toArray()[this.viewIndex].nativeElement.getBoundingClientRect();
            imageView.styles = {
                width: imgRect.width,
                height: imgRect.height,
                left: imgRect.left,
                top: imgRect.top
            };
        }
        ev.srcEvent.stopPropagation();
    }
    drag(ev, imageView) {
        if (ev.firstDirection === 'down') {
            const scale = ev.cumulativeDistanceY < 0 ? 1 :
                (1 - ev.cumulativeDistanceY / 4 / Number(imageView.styles.height));
            const transforms = [
                `translate(${ev.cumulativeDistanceX}px, ${ev.cumulativeDistanceY}px)`,
                `scale(${scale})`
            ];
            imageView.styles.transition = 'none';
            imageView.styles.transform = transforms.join(' ');
            const alpha = ev.cumulativeDistanceY < 0 ? 1 : (1 - ev.cumulativeDistanceY / Number(imageView.styles.height));
            this.bgColor = `rgba(0,0,0,${alpha < 0.5 ? 0.5 : alpha})`;
            if (ev.type === 'touchend') {
                this.bgColor = '';
                ev.resetCumulative();
                if (ev.cumulativeDistanceY > this.elementRef.nativeElement.offsetHeight / 4) {
                    this.imageViewerService.hide();
                    const rect = imageView.srcElement.getBoundingClientRect();
                    const fn = this.renderer2.listen(this.imgElements.toArray()[this.viewIndex].nativeElement, 'transitionend', () => {
                        this.opacity = 0;
                        this.isShowSlide = false;
                        this.renderer2.setStyle(imageView.srcElement, 'opacity', 1);
                        fn();
                    });
                    imageView.styles = {
                        width: rect.width,
                        height: rect.height,
                        left: rect.left,
                        top: rect.top,
                        transition: 'all .3s'
                    };
                }
                else {
                    imageView.styles.transform = 'none';
                    imageView.styles.transition = 'all .3s';
                }
            }
        }
        else {
            ev.resetCumulative();
            ev.stop();
        }
    }
    setViewIndex(index) {
        this.viewIndex = index;
    }
    findIndex(src) {
        for (let i = 0; i < this.images.length; i++) {
            if (this.images[i].src === src) {
                return i;
            }
        }
        return -1;
    }
    setupInitStyles() {
        this.images.forEach((item, index) => {
            item.styles = {};
            if (index === this.viewIndex) {
                if (item.srcElement) {
                    const rect = item.srcElement.getBoundingClientRect();
                    item.styles = {
                        left: rect.left,
                        top: rect.top,
                        width: rect.width,
                        height: rect.height,
                        transition: 'all .3s'
                    };
                }
            }
            requestAnimationFrame(() => {
                if (item.srcElement) {
                    if (index === this.viewIndex) {
                        this.renderer2.setStyle(item.srcElement, 'opacity', 0);
                        const fn = this.renderer2.listen(this.imgElements.toArray()[this.viewIndex].nativeElement, 'transitionend', () => {
                            this.renderer2.setStyle(item.srcElement, 'opacity', 1);
                            item.styles.transition = 'none';
                            fn();
                        });
                    }
                    const containerRect = this.elementRef.nativeElement.getBoundingClientRect();
                    const contentWidth = item.srcElement.naturalWidth;
                    const contentHeight = item.srcElement.naturalHeight;
                    const proportion = (containerRect.width / containerRect.height) / (contentWidth / contentHeight);
                    item.defaultStyles = {};
                    if (contentHeight <= containerRect.height && contentWidth <= containerRect.width) {
                        item.defaultStyles.height = contentHeight;
                        item.defaultStyles.width = contentWidth;
                        item.defaultStyles.top = (containerRect.height - contentHeight) / 2;
                        item.defaultStyles.left = (containerRect.width - contentWidth) / 2;
                    }
                    else {
                        if (proportion > 1) {
                            // 高比宽大
                            const width = contentWidth * containerRect.height / contentHeight;
                            item.defaultStyles.height = containerRect.height;
                            item.defaultStyles.width = width;
                            item.defaultStyles.top = 0;
                            item.defaultStyles.left = (containerRect.width - width) / 2;
                        }
                        else {
                            // 高比宽小
                            const height = contentHeight * containerRect.width / contentWidth;
                            item.defaultStyles.height = height;
                            item.defaultStyles.width = containerRect.width;
                            item.defaultStyles.top = (containerRect.height - height) / 2;
                            item.defaultStyles.left = 0;
                        }
                    }
                    Object.assign(item.styles, item.defaultStyles);
                }
                requestAnimationFrame(() => {
                    this.opacity = 1;
                });
            });
        });
    }
};
ImageViewerComponent.ctorParameters = () => [
    { type: ImageViewerController },
    { type: Renderer2 },
    { type: ElementRef }
];
__decorate([
    ViewChildren('img'),
    __metadata("design:type", QueryList)
], ImageViewerComponent.prototype, "imgElements", void 0);
__decorate([
    HostBinding('class.ui-show'),
    __metadata("design:type", Object)
], ImageViewerComponent.prototype, "isShow", void 0);
__decorate([
    HostBinding('style.backgroundColor'),
    __metadata("design:type", Object)
], ImageViewerComponent.prototype, "bgColor", void 0);
__decorate([
    HostBinding('style.transition'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ImageViewerComponent.prototype, "transition", null);
__decorate([
    HostListener('click'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ImageViewerComponent.prototype, "click", null);
ImageViewerComponent = __decorate([
    Component({
        selector: 'ui-image-viewer',
        template: "<ui-slide (uiPlayed)=\"setViewIndex($event)\" *ngIf=\"isShowSlide\" [autoPlay]=\"false\" [initIndex]=\"viewIndex\">\n  <ui-slide-item class=\"ui-image-viewer-item\" [ngStyle]=\"{opacity: opacity}\"\n                 (uiPinch)=\"pinch($event, item)\" (uiPan)=\"drag($event, item)\"\n                 *ngFor=\"let item of images\">\n    <img #img\n         class=\"ui-image-viewer-img\"\n         [src]=\"item.src\"\n         [ngStyle]=\"{\n            width: item.styles.width + 'px',\n            height: item.styles.height + 'px',\n            left: item.styles.left + 'px',\n            top: item.styles.top + 'px',\n            transition: item.styles.transition,\n            transform: item.styles.transform,\n            transformOrigin: item.styles.transformOrigin\n           }\">\n  </ui-slide-item>\n</ui-slide>"
    }),
    __metadata("design:paramtypes", [ImageViewerController,
        Renderer2,
        ElementRef])
], ImageViewerComponent);

let ButtonsComponent = class ButtonsComponent {
};
ButtonsComponent = __decorate([
    Component({
        selector: 'ui-buttons',
        template: "<ng-content></ng-content>"
    })
], ButtonsComponent);

let RouteCacheController = class RouteCacheController {
    constructor() {
        this.hasHistorySource = new Subject();
        this.hasCache = this.hasHistorySource.asObservable();
    }
    isCacheCurrentView(x) {
        this.hasHistorySource.next(x);
    }
};
RouteCacheController = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], RouteCacheController);

var UIRouteReuseStrategy_1;
let UIRouteReuseStrategy = UIRouteReuseStrategy_1 = class UIRouteReuseStrategy {
    constructor(routeCacheController) {
        this.routeCacheController = routeCacheController;
        this.caches = {};
        this.isCache = false;
        this.routeSequence = [];
        this.routeCacheController.hasCache.subscribe(b => {
            this.isCache = b;
        });
    }
    static getRouteIdentifier(route) {
        const urls = [];
        if (route.routeConfig.loadChildren) {
            return UIRouteReuseStrategy_1.getRouteIdentifier(route.firstChild);
        }
        let r = route;
        while (r.firstChild) {
            r = r.firstChild;
        }
        while (r && r.routeConfig) {
            urls.unshift(r.routeConfig.path);
            r = r.parent;
        }
        return urls.join('/');
    }
    // 是否缓存路由
    shouldDetach(route) {
        return this.isCache;
    }
    // 存储路由
    store(route, handle) {
        const url = UIRouteReuseStrategy_1.getRouteIdentifier(route);
        if (handle) {
            this.routeSequence.push(url);
            if (!this.caches[url]) {
                this.caches[url] = [];
            }
            this.caches[url].push(handle);
            return;
        }
        const caches = this.caches[url];
        caches.pop();
        if (caches.length === 0) {
            delete this.caches[url];
        }
    }
    // 是否还原当前路由
    shouldAttach(route) {
        const url = UIRouteReuseStrategy_1.getRouteIdentifier(route);
        const lastUrl = this.routeSequence[this.routeSequence.length - 1];
        if (this.caches[url] && url === lastUrl) {
            this.routeSequence.pop();
            return true;
        }
        return false;
    }
    // 取得之前缓存的路由
    retrieve(route) {
        if (!route.routeConfig || this.isCache) {
            return null;
        }
        const url = UIRouteReuseStrategy_1.getRouteIdentifier(route);
        const caches = this.caches[url] || [];
        return caches[caches.length - 1];
    }
    // 是否重用路由
    shouldReuseRoute(future, current) {
        return future.routeConfig === current.routeConfig;
    }
};
UIRouteReuseStrategy.ctorParameters = () => [
    { type: RouteCacheController }
];
UIRouteReuseStrategy = UIRouteReuseStrategy_1 = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [RouteCacheController])
], UIRouteReuseStrategy);

let TabController = class TabController {
    constructor() {
        this.tabIndexSource = new Subject();
        this.tabIndex = this.tabIndexSource.asObservable();
    }
    publishIndex(index) {
        this.tabIndexSource.next(index);
    }
};
TabController = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], TabController);

let RouterService = class RouterService {
    constructor() {
        this.parentTouchEvent = new Subject();
        this.activatedSource = new Subject();
        this.animationProgressSource = new Subject();
        this.moveBackProgressSource = new Subject();
        this.activated = this.activatedSource.asObservable();
        this.animationProgress = this.animationProgressSource.asObservable();
        this.moveBackProgress = this.moveBackProgressSource.asObservable();
        this.parentTouchProgress = this.parentTouchEvent.asObservable();
    }
    publish(activateView) {
        this.activatedSource.next(activateView);
    }
    publishAnimationProgress(progress) {
        this.animationProgressSource.next(progress);
    }
    publishMoveBackProgress(progress) {
        this.moveBackProgressSource.next(progress);
    }
    parentTouching(progress) {
        this.parentTouchEvent.next(progress);
    }
};
RouterService = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], RouterService);

var ViewState;
(function (ViewState) {
    ViewState["Activate"] = "Activate";
    ViewState["ToStack"] = "ToStack";
    ViewState["Destroy"] = "Destroy";
    ViewState["Reactivate"] = "Reactivate";
    ViewState["Sleep"] = "Sleep";
})(ViewState || (ViewState = {}));
const UI_VIEW_INIT_STATE = new InjectionToken('UI_VIEW_INIT_STATE');
const UI_VIEW_INIT_TOUCH_BACK = new InjectionToken('UI_VIEW_INIT_TOUCH_BACK');
let ViewStateService = class ViewStateService {
    constructor() {
        this.stateSource = new Subject();
        this.progressSource = new Subject();
        this.touchEvent = new Subject();
        this.state = this.stateSource.asObservable();
        this.progress = this.progressSource.asObservable();
        this.touchProgress = this.touchEvent.asObservable();
    }
    changeState(state) {
        this.stateSource.next(state);
    }
    updateProgress(n) {
        this.progressSource.next(n);
    }
    touching(progress) {
        this.touchEvent.next(progress);
    }
};
ViewStateService = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], ViewStateService);

/**
 * @whatItDoes 定义路由转声动画的帧数
 *
 * 你可以通过 provide 自定义帧数，帧数越多，转场动画的时间越长
 *
 * @howToUse
 * ```typescript
 * import { NgModule } from '@angular/core';
 * import { UI_ROUTER_ANIMATION_STEPS } from '@tanbo/ui-native';
 *
 * @NgModule({
 *   ...
 *   providers: [{
 *     provide: UI_ROUTER_ANIMATION_STEPS,
 *     useValue: 30
 *   }]
 * })
 * export class AppModule {
 * }
 * ```
 */
const UI_ROUTER_ANIMATION_STEPS = new InjectionToken('UI_ROUTER_ANIMATION_STEPS');
/**
 * @whatItDoes 定义 ui-back 组件左箭头的 html class 的值
 *
 * 你可以通过 provide 定义自己的 class
 *
 * @howToUse
 * ```typescript
 * import { NgModule } from '@angular/core';
 * import { UI_BACK_ICON_CLASSNAME } from '@tanbo/ui-native';
 *
 * @NgModule({
 *   ...
 *   providers: [{
 *     provide: UI_BACK_ICON_CLASSNAME,
 *     useValue: 'custom-classname'
 *   }]
 * })
 * export class AppModule {
 * }
 * ```
 */
const UI_BACK_ICON_CLASSNAME = new InjectionToken('UI_BACK_ICON_CLASSNAME');

let BackComponent = class BackComponent {
    constructor(document, steps, scale, iconName, state, location, router, appController, viewStateService) {
        this.document = document;
        this.steps = steps;
        this.scale = scale;
        this.state = state;
        this.location = location;
        this.router = router;
        this.appController = appController;
        this.viewStateService = viewStateService;
        this.icon = '';
        this.closeBackHandle = false;
        this.opacity = 1;
        this.subs = [];
        this.timer = null;
        this.icon = iconName || '';
    }
    resize() {
        this.docWidth = this.document.body.offsetWidth;
        this.translateDistance = this.docWidth / 2 - (this.leftDistance + 10 * this.scale) - this.contentWidth / 2;
    }
    click() {
        // 当用户点击组件时，触发视图返回上一页
        clearTimeout(this.timer);
        if (!this.closeBackHandle) {
            this.timer = setTimeout(() => {
                this.appController.quit();
            }, 300);
            this.location.back();
        }
    }
    ngOnInit() {
        this.subs.push(this.router.events.subscribe((event) => {
            if (event instanceof NavigationStart) {
                clearTimeout(this.timer);
            }
        }));
        const steps = this.steps;
        const bezier = new CubicBezier(.36, .66, .04, 1);
        this.subs.push(this.viewStateService.state.subscribe((state) => {
            this.state = state;
        }));
        this.subs.push(this.viewStateService.touchProgress.subscribe(p => {
            const progress = bezier.update(p / steps);
            const n = p / 50;
            let m;
            if (this.state === ViewState.Activate || this.state === ViewState.Reactivate) {
                this.translate = `translate3d(${p / steps * this.translateDistance}px, 0, 0)`;
                m = 1 - n;
                this.opacity = m < 0 ? 0 : m;
            }
            else if (this.state === ViewState.ToStack) {
                this.translate = `translate3d(${-50 + 50 * p / steps}%, 0, 0)`;
                m = progress * 2;
                this.opacity = m > 1 ? 1 : m;
            }
        }));
        this.subs.push(this.viewStateService.progress.subscribe((p) => {
            const progress = bezier.update(p / steps);
            const n = p / 50;
            let m;
            switch (this.state) {
                case ViewState.Activate:
                    this.opacity = 1;
                    const distance = this.translateDistance - progress * this.translateDistance;
                    this.translate = `translate3d(${distance}px, 0, 0)`;
                    break;
                case ViewState.Destroy:
                    this.opacity = 1;
                    this.translate = `translate3d(${this.translateDistance * progress}px, 0, 0)`;
                    break;
                case ViewState.ToStack:
                    this.translate = `translate3d(${-progress * 100}%, 0, 0)`;
                    m = 1 - n;
                    this.opacity = m < 0 ? 0 : m;
                    break;
                case ViewState.Reactivate:
                    this.translate = `translate3d(${-50 + progress * 50}%, 0, 0)`;
                    m = progress * 2;
                    this.opacity = m > 1 ? 1 : m;
                    break;
            }
        }));
    }
    ngAfterViewInit() {
        this.docWidth = this.document.body.offsetWidth;
        this.contentWidth = this.textElement.nativeElement.offsetWidth;
        this.leftDistance = this.textElement.nativeElement.offsetLeft;
        this.translateDistance = this.docWidth / 2 - (this.leftDistance + 10 * this.scale) - this.contentWidth / 2;
    }
    ngOnDestroy() {
        this.subs.forEach(item => {
            item.unsubscribe();
        });
    }
};
BackComponent.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
    { type: Number, decorators: [{ type: Inject, args: [UI_ROUTER_ANIMATION_STEPS,] }] },
    { type: Number, decorators: [{ type: Inject, args: [UI_SCREEN_SCALE,] }] },
    { type: String, decorators: [{ type: Inject, args: [UI_BACK_ICON_CLASSNAME,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [UI_VIEW_INIT_STATE,] }] },
    { type: Location },
    { type: Router },
    { type: AppController },
    { type: ViewStateService }
];
__decorate([
    ViewChild('text', { static: true }),
    __metadata("design:type", ElementRef)
], BackComponent.prototype, "textElement", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], BackComponent.prototype, "icon", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], BackComponent.prototype, "closeBackHandle", void 0);
__decorate([
    HostListener('window:resize'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], BackComponent.prototype, "resize", null);
__decorate([
    HostListener('click'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], BackComponent.prototype, "click", null);
BackComponent = __decorate([
    Component({
        selector: 'ui-back',
        template: "<div class=\"ui-back\">\n  <div class=\"ui-back-icon\">\n    <div [ngClass]=\"icon\"></div>\n  </div>\n  <div #text class=\"ui-back-text\" [ngStyle]=\"{transform: translate, webkitTransform: translate, opacity: opacity}\">\n    <ng-content></ng-content>\n  </div>\n</div>"
    }),
    __param(0, Inject(DOCUMENT)),
    __param(1, Inject(UI_ROUTER_ANIMATION_STEPS)),
    __param(2, Inject(UI_SCREEN_SCALE)),
    __param(3, Inject(UI_BACK_ICON_CLASSNAME)),
    __param(4, Inject(UI_VIEW_INIT_STATE)),
    __metadata("design:paramtypes", [Object, Number, Number, String, String, Location,
        Router,
        AppController,
        ViewStateService])
], BackComponent);

let TabViewItemService = class TabViewItemService {
    constructor() {
        this.changeEvent = new Subject();
        this.stateSource = new Subject();
        this.progressSource = new Subject();
        this.touchEvent = new Subject();
        this.activateState = this.changeEvent.asObservable();
        this.state = this.stateSource.asObservable();
        this.progress = this.progressSource.asObservable();
        this.touchProgress = this.touchEvent.asObservable();
    }
    change(state) {
        this.changeEvent.next(state);
    }
    changeState(state) {
        this.stateSource.next(state);
    }
    updateProgress(n) {
        this.progressSource.next(n);
    }
    touching(progress) {
        this.touchEvent.next(progress);
    }
};
TabViewItemService = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], TabViewItemService);

let RouterComponent = class RouterComponent {
    constructor(parentContexts, routerService, resolver, routeCacheController, viewContainerRef, location, appController, tabViewItemService, state, initTouchBack, steps) {
        this.parentContexts = parentContexts;
        this.routerService = routerService;
        this.resolver = resolver;
        this.routeCacheController = routeCacheController;
        this.viewContainerRef = viewContainerRef;
        this.location = location;
        this.appController = appController;
        this.tabViewItemService = tabViewItemService;
        this.state = state;
        this.initTouchBack = initTouchBack;
        this.steps = steps;
        /* tslint:disable */
        this.activateEvents = new EventEmitter();
        this.deactivateEvents = new EventEmitter();
        /* tslint:enable */
        this.name = PRIMARY_OUTLET;
        this.views = [];
        this.subs = [];
        this.activated = null;
        this._activatedRoute = null;
        this.isMoveBack = false;
        this.isBack = false;
    }
    get openMoveBack() {
        return this.views.length > 1 || this.initTouchBack;
    }
    get isActivated() {
        return !!this.activated;
    }
    get component() {
        if (!this.activated) {
            throw new Error('Outlet is not activated');
        }
        return this.activated.instance;
    }
    get activatedRoute() {
        if (!this.activated) {
            throw new Error('Outlet is not activated');
        }
        return this._activatedRoute;
    }
    get activatedRouteData() {
        if (this._activatedRoute) {
            return this._activatedRoute.snapshot.data;
        }
        return {};
    }
    ngOnInit() {
        this.subs.push(this.routeCacheController.hasCache.pipe(distinctUntilChanged()).subscribe(b => {
            this.isBack = !b;
        }));
        if (this.tabViewItemService) {
            this.subs.push(this.tabViewItemService.activateState.subscribe(b => {
                if (b) {
                    this.setViewState([ViewState.Activate]);
                }
                else {
                    this.setViewState([ViewState.ToStack]);
                }
            }));
            this.subs.push(this.tabViewItemService.state.subscribe(state => {
                this.setViewState([state]);
            }));
            this.subs.push(this.tabViewItemService.progress.subscribe(p => {
                this.routerService.publishAnimationProgress(p);
            }));
            this.subs.push(this.tabViewItemService.touchProgress.subscribe(p => {
                this.routerService.parentTouching(p);
            }));
        }
        this.parentContexts.onChildOutletCreated(this.name, this);
        if (!this.activated) {
            const context = this.parentContexts.getContext(this.name);
            if (context && context.route) {
                this.activateWith(context.route, context.resolver || null);
            }
        }
        // 订阅当前活动的组件
        this.subs.push(this.routerService.activated.subscribe((componentRef) => {
            this.activated = componentRef;
        }));
        const parentRouterService = this.viewContainerRef.parentInjector.get(RouterService, null);
        this.subs.push(this.routerService.moveBackProgress.subscribe(progress => {
            if (progress <= 0) {
                this.setViewState([ViewState.Sleep, ViewState.Activate]);
            }
            else {
                this.setViewState([ViewState.ToStack, ViewState.Activate]);
            }
            if (parentRouterService && this.views.length <= 1) {
                parentRouterService.publishMoveBackProgress(progress);
            }
            else if (progress >= this.steps && this.openMoveBack) {
                this.isMoveBack = true;
                this.location.back();
            }
        }));
    }
    ngOnDestroy() {
        this.parentContexts.onChildOutletDestroyed(this.name);
        this.subs.forEach(item => {
            item.unsubscribe();
        });
    }
    // 从子树分离时调用
    detach() {
        if (!this.activated) {
            throw new Error('Outlet is not activated');
        }
        const comp = this.activated;
        this._activatedRoute = null;
        this.activated = null;
        return comp;
    }
    // 将以前分离的子树重新附加时调用
    attach(ref, activatedRoute) {
        this.activated = ref;
        this._activatedRoute = activatedRoute;
        this.routeCacheController.isCacheCurrentView(true);
        if (this.isMoveBack) {
            this.views.pop();
            this.isMoveBack = false;
            this.setViewState([ViewState.Activate]);
            setTimeout(() => {
                this.routerService.publishAnimationProgress(this.steps);
            });
            return;
        }
        this.setViewState([ViewState.Reactivate, ViewState.Destroy]);
        setTimeout(() => {
            this.setupRouterAnimation();
        });
    }
    activateWith(activatedRoute, resolver) {
        this._activatedRoute = activatedRoute;
        const snapshot = activatedRoute._futureSnapshot;
        const component = snapshot.routeConfig.component;
        resolver = resolver || this.resolver;
        const childContexts = this.parentContexts.getOrCreateContext(this.name).children;
        if (this.isBack) {
            this.setViewState([ViewState.Destroy]);
            this.views.unshift({
                state: ViewState.Reactivate,
                componentFactory: resolver.resolveComponentFactory(component),
                childContexts,
                activatedRoute
            });
        }
        else {
            this.setViewState([ViewState.ToStack]);
            let state;
            if (this.state) {
                state = this.state;
                this.state = null;
            }
            else {
                state = ViewState.Activate;
            }
            this.views.push({
                state,
                componentFactory: resolver.resolveComponentFactory(component),
                childContexts,
                activatedRoute
            });
        }
        this.routeCacheController.isCacheCurrentView(true);
        this.setupRouterAnimation();
    }
    deactivate() {
        this.activated = null;
        this._activatedRoute = null;
    }
    setViewState(status) {
        let i = this.views.length - 1;
        while (i >= 0) {
            this.views[i].state = status.pop() || ViewState.Sleep;
            i--;
        }
    }
    setupRouterAnimation() {
        if (this.views.length === 1) {
            this.routerService.publishAnimationProgress(this.steps);
            return;
        }
        let i = 0;
        this.appController.transitionStart();
        const fn = () => {
            if (i > this.steps) {
                this.appController.transitionEnd();
                switch (this.views[this.views.length - 1].state) {
                    case ViewState.Destroy:
                        this.views.pop();
                        this.setViewState([ViewState.Activate]);
                        break;
                    case ViewState.Activate:
                        this.setViewState([ViewState.Sleep, ViewState.Activate]);
                        break;
                }
            }
            else {
                requestAnimationFrame(fn);
                this.routerService.publishAnimationProgress(i);
                i++;
            }
        };
        fn();
    }
};
RouterComponent.ctorParameters = () => [
    { type: ChildrenOutletContexts },
    { type: RouterService },
    { type: ComponentFactoryResolver },
    { type: RouteCacheController },
    { type: ViewContainerRef },
    { type: Location },
    { type: AppController },
    { type: TabViewItemService, decorators: [{ type: Optional }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [UI_VIEW_INIT_STATE,] }] },
    { type: Boolean, decorators: [{ type: Optional }, { type: Inject, args: [UI_VIEW_INIT_TOUCH_BACK,] }] },
    { type: Number, decorators: [{ type: Inject, args: [UI_ROUTER_ANIMATION_STEPS,] }] }
];
__decorate([
    Output('activate'),
    __metadata("design:type", Object)
], RouterComponent.prototype, "activateEvents", void 0);
__decorate([
    Output('deactivate'),
    __metadata("design:type", Object)
], RouterComponent.prototype, "deactivateEvents", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], RouterComponent.prototype, "name", void 0);
RouterComponent = __decorate([
    Component({
        selector: 'ui-router',
        template: "<ui-view\n        *ngFor=\"let item of views; let i = index\"\n        [openMoveBack]=\"openMoveBack\"\n        [componentFactory]=\"item.componentFactory\"\n        [activatedRoute]=\"item.activatedRoute\"\n        [parentContexts]=\"item.childContexts\"\n        [state]=\"item.state\"\n        [ngStyle]=\"{zIndex: i}\">\n</ui-view>\n",
        providers: [
            RouterService
        ]
    }),
    __param(7, Optional()),
    __param(8, Optional()), __param(8, Inject(UI_VIEW_INIT_STATE)),
    __param(9, Optional()), __param(9, Inject(UI_VIEW_INIT_TOUCH_BACK)),
    __param(10, Inject(UI_ROUTER_ANIMATION_STEPS)),
    __metadata("design:paramtypes", [ChildrenOutletContexts,
        RouterService,
        ComponentFactoryResolver,
        RouteCacheController,
        ViewContainerRef,
        Location,
        AppController,
        TabViewItemService, String, Boolean, Number])
], RouterComponent);

let TabComponent = class TabComponent {
};
TabComponent = __decorate([
    Component({
        selector: 'ui-tab',
        template: "<ng-content></ng-content>",
        providers: [
            TabController
        ]
    })
], TabComponent);

let TabBarItemComponent = class TabBarItemComponent {
    constructor() {
        this.uiSelected = new EventEmitter();
        this.active = false;
    }
    click() {
        this.uiSelected.emit();
    }
};
__decorate([
    Output(),
    __metadata("design:type", Object)
], TabBarItemComponent.prototype, "uiSelected", void 0);
__decorate([
    Input(),
    HostBinding('class.ui-active'),
    __metadata("design:type", Boolean)
], TabBarItemComponent.prototype, "active", void 0);
__decorate([
    HostListener('click'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], TabBarItemComponent.prototype, "click", null);
TabBarItemComponent = __decorate([
    Component({
        selector: 'ui-tab-bar-item',
        template: "<ng-content></ng-content>"
    })
], TabBarItemComponent);

let TabBarComponent = class TabBarComponent {
    constructor(tabService, viewStateService, state, steps) {
        this.tabService = tabService;
        this.viewStateService = viewStateService;
        this.state = state;
        this.steps = steps;
        this.tabIndex = 0;
        this.subs = [];
    }
    ngOnInit() {
        const steps = this.steps;
        const bezier = new CubicBezier(.36, .66, .04, 1);
        this.subs.push(this.viewStateService.state.subscribe(state => {
            this.state = state;
        }));
        this.subs.push(this.viewStateService.touchProgress.subscribe(p => {
            if (this.state === ViewState.Activate || this.state === ViewState.Reactivate) {
                this.translate = `translate3d(${p / steps * 100}%, 0, 0)`;
            }
            else if (this.state === ViewState.ToStack) {
                this.translate = `translate3d(${-33 + 33 * p / steps}%, 0, 0)`;
                this.opacity = 0.9 + 0.1 * p / steps;
            }
        }));
        this.subs.push(this.viewStateService.progress.subscribe((p) => {
            const progress = bezier.update(p / steps);
            switch (this.state) {
                case ViewState.Activate:
                    this.opacity = 1;
                    this.translate = `translate3d(${100 - progress * 100}%, 0, 0)`;
                    break;
                case ViewState.Destroy:
                    this.opacity = 1;
                    this.translate = `translate3d(${progress * 100}%, 0, 0)`;
                    break;
                case ViewState.ToStack:
                    this.translate = `translate3d(${progress * -33}%, 0, 0)`;
                    this.opacity = 1 - 0.1 * p / steps;
                    break;
                case ViewState.Reactivate:
                    const n = -33 + progress * 33;
                    // 当dom元素的style有transform属性时，会导致子级元素 position: fixed 全屏失效
                    // 会跟着有定位的父级同样大小
                    this.translate = n === 0 ? '' : `translate3d(${n}%, 0, 0)`;
                    this.opacity = 0.9 + 0.1 * progress;
                    break;
            }
        }));
    }
    ngAfterContentInit() {
        // 当用户点击或选中某一个按扭时，发布相应事件
        this.tabBarItems.forEach((item, index) => {
            const sub = item.uiSelected.subscribe(() => {
                this.tabService.publishIndex(index);
            });
            this.subs.push(sub);
        });
        // 当用户切换tab时，显示/隐藏对应视图
        this.subs.push(this.tabService.tabIndex.subscribe((index) => {
            this.tabBarItems.forEach((item, i) => {
                item.active = i === index;
            });
        }));
        this.tabService.publishIndex(this.tabIndex);
    }
    ngOnDestroy() {
        this.subs.forEach(item => {
            item.unsubscribe();
        });
    }
};
TabBarComponent.ctorParameters = () => [
    { type: TabController },
    { type: ViewStateService },
    { type: undefined, decorators: [{ type: Inject, args: [UI_VIEW_INIT_STATE,] }] },
    { type: Number, decorators: [{ type: Inject, args: [UI_ROUTER_ANIMATION_STEPS,] }] }
];
__decorate([
    HostBinding('style.transform'),
    HostBinding('style.webkitTransform'),
    __metadata("design:type", String)
], TabBarComponent.prototype, "translate", void 0);
__decorate([
    HostBinding('style.opacity'),
    __metadata("design:type", Number)
], TabBarComponent.prototype, "opacity", void 0);
__decorate([
    ContentChildren(TabBarItemComponent),
    __metadata("design:type", QueryList)
], TabBarComponent.prototype, "tabBarItems", void 0);
__decorate([
    Input(),
    __metadata("design:type", Number)
], TabBarComponent.prototype, "tabIndex", void 0);
TabBarComponent = __decorate([
    Component({
        selector: 'ui-tab-bar',
        template: "<div class=\"ui-tab-bar-shadow\"></div>\n<div class=\"ui-tab-bar\">\n  <ng-content></ng-content>\n</div>"
    }),
    __param(2, Inject(UI_VIEW_INIT_STATE)),
    __param(3, Inject(UI_ROUTER_ANIMATION_STEPS)),
    __metadata("design:paramtypes", [TabController,
        ViewStateService, String, Number])
], TabBarComponent);

let TabViewItemComponent = class TabViewItemComponent {
    constructor(changeDetectorRef, viewStateService, tabViewItemService) {
        this.changeDetectorRef = changeDetectorRef;
        this.viewStateService = viewStateService;
        this.tabViewItemService = tabViewItemService;
        this._active = false;
        this.isInit = false;
        this.subs = [];
    }
    set active(value) {
        this._active = value;
        this.tabViewItemService.change(value);
        if (value) {
            this.isInit = true;
            this.changeDetectorRef.reattach();
        }
        else {
            this.changeDetectorRef.detach();
            if (this.isInit) {
                this.changeDetectorRef.detectChanges();
            }
        }
    }
    get active() {
        return this._active;
    }
    ngOnInit() {
        this.subs.push(this.viewStateService.state.subscribe(state => {
            if (this.active) {
                this.tabViewItemService.changeState(state);
            }
        }));
        this.subs.push(this.viewStateService.progress.subscribe(p => {
            if (this.active) {
                this.tabViewItemService.updateProgress(p);
            }
        }));
        this.subs.push(this.viewStateService.touchProgress.subscribe(p => {
            if (this.active) {
                this.tabViewItemService.touching(p);
            }
        }));
    }
    ngOnDestroy() {
        this.subs.forEach(item => item.unsubscribe());
    }
};
TabViewItemComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: ViewStateService },
    { type: TabViewItemService }
];
__decorate([
    Input(),
    __metadata("design:type", String)
], TabViewItemComponent.prototype, "name", void 0);
__decorate([
    HostBinding('class.ui-active'),
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [Boolean])
], TabViewItemComponent.prototype, "active", null);
TabViewItemComponent = __decorate([
    Component({
        selector: 'ui-tab-view-item',
        template: "<ng-content></ng-content>\n<ui-router [name]=\"name\"></ui-router>",
        providers: [
            TabViewItemService
        ]
    }),
    __metadata("design:paramtypes", [ChangeDetectorRef,
        ViewStateService,
        TabViewItemService])
], TabViewItemComponent);

let TabViewComponent = class TabViewComponent {
    constructor(tabService) {
        this.tabService = tabService;
        this.subs = [];
    }
    ngAfterContentInit() {
        // 订阅tab切换事件，如果发生切换，显示/隐藏对应视图
        this.subs.push(this.tabService.tabIndex.subscribe((index) => {
            this.tabViewItems.forEach((item, i) => {
                item.active = i === index;
            });
        }));
        // 初始化时，默认为第一项
        const items = this.tabViewItems.toArray();
        if (items[0]) {
            items[0].active = true;
        }
    }
    ngOnDestroy() {
        this.subs.forEach(item => item.unsubscribe());
    }
};
TabViewComponent.ctorParameters = () => [
    { type: TabController }
];
__decorate([
    ContentChildren(TabViewItemComponent),
    __metadata("design:type", QueryList)
], TabViewComponent.prototype, "tabViewItems", void 0);
TabViewComponent = __decorate([
    Component({
        selector: 'ui-tab-view',
        template: "<ng-content></ng-content>",
        providers: []
    }),
    __metadata("design:paramtypes", [TabController])
], TabViewComponent);

let ComponentHostDirective = class ComponentHostDirective {
    constructor(viewContainerRef) {
        this.viewContainerRef = viewContainerRef;
    }
};
ComponentHostDirective.ctorParameters = () => [
    { type: ViewContainerRef }
];
ComponentHostDirective = __decorate([
    Directive({
        selector: '[uiComponentHost]'
    }),
    __metadata("design:paramtypes", [ViewContainerRef])
], ComponentHostDirective);

let ActionSheetService = class ActionSheetService {
    constructor() {
        this.showEvent = new Subject();
        this.onShow = this.showEvent.asObservable();
    }
    push(el) {
        this.showEvent.next(el);
    }
};
ActionSheetService = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], ActionSheetService);

let ContentLoadingController = class ContentLoadingController {
    constructor() {
        this.showLoadingSource = new Subject();
        this.hideLoadingSource = new Subject();
        this.showLoading = this.showLoadingSource.asObservable();
        this.hideLoading = this.hideLoadingSource.asObservable();
    }
    show(text) {
        this.showLoadingSource.next(text);
    }
    hide() {
        this.hideLoadingSource.next();
    }
};
ContentLoadingController = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], ContentLoadingController);

let ActionSheetComponent = class ActionSheetComponent {
    constructor(actionSheetService, elementRef) {
        this.actionSheetService = actionSheetService;
        this.elementRef = elementRef;
        this.isPopUp = false;
        this._show = false;
        this.timer = null;
    }
    set show(value) {
        clearTimeout(this.timer);
        if (value) {
            this._show = value;
            this.timer = setTimeout(() => {
                this.isPopUp = value;
            }, 20);
        }
        else {
            this.isPopUp = value;
            this.timer = setTimeout(() => {
                this._show = value;
            }, 300);
        }
    }
    get show() {
        return this._show;
    }
    ngOnInit() {
        this.actionSheetService.push(this.elementRef.nativeElement);
    }
};
ActionSheetComponent.ctorParameters = () => [
    { type: ActionSheetService },
    { type: ElementRef }
];
__decorate([
    Input(),
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [Boolean])
], ActionSheetComponent.prototype, "show", null);
ActionSheetComponent = __decorate([
    Component({
        selector: 'ui-action-sheet',
        template: "<ui-mask [show]=\"isPopUp\">\n  <div class=\"ui-action-sheet\" [ngClass]=\"{'ui-show': show, 'ui-pop-up': isPopUp}\">\n    <ng-content></ng-content>\n  </div>\n</ui-mask>\n"
    }),
    __metadata("design:paramtypes", [ActionSheetService,
        ElementRef])
], ActionSheetComponent);

let ContentLoadingComponent = class ContentLoadingComponent {
    constructor(contentLoadingController) {
        this.contentLoadingController = contentLoadingController;
        this.show = false;
        this.text = '';
        this.fadeIn = false;
        this.fadeOut = false;
        this.subs = [];
        this.timer = null;
    }
    ngOnInit() {
        this.subs.push(this.contentLoadingController.showLoading.subscribe((text) => {
            this.fadeOut = false;
            this.fadeIn = true;
            clearTimeout(this.timer);
            this.timer = setTimeout(() => {
                this.show = true;
                this.text = text;
            });
        }));
        this.subs.push(this.contentLoadingController.hideLoading.subscribe(() => {
            this.fadeIn = false;
            this.fadeOut = true;
            clearTimeout(this.timer);
            this.timer = setTimeout(() => {
                this.show = false;
                this.text = '';
            }, 150);
        }));
    }
    ngOnDestroy() {
        clearTimeout(this.timer);
        this.subs.forEach(item => {
            item.unsubscribe();
        });
    }
};
ContentLoadingComponent.ctorParameters = () => [
    { type: ContentLoadingController }
];
__decorate([
    HostBinding('class.ui-content-loading-fadein'),
    __metadata("design:type", Boolean)
], ContentLoadingComponent.prototype, "fadeIn", void 0);
__decorate([
    HostBinding('class.ui-content-loading-fadeout'),
    __metadata("design:type", Boolean)
], ContentLoadingComponent.prototype, "fadeOut", void 0);
ContentLoadingComponent = __decorate([
    Component({
        selector: 'ui-content-loading',
        template: "<div class=\"ui-content-loading\" *ngIf=\"show\">\n  <ui-loading color=\"#fff\"></ui-loading>\n  <p class=\"ui-content-loading-text\">{{ text || '\u52A0\u8F7D\u4E2D\u2026' }}</p>\n</div>",
        animations: [
            trigger('loadingAnimate', [transition(':enter', animate('5s', keyframes([
                    style({
                        opacity: 0,
                        offset: 0
                    }),
                    style({
                        opacity: 1,
                        offset: 1
                    })
                ]))), transition(':leave', animate('0.15s', keyframes([
                    style({
                        opacity: 1,
                        offset: 0
                    }),
                    style({
                        opacity: 0,
                        offset: 1
                    })
                ])))])
        ]
    }),
    __metadata("design:paramtypes", [ContentLoadingController])
], ContentLoadingComponent);

let FixedBarComponent = class FixedBarComponent {
};
FixedBarComponent = __decorate([
    Component({
        selector: 'ui-fixed-bar',
        template: "<ng-content></ng-content>"
    })
], FixedBarComponent);

let LoadingComponent = class LoadingComponent {
    constructor() {
        this.scale = 1;
        this.color = '';
    }
    get size() {
        return `scale(${this.scale})`;
    }
};
__decorate([
    Input(),
    __metadata("design:type", Number)
], LoadingComponent.prototype, "scale", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], LoadingComponent.prototype, "color", void 0);
__decorate([
    HostBinding('style.transform'),
    HostBinding('style.webkitTransform'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], LoadingComponent.prototype, "size", null);
LoadingComponent = __decorate([
    Component({
        selector: 'ui-loading',
        template: "<div class=\"ui-circle-1\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-2\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-3\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-4\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-5\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-6\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-7\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-8\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-9\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-10\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-11\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-12\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>"
    })
], LoadingComponent);

let StopPropagationDirective = class StopPropagationDirective {
    constructor(elementRef, renderer) {
        this.elementRef = elementRef;
        this.renderer = renderer;
        this.callbacks = [];
    }
    ngOnInit() {
        if (typeof this.uiStopPropagation === 'string') {
            this.addEvent(this.uiStopPropagation || 'click');
        }
        else {
            this.uiStopPropagation.forEach(eventType => {
                this.addEvent(eventType);
            });
        }
    }
    ngOnDestroy() {
        this.callbacks.forEach(fn => {
            fn();
        });
    }
    addEvent(eventType) {
        const fn = this.renderer.listen(this.elementRef.nativeElement, eventType, (ev) => {
            ev.stopPropagation();
        });
        this.callbacks.push(fn);
    }
};
StopPropagationDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], StopPropagationDirective.prototype, "uiStopPropagation", void 0);
StopPropagationDirective = __decorate([
    Directive({
        selector: '[uiStopPropagation]'
    }),
    __metadata("design:paramtypes", [ElementRef, Renderer2])
], StopPropagationDirective);

class ViewInjector {
    constructor(route, childContexts, initState, isOpenTouchBack, parent) {
        this.route = route;
        this.childContexts = childContexts;
        this.initState = initState;
        this.isOpenTouchBack = isOpenTouchBack;
        this.parent = parent;
    }
    get(token, notFoundValue) {
        if (token === ActivatedRoute) {
            return this.route;
        }
        if (token === ChildrenOutletContexts) {
            return this.childContexts;
        }
        if (token === UI_VIEW_INIT_STATE) {
            return this.initState;
        }
        if (token === UI_VIEW_INIT_TOUCH_BACK) {
            return this.isOpenTouchBack;
        }
        return this.parent.get(token, notFoundValue);
    }
}
let ViewComponent = class ViewComponent {
    constructor(viewStateService, viewContainerRef, _activatedRoute, changeDetectorRef, actionSheetService, elementRef, routerService) {
        this.viewStateService = viewStateService;
        this.viewContainerRef = viewContainerRef;
        this._activatedRoute = _activatedRoute;
        this.changeDetectorRef = changeDetectorRef;
        this.actionSheetService = actionSheetService;
        this.elementRef = elementRef;
        this.routerService = routerService;
        this.openMoveBack = false;
        this.subs = [];
    }
    set state(value) {
        if (value === this._state) {
            return;
        }
        // 根据视图不同状态，调用生命周期勾子
        this.viewStateService.changeState(value);
        if (this.childInstance) {
            switch (value) {
                case ViewState.Activate:
                case ViewState.Reactivate:
                    if (this._state !== ViewState.Activate && this._state !== ViewState.Reactivate) {
                        if (typeof this.childInstance['uiOnViewEnter'] === 'function') {
                            this.childInstance['uiOnViewEnter']();
                        }
                        this.routerService.publish(this.componentRef);
                    }
                    break;
                case ViewState.ToStack:
                case ViewState.Destroy:
                    if (typeof this.childInstance['uiOnViewLeave'] === 'function') {
                        this.childInstance['uiOnViewLeave']();
                    }
                    break;
            }
            if (value === ViewState.Sleep) {
                this.changeDetectorRef.detach();
            }
            else {
                this.changeDetectorRef.reattach();
            }
        }
        this._state = value;
    }
    get state() {
        return this._state;
    }
    get sleep() {
        return this.state === ViewState.Sleep;
    }
    ngOnInit() {
        if (this.componentFactory) {
            const injector = new ViewInjector(this.activatedRoute || this._activatedRoute, this.parentContexts, this.state, this.openMoveBack, this.viewContainerRef.injector);
            this.componentRef = this.componentHost.viewContainerRef.createComponent(this.componentFactory, this.componentHost.viewContainerRef.length, injector);
            this.childInstance = this.componentRef.instance;
            this.routerService.publish(this.componentRef);
            if (typeof this.childInstance['uiOnViewEnter'] === 'function') {
                this.childInstance['uiOnViewEnter']();
            }
        }
        this.subs.push(this.routerService.animationProgress.subscribe(progress => {
            if (this.state !== ViewState.Sleep) {
                this.viewStateService.updateProgress(progress);
            }
        }));
        this.subs.push(this.routerService.moveBackProgress.subscribe(progress => {
            if (this.state !== ViewState.Sleep && this.openMoveBack) {
                this.viewStateService.touching(progress);
            }
        }));
        this.subs.push(this.routerService.parentTouchProgress.subscribe(progress => {
            if (this.state !== ViewState.Sleep) {
                this.viewStateService.touching(progress);
            }
        }));
        this.subs.push(this.actionSheetService.onShow.subscribe(el => {
            this.elementRef.nativeElement.appendChild(el);
        }));
    }
    ngOnDestroy() {
        this.subs.forEach(item => {
            item.unsubscribe();
        });
    }
};
ViewComponent.ctorParameters = () => [
    { type: ViewStateService },
    { type: ViewContainerRef },
    { type: ActivatedRoute },
    { type: ChangeDetectorRef },
    { type: ActionSheetService },
    { type: ElementRef },
    { type: RouterService }
];
__decorate([
    ViewChild(ComponentHostDirective, { static: true }),
    __metadata("design:type", ComponentHostDirective)
], ViewComponent.prototype, "componentHost", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], ViewComponent.prototype, "componentFactory", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], ViewComponent.prototype, "openMoveBack", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], ViewComponent.prototype, "parentContexts", void 0);
__decorate([
    Input(),
    __metadata("design:type", ActivatedRoute)
], ViewComponent.prototype, "activatedRoute", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], ViewComponent.prototype, "state", null);
__decorate([
    HostBinding('class.ui-sleep'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ViewComponent.prototype, "sleep", null);
ViewComponent = __decorate([
    Component({
        selector: 'ui-view',
        template: "<ng-template uiComponentHost></ng-template>\n<ui-content-loading></ui-content-loading>",
        providers: [
            ViewStateService,
            ContentLoadingController,
            PullDownRefreshController,
            PullUpLoadController,
            ActionSheetService
        ]
    }),
    __metadata("design:paramtypes", [ViewStateService,
        ViewContainerRef,
        ActivatedRoute,
        ChangeDetectorRef,
        ActionSheetService,
        ElementRef,
        RouterService])
], ViewComponent);

let ContentComponent = class ContentComponent {
    constructor(viewStateService, elementRef, routerService, state, scale, renderer, steps) {
        this.viewStateService = viewStateService;
        this.elementRef = elementRef;
        this.routerService = routerService;
        this.state = state;
        this.scale = scale;
        this.renderer = renderer;
        this.steps = steps;
        this.subs = [];
        this.distanceX = 0;
    }
    ngOnInit() {
        const steps = this.steps;
        const bezier = new CubicBezier(.36, .66, .04, 1);
        this.subs.push(this.viewStateService.state.subscribe(state => {
            this.state = state;
        }));
        this.subs.push(this.viewStateService.touchProgress.subscribe(p => {
            if (this.state === ViewState.Activate || this.state === ViewState.Reactivate) {
                this.translate = `translate3d(${p / steps * 100}%, 0, 0)`;
            }
            else if (this.state === ViewState.ToStack) {
                this.translate = `translate3d(${-33 + 33 * p / steps}%, 0, 0)`;
                this.opacity = 0.9 + 0.1 * p / steps;
            }
        }));
        this.subs.push(this.viewStateService.progress.subscribe((p) => {
            const progress = bezier.update(p / steps);
            switch (this.state) {
                case ViewState.Activate:
                    this.opacity = 1;
                    this.translate = `translate3d(${100 - progress * 100}%, 0, 0)`;
                    break;
                case ViewState.Destroy:
                    this.opacity = 1;
                    this.translate = `translate3d(${progress * 100}%, 0, 0)`;
                    break;
                case ViewState.ToStack:
                    this.translate = `translate3d(${progress * -33}%, 0, 0)`;
                    this.opacity = 1 - 0.1 * p / steps;
                    break;
                case ViewState.Reactivate:
                    const n = -33 + progress * 33;
                    // 当dom元素的style有transform属性时，会导致子级元素 position: fixed 全屏失效
                    // 会跟着有定位的父级同样大小
                    this.translate = n === 0 ? '' : `translate3d(${n}%, 0, 0)`;
                    this.opacity = 0.9 + 0.1 * progress;
                    break;
            }
        }));
    }
    ngOnDestroy() {
        this.subs.forEach(item => item.unsubscribe());
    }
    touchStart(event) {
        if (navigator.userAgent.indexOf('web-2.0') > -1) {
            return;
        }
        const startPoint = event.touches[0];
        const startX = startPoint.pageX;
        const startY = startPoint.pageY;
        if (startX > 50 * this.scale && this.state !== ViewState.Sleep) {
            return;
        }
        cancelAnimationFrame(this.animationId);
        let unbindTouchMoveFn;
        let unbindTouchEndFn;
        let unbindTouchCancelFn;
        let unbindFn;
        const element = this.elementRef.nativeElement;
        const maxWidth = element.offsetWidth;
        const startTime = Date.now();
        this.maxWidth = maxWidth;
        let isBack = false;
        let progress = 0;
        unbindTouchMoveFn = this.renderer.listen(element, 'touchmove', (moveEvent) => {
            const movePoint = moveEvent.touches[0];
            const moveX = movePoint.pageX;
            const moveY = movePoint.pageY;
            const distanceY = moveY - startY;
            let distanceX = moveX - startX;
            if (distanceX < distanceY && !isBack) {
                unbindFn();
                return;
            }
            isBack = true;
            if (distanceX < 0) {
                distanceX = 0;
            }
            else if (distanceX > maxWidth) {
                distanceX = maxWidth;
            }
            this.distanceX = Math.floor(distanceX);
            progress = distanceX / maxWidth * this.steps;
            this.routerService.publishMoveBackProgress(progress);
        });
        unbindFn = () => {
            unbindTouchMoveFn();
            unbindTouchEndFn();
            unbindTouchCancelFn();
            if (progress === 0) {
                return;
            }
            const endTime = Date.now();
            if (endTime - startTime < 100 && this.distanceX > 50 * this.scale) {
                this.animationTo(maxWidth);
                return;
            }
            if (this.distanceX < maxWidth / 3) {
                this.animationTo(0);
            }
            else {
                this.animationTo(maxWidth);
            }
        };
        unbindTouchEndFn = this.renderer.listen(element, 'touchend', unbindFn);
        unbindTouchCancelFn = this.renderer.listen(element, 'touchcancel', unbindFn);
    }
    animationTo(target) {
        const l = target - this.distanceX;
        const oldDistanceX = this.distanceX;
        let i = 0;
        const fn = () => {
            i++;
            this.distanceX = oldDistanceX + l * Easing.Cubic.Out(i / 10);
            this.routerService.publishMoveBackProgress(this.distanceX / this.maxWidth * this.steps);
            if (i < 10) {
                this.animationId = requestAnimationFrame(fn);
            }
        };
        this.animationId = requestAnimationFrame(fn);
    }
};
ContentComponent.ctorParameters = () => [
    { type: ViewStateService },
    { type: ElementRef },
    { type: RouterService },
    { type: undefined, decorators: [{ type: Inject, args: [UI_VIEW_INIT_STATE,] }] },
    { type: Number, decorators: [{ type: Inject, args: [UI_SCREEN_SCALE,] }] },
    { type: Renderer2 },
    { type: Number, decorators: [{ type: Inject, args: [UI_ROUTER_ANIMATION_STEPS,] }] }
];
__decorate([
    HostBinding('style.transform'),
    HostBinding('style.webkitTransform'),
    __metadata("design:type", String)
], ContentComponent.prototype, "translate", void 0);
__decorate([
    HostBinding('style.opacity'),
    __metadata("design:type", Number)
], ContentComponent.prototype, "opacity", void 0);
__decorate([
    HostListener('touchstart', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], ContentComponent.prototype, "touchStart", null);
ContentComponent = __decorate([
    Component({
        selector: 'ui-content',
        template: "<div class=\"ui-content-shadow\"></div>\n<div class=\"ui-content\">\n  <ng-content></ng-content>\n</div>\n"
    }),
    __param(3, Inject(UI_VIEW_INIT_STATE)),
    __param(4, Inject(UI_SCREEN_SCALE)),
    __param(6, Inject(UI_ROUTER_ANIMATION_STEPS)),
    __metadata("design:paramtypes", [ViewStateService,
        ElementRef,
        RouterService, String, Number, Renderer2, Number])
], ContentComponent);

let FooterComponent = class FooterComponent {
    constructor(viewStateService, state, steps) {
        this.viewStateService = viewStateService;
        this.state = state;
        this.steps = steps;
        this.subs = [];
    }
    ngOnInit() {
        const steps = this.steps;
        const bezier = new CubicBezier(.36, .66, .04, 1);
        this.subs.push(this.viewStateService.state.subscribe(state => {
            this.state = state;
        }));
        this.subs.push(this.viewStateService.touchProgress.subscribe(p => {
            if (this.state === ViewState.Activate || this.state === ViewState.Reactivate) {
                this.translate = `translate3d(${p / steps * 100}%, 0, 0)`;
            }
            else if (this.state === ViewState.ToStack) {
                this.translate = `translate3d(${-33 + 33 * p / steps}%, 0, 0)`;
                this.opacity = 0.9 + 0.1 * p / steps;
            }
        }));
        this.subs.push(this.viewStateService.progress.subscribe((p) => {
            const progress = bezier.update(p / steps);
            switch (this.state) {
                case ViewState.Activate:
                    this.opacity = 1;
                    this.translate = `translate3d(${100 - progress * 100}%, 0, 0)`;
                    break;
                case ViewState.Destroy:
                    this.opacity = 1;
                    this.translate = `translate3d(${progress * 100}%, 0, 0)`;
                    break;
                case ViewState.ToStack:
                    this.translate = `translate3d(${progress * -33}%, 0, 0)`;
                    this.opacity = 1 - 0.1 * p / steps;
                    break;
                case ViewState.Reactivate:
                    const n = -33 + progress * 33;
                    // 当dom元素的style有transform属性时，会导致子级元素 position: fixed 全屏失效
                    // 会跟着有定位的父级同样大小
                    this.translate = n === 0 ? '' : `translate3d(${n}%, 0, 0)`;
                    this.opacity = 0.9 + 0.1 * progress;
                    break;
            }
        }));
    }
    ngOnDestroy() {
        this.subs.forEach(item => item.unsubscribe());
    }
};
FooterComponent.ctorParameters = () => [
    { type: ViewStateService },
    { type: undefined, decorators: [{ type: Inject, args: [UI_VIEW_INIT_STATE,] }] },
    { type: Number, decorators: [{ type: Inject, args: [UI_ROUTER_ANIMATION_STEPS,] }] }
];
__decorate([
    HostBinding('style.transform'),
    HostBinding('style.webkitTransform'),
    __metadata("design:type", String)
], FooterComponent.prototype, "translate", void 0);
__decorate([
    HostBinding('style.opacity'),
    __metadata("design:type", Number)
], FooterComponent.prototype, "opacity", void 0);
FooterComponent = __decorate([
    Component({
        selector: 'ui-footer',
        template: "<div class=\"ui-footer-shadow\"></div>\n<div class=\"ui-footer\">\n  <ng-content></ng-content>\n</div>"
    }),
    __param(1, Inject(UI_VIEW_INIT_STATE)),
    __param(2, Inject(UI_ROUTER_ANIMATION_STEPS)),
    __metadata("design:paramtypes", [ViewStateService, String, Number])
], FooterComponent);

let HeaderComponent = class HeaderComponent {
    constructor(viewStateService, state, steps) {
        this.viewStateService = viewStateService;
        this.state = state;
        this.steps = steps;
        this.subs = [];
    }
    ngOnInit() {
        const steps = this.steps;
        this.subs.push(this.viewStateService.state.subscribe(state => {
            this.state = state;
        }));
        this.subs.push(this.viewStateService.touchProgress.subscribe(p => {
            if (this.state === ViewState.Activate || this.state === ViewState.Reactivate) {
                this.opacity = 1 - p / steps;
            }
        }));
        this.subs.push(this.viewStateService.progress.subscribe((p) => {
            switch (this.state) {
                case ViewState.Activate:
                    this.opacity = p / steps;
                    break;
                case ViewState.Destroy:
                    this.opacity = 1 - p / steps;
                    break;
                case ViewState.ToStack:
                case ViewState.Reactivate:
                    this.opacity = 1;
                    break;
            }
        }));
    }
    ngOnDestroy() {
        this.subs.forEach(item => item.unsubscribe());
    }
};
HeaderComponent.ctorParameters = () => [
    { type: ViewStateService },
    { type: undefined, decorators: [{ type: Inject, args: [UI_VIEW_INIT_STATE,] }] },
    { type: Number, decorators: [{ type: Inject, args: [UI_ROUTER_ANIMATION_STEPS,] }] }
];
__decorate([
    HostBinding('style.opacity'),
    __metadata("design:type", Number)
], HeaderComponent.prototype, "opacity", void 0);
HeaderComponent = __decorate([
    Component({
        selector: 'ui-header',
        template: "<ng-content></ng-content>\n"
    }),
    __param(1, Inject(UI_VIEW_INIT_STATE)),
    __param(2, Inject(UI_ROUTER_ANIMATION_STEPS)),
    __metadata("design:paramtypes", [ViewStateService, String, Number])
], HeaderComponent);

let NavBarComponent = class NavBarComponent {
};
NavBarComponent = __decorate([
    Component({
        selector: 'ui-navbar',
        template: "<ng-content></ng-content>"
    })
], NavBarComponent);

let PageComponent = class PageComponent {
};
PageComponent = __decorate([
    Component({
        selector: 'ui-page',
        template: "<ng-content></ng-content>"
    })
], PageComponent);

let TitleComponent = class TitleComponent {
    constructor(steps, state, elementRef, viewStateService) {
        this.steps = steps;
        this.state = state;
        this.elementRef = elementRef;
        this.viewStateService = viewStateService;
        this.subs = [];
        this.contentWidth = 0;
    }
    resize() {
        this.docWidth = this.elementRef.nativeElement.offsetWidth;
        this.translateDistance = this.docWidth / 2 - this.contentWidth / 2;
    }
    ngOnInit() {
        const steps = this.steps;
        const bezier = new CubicBezier(.36, .66, .04, 1);
        this.subs.push(this.viewStateService.state.subscribe(state => {
            this.state = state;
        }));
        this.subs.push(this.viewStateService.touchProgress.subscribe(p => {
            let translate;
            if (this.state === ViewState.Activate || this.state === ViewState.Reactivate) {
                this.translate = `translate3d(${p / steps * 100 * 0.7}%, 0, 0)`;
            }
            else if (this.state === ViewState.ToStack) {
                translate = -this.translateDistance + this.translateDistance * p / steps;
                this.translate = `translate3d(${translate}px, 0, 0)`;
                this.opacity = 0.9 + 0.1 * p / steps;
            }
        }));
        this.subs.push(this.viewStateService.progress.subscribe((p) => {
            const progress = bezier.update(p / steps);
            let n;
            let translate;
            switch (this.state) {
                case ViewState.Activate:
                    this.opacity = 1;
                    this.translate = `translate3d(${70 - progress * 70}%, 0, 0)`;
                    break;
                case ViewState.Destroy:
                    this.opacity = 1;
                    this.translate = `translate3d(${progress * 70}%, 0, 0)`;
                    break;
                case ViewState.ToStack:
                    this.translate = `translate3d(${-progress * this.translateDistance}px, 0, 0)`;
                    n = 1 - progress * 1.3;
                    this.opacity = n < 0 ? 0 : n;
                    break;
                case ViewState.Reactivate:
                    translate = -this.translateDistance + this.translateDistance * progress;
                    this.translate = `translate3d(${translate}px, 0, 0)`;
                    n = progress * 2;
                    this.opacity = n > 1 ? 1 : n;
                    break;
            }
        }));
    }
    ngAfterViewInit() {
        this.docWidth = this.elementRef.nativeElement.offsetWidth;
        this.contentWidth = this.contentElement.nativeElement.offsetWidth;
        this.translateDistance = this.docWidth / 2 - this.contentWidth / 2;
    }
    ngOnDestroy() {
        this.subs.forEach(item => item.unsubscribe());
    }
};
TitleComponent.ctorParameters = () => [
    { type: Number, decorators: [{ type: Inject, args: [UI_ROUTER_ANIMATION_STEPS,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [UI_VIEW_INIT_STATE,] }] },
    { type: ElementRef },
    { type: ViewStateService }
];
__decorate([
    HostBinding('style.transform'),
    HostBinding('style.webkitTransform'),
    __metadata("design:type", String)
], TitleComponent.prototype, "translate", void 0);
__decorate([
    HostBinding('style.opacity'),
    __metadata("design:type", Number)
], TitleComponent.prototype, "opacity", void 0);
__decorate([
    ViewChild('content', { static: true }),
    __metadata("design:type", ElementRef)
], TitleComponent.prototype, "contentElement", void 0);
__decorate([
    HostListener('window:resize'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], TitleComponent.prototype, "resize", null);
TitleComponent = __decorate([
    Component({
        selector: 'ui-title',
        template: "<div #content>\n  <ng-content></ng-content>\n</div>"
    }),
    __param(0, Inject(UI_ROUTER_ANIMATION_STEPS)),
    __param(1, Inject(UI_VIEW_INIT_STATE)),
    __metadata("design:paramtypes", [Number, String, ElementRef,
        ViewStateService])
], TitleComponent);

let ListActivatedService = class ListActivatedService {
    constructor() {
        this.activatedComponentSource = new Subject();
        this.activatedComponent = this.activatedComponentSource.asObservable();
    }
    publish() {
        this.activatedComponentSource.next();
    }
};
ListActivatedService = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], ListActivatedService);

let ListEventService = class ListEventService {
    constructor() {
        this.listOptionsSource = new Subject();
        this.listOptions = this.listOptionsSource.asObservable();
    }
    addOption(elementRef) {
        this.listOptionsSource.next(elementRef);
    }
};
ListEventService = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], ListEventService);

let ListComponent = class ListComponent {
};
ListComponent = __decorate([
    Component({
        selector: 'ui-list',
        template: "<ng-content></ng-content>"
    })
], ListComponent);

let ListHeaderComponent = class ListHeaderComponent {
};
ListHeaderComponent = __decorate([
    Component({
        selector: 'ui-list-header',
        template: "<ng-content></ng-content>"
    })
], ListHeaderComponent);

let ListInnerComponent = class ListInnerComponent {
};
ListInnerComponent = __decorate([
    Component({
        selector: 'ui-list-inner',
        template: "<ng-content></ng-content>"
    })
], ListInnerComponent);

let ListItemComponent = class ListItemComponent {
};
ListItemComponent = __decorate([
    Component({
        selector: 'ui-list-item',
        template: "<ng-content></ng-content>",
        providers: [
            // 提供一个服务，供 list-sliding 组件发起通信
            ListEventService
        ]
    })
], ListItemComponent);

let ListOptionComponent = class ListOptionComponent {
    constructor(listEventService, elementRef) {
        this.listEventService = listEventService;
        this.elementRef = elementRef;
    }
    ngAfterViewInit() {
        // 发布当前组件的 dom 元素，以便让 list-sliding 组件在划动时，计算最大划动宽度
        this.listEventService.addOption(this.elementRef);
    }
};
ListOptionComponent.ctorParameters = () => [
    { type: ListEventService },
    { type: ElementRef }
];
ListOptionComponent = __decorate([
    Component({
        selector: 'ui-list-option',
        template: "<ng-content></ng-content>"
    }),
    __metadata("design:paramtypes", [ListEventService,
        ElementRef])
], ListOptionComponent);

let ListSlidingComponent = class ListSlidingComponent {
    constructor(listEventService, scale, listActivatedService, renderer) {
        this.listEventService = listEventService;
        this.scale = scale;
        this.listActivatedService = listActivatedService;
        this.renderer = renderer;
        this.subs = [];
        this.distanceX = 0;
        this.refs = [];
        this.isFocus = false;
    }
    ngOnInit() {
        // 拿到当前 list-item 组件下，所有的 list-options 组件，以便在划动时，计算 dom 的宽度
        this.subs.push(this.listEventService.listOptions.subscribe((elementRef) => {
            this.refs.push(elementRef);
        }));
        // 订阅选中事件，并判断用户操作的 list-sliding 是不是自身，如果不是自身，则修改自身相应样式
        this.subs.push(this.listActivatedService.activatedComponent.subscribe(() => {
            if (!this.isFocus) {
                this.distanceX = 0;
                this.transitionDuration = '';
                this.transform = 'translateX(0px)';
            }
        }));
    }
    ngOnDestroy() {
        this.subs.forEach(item => {
            item.unsubscribe();
        });
    }
    touchstart(event) {
        this.isFocus = true;
        const touchPoint = event.touches[0];
        const startX = touchPoint.pageX;
        const startY = touchPoint.pageY;
        const oldDistanceX = this.distanceX;
        const startTime = Date.now();
        let maxDistance = 0;
        this.refs.forEach((item) => {
            maxDistance += item.nativeElement.offsetWidth;
        });
        let isClick = true;
        let isScroll = true;
        // 设置当前元素的 css 动画时间为0，防止有迟滞感
        this.transitionDuration = '0s';
        let unBindTouchEndFn;
        let unBindTouchCancelFn;
        const unBindTouchMoveFn = this.renderer.listen('document', 'touchmove', (moveEvent) => {
            isClick = false;
            const newTouchPoint = moveEvent.touches[0];
            const moveX = newTouchPoint.pageX;
            const moveY = newTouchPoint.pageY;
            // 当用户触摸的纵向距离大于横向距离，能忽略此次划动，不触发各左滑动的动作
            if (isScroll && Math.abs(moveX - startX) < Math.abs(moveY - startY)) {
                unBindTouchCancelFn();
                unBindTouchMoveFn();
                unBindTouchEndFn();
                return;
            }
            // 计算向左滑动的距离
            this.distanceX = moveX - startX + oldDistanceX;
            if (this.distanceX > 0) {
                this.distanceX = 0;
            }
            if (this.distanceX < -maxDistance) {
                this.distanceX = -maxDistance;
            }
            // 设置相应样式，并阻止事件冒泡和默认事件
            this.transform = `translateX(${this.distanceX}px)`;
            isScroll = false;
            moveEvent.stopPropagation();
            return false;
        });
        const touchEndFn = () => {
            this.isFocus = false;
            const endTime = Date.now();
            const distance = oldDistanceX - this.distanceX;
            // 当用户触摸完成后，时间小于 100ms，并且距离大于20，则认为是惯性触摸
            if (endTime - startTime < 100 && Math.abs(distance) > 20 * this.scale) {
                this.distanceX = distance < 0 ? 0 : -maxDistance;
            }
            else {
                // 否则按 50% 区分向左还是向右
                this.distanceX = this.distanceX > maxDistance / -2 ? 0 : -maxDistance;
            }
            this.transitionDuration = '';
            this.transform = `translateX(${this.distanceX}px)`;
            unBindTouchCancelFn();
            unBindTouchMoveFn();
            unBindTouchEndFn();
        };
        unBindTouchEndFn = this.renderer.listen('document', 'touchend', touchEndFn);
        unBindTouchCancelFn = this.renderer.listen('document', 'touchcancel', touchEndFn);
        setTimeout(() => {
            if (isClick) {
                unBindTouchCancelFn();
                unBindTouchMoveFn();
                unBindTouchEndFn();
            }
        }, 100);
    }
};
ListSlidingComponent.ctorParameters = () => [
    { type: ListEventService },
    { type: Number, decorators: [{ type: Inject, args: [UI_SCREEN_SCALE,] }] },
    { type: ListActivatedService },
    { type: Renderer2 }
];
__decorate([
    HostBinding('style.transitionDuration'),
    __metadata("design:type", String)
], ListSlidingComponent.prototype, "transitionDuration", void 0);
__decorate([
    HostBinding('style.transform'),
    __metadata("design:type", String)
], ListSlidingComponent.prototype, "transform", void 0);
__decorate([
    HostListener('touchstart', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], ListSlidingComponent.prototype, "touchstart", null);
ListSlidingComponent = __decorate([
    Component({
        selector: 'ui-list-sliding',
        template: "<ng-content></ng-content>"
    }),
    __param(1, Inject(UI_SCREEN_SCALE)),
    __metadata("design:paramtypes", [ListEventService, Number, ListActivatedService,
        Renderer2])
], ListSlidingComponent);

let ListThumbnailComponent = class ListThumbnailComponent {
};
ListThumbnailComponent = __decorate([
    Component({
        selector: 'ui-list-thumbnail',
        template: "<ng-content></ng-content>"
    })
], ListThumbnailComponent);

let AlertController = class AlertController {
    constructor() {
        this.alertActionSource = new Subject();
        this.alertConfigSource = new Subject();
        this.alertAction = this.alertActionSource.asObservable();
        this.alertConfig = this.alertConfigSource.asObservable();
    }
    show(params) {
        // 当接收到用户事件时，发布相应事件，并返回 promise，当用户点击确定时，resolve 当前 promise;
        this.alertConfigSource.next(typeof params === 'string' ? {
            content: params
        } : params);
        return new Promise((resolve) => {
            const sub = this.alertAction.subscribe(() => {
                resolve();
                sub.unsubscribe();
            });
        });
    }
    publishAction() {
        // 发布用户点击弹出框的确认按扭事件
        this.alertActionSource.next();
    }
};
AlertController = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], AlertController);

let DialogController = class DialogController {
    constructor() {
        this.actionEvent = new Subject();
        this.configSource = new Subject();
        this.action = this.actionEvent.asObservable();
        this.config = this.configSource.asObservable();
    }
    show(params) {
        this.configSource.next(typeof params === 'string' ? {
            content: params
        } : params);
        return new Promise((resolve) => {
            const sub = this.action.subscribe((result) => {
                resolve(result);
                sub.unsubscribe();
            });
        });
    }
    publishAction(result) {
        this.actionEvent.next(result);
    }
};
DialogController = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], DialogController);

let ModalController = class ModalController {
    constructor() {
        this.displayTemplate = new Subject();
        this.hideEvent = new Subject();
        this.template = this.displayTemplate.asObservable();
        this.onHide = this.hideEvent.asObservable();
    }
    show(template) {
        this.displayTemplate.next(template);
    }
    hide() {
        this.hideEvent.next();
    }
};
ModalController = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], ModalController);

var ToastType;
(function (ToastType) {
    ToastType["Default"] = "default";
    ToastType["Primary"] = "primary";
    ToastType["Info"] = "info";
    ToastType["Warning"] = "warning";
    ToastType["Success"] = "success";
    ToastType["Danger"] = "danger";
})(ToastType || (ToastType = {}));
let ToastController = class ToastController {
    constructor() {
        this.notifySource = new Subject();
        this.notify = this.notifySource.asObservable();
    }
    push(config) {
        this.notifySource.next(config);
    }
};
ToastController = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], ToastController);

let AlertComponent = class AlertComponent {
    constructor(alertController, router) {
        this.alertController = alertController;
        this.router = router;
        this.show = false;
        this.title = '';
        this.content = '';
        this.btnText = '';
        this.subs = [];
    }
    get titleIsString() {
        return typeof this.title === 'string';
    }
    get contentIsString() {
        return typeof this.content === 'string';
    }
    ngOnInit() {
        if (this.router) {
            this.subs.push(this.router.events.subscribe(event => {
                if (event instanceof NavigationStart) {
                    this.checked();
                }
            }));
        }
        // 订阅alert事件
        this.subs.push(this.alertController.alertConfig.subscribe((params) => {
            // 设置状态，以弹出对话框
            this.show = true;
            this.title = params.title;
            this.content = params.content;
            this.btnText = params.btnText;
        }));
    }
    ngOnDestroy() {
        this.subs.forEach(item => item.unsubscribe());
    }
    checked() {
        // 当点击确认按扭时关闭弹窗
        this.show = false;
    }
    hide() {
        // 当弹窗动画完成时，发布事件
        this.alertController.publishAction();
    }
};
AlertComponent.ctorParameters = () => [
    { type: AlertController },
    { type: Router, decorators: [{ type: Optional }] }
];
AlertComponent = __decorate([
    Component({
        selector: 'ui-alert',
        template: "<ui-dialog-base [show]=\"show\" (uiHide)=\"hide()\">\n  <ui-modal>\n    <ng-container *ngIf=\"title\">\n      <ui-modal-header *ngIf=\"titleIsString\">{{ title }}</ui-modal-header>\n      <ui-modal-header *ngIf=\"!titleIsString\">\n        <ng-container [ngTemplateOutlet]=\"title\"></ng-container>\n      </ui-modal-header>\n    </ng-container>\n    <ui-modal-inner *ngIf=\"contentIsString\">{{ content }}</ui-modal-inner>\n    <ui-modal-inner *ngIf=\"!contentIsString\">\n      <ng-container [ngTemplateOutlet]=\"content\"></ng-container>\n    </ui-modal-inner>\n    <ui-modal-footer class=\"ui-alert-footer\">\n      <button (click)=\"checked()\" type=\"button\" class=\"ui-btn ui-btn-lg ui-btn-block\">{{ btnText || '\u786E\u5B9A' }}</button>\n    </ui-modal-footer>\n  </ui-modal>\n</ui-dialog-base>"
    }),
    __param(1, Optional()),
    __metadata("design:paramtypes", [AlertController,
        Router])
], AlertComponent);

let DialogComponent = class DialogComponent {
    constructor(dialogController, router) {
        this.dialogController = dialogController;
        this.router = router;
        this.show = false;
        this.title = '';
        this.content = '';
        this.btnsText = ['取消', '确认'];
        this.result = false;
        this.subs = [];
    }
    get titleIsString() {
        return typeof this.title === 'string';
    }
    get contentIsString() {
        return typeof this.content === 'string';
    }
    ngOnInit() {
        if (this.router) {
            this.subs.push(this.router.events.subscribe(event => {
                if (event instanceof NavigationStart) {
                    this.show = false;
                }
            }));
        }
        // 订阅用户事件
        this.subs.push(this.dialogController.config.subscribe((params) => {
            // 设置动画状态
            this.show = true;
            // 赋值相应参数
            this.title = params.title;
            this.content = params.content;
            if (params.btnsText) {
                this.btnsText = params.btnsText;
            }
        }));
    }
    ngOnDestroy() {
        this.subs.forEach(item => item.unsubscribe());
    }
    checked(result) {
        // 当用户点击按扭时，关闭弹窗
        this.show = false;
        this.result = result;
    }
    hide() {
        // 当弹窗关闭动画完成时，发布相应事件
        this.dialogController.publishAction(this.result);
    }
};
DialogComponent.ctorParameters = () => [
    { type: DialogController },
    { type: Router, decorators: [{ type: Optional }] }
];
DialogComponent = __decorate([
    Component({
        selector: 'ui-dialog',
        template: "<ui-dialog-base [show]=\"show\" (uiHide)=\"hide()\">\n  <ui-modal>\n    <ng-container *ngIf=\"title\">\n      <ui-modal-header *ngIf=\"titleIsString\">{{ title }}</ui-modal-header>\n      <ui-modal-header *ngIf=\"!titleIsString\">\n        <ng-container [ngTemplateOutlet]=\"title\"></ng-container>\n      </ui-modal-header>\n    </ng-container>\n    <ui-modal-inner *ngIf=\"contentIsString\">{{ content }}</ui-modal-inner>\n    <ui-modal-inner *ngIf=\"!contentIsString\">\n      <ng-container [ngTemplateOutlet]=\"content\"></ng-container>\n    </ui-modal-inner>\n    <ui-modal-footer class=\"ui-dialog-footer\">\n      <button (click)=\"checked(false)\" type=\"button\" class=\"ui-btn ui-btn-lg ui-btn-block\">{{ btnsText[0] }}</button>\n      <button (click)=\"checked(true)\" type=\"button\" class=\"ui-btn ui-btn-lg ui-btn-block\">{{ btnsText[1] }}</button>\n    </ui-modal-footer>\n  </ui-modal>\n</ui-dialog-base>"
    }),
    __param(1, Optional()),
    __metadata("design:paramtypes", [DialogController,
        Router])
], DialogComponent);

let DialogBaseComponent = class DialogBaseComponent {
    constructor() {
        this.uiHide = new EventEmitter();
        this.display = false;
        this._show = false;
    }
    set show(value) {
        this._show = value;
        if (value) {
            this.display = true;
        }
    }
    get show() {
        return this._show;
    }
    animationEnd() {
        this.uiHide.emit();
        this.display = false;
    }
    hide() {
        if (!this.show) {
            this.display = false;
        }
    }
};
__decorate([
    Output(),
    __metadata("design:type", Object)
], DialogBaseComponent.prototype, "uiHide", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [Boolean])
], DialogBaseComponent.prototype, "show", null);
DialogBaseComponent = __decorate([
    Component({
        selector: 'ui-dialog-base',
        template: "<ui-mask [show]=\"show\" (uiHide)=\"animationEnd()\">\n  <div class=\"ui-dialog-base\" *ngIf=\"display\" (animationend)=\"hide()\" [ngClass]=\"show ? 'ui-in' : 'ui-out'\">\n    <ng-content></ng-content>\n  </div>\n</ui-mask>"
    })
], DialogBaseComponent);

let MaskComponent = class MaskComponent {
    constructor() {
        this.uiHide = new EventEmitter();
        this.show = false;
    }
    done() {
        if (!this.show) {
            this.uiHide.emit();
        }
    }
};
__decorate([
    Output(),
    __metadata("design:type", Object)
], MaskComponent.prototype, "uiHide", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], MaskComponent.prototype, "show", void 0);
MaskComponent = __decorate([
    Component({
        selector: 'ui-mask',
        template: "<div [ngClass]=\"{'ui-show': show}\" class=\"ui-mask\" (webkitTransitionEnd)=\"done()\" (transitionend)=\"done()\"></div>\n<ng-content></ng-content>"
    })
], MaskComponent);

let ModalComponent = class ModalComponent {
};
ModalComponent = __decorate([
    Component({
        selector: 'ui-modal',
        template: "<ng-content></ng-content>\n"
    })
], ModalComponent);

let ModalBaseComponent = class ModalBaseComponent {
    constructor(modalController, router) {
        this.modalController = modalController;
        this.router = router;
        this.isShow = false;
        this.subs = [];
    }
    ngOnInit() {
        if (this.router) {
            this.subs.push(this.router.events.subscribe(event => {
                if (event instanceof NavigationStart) {
                    this.isShow = false;
                }
            }));
        }
        this.subs.push(this.modalController.template.subscribe(template => {
            this.isShow = true;
            this.template = template;
        }));
        this.subs.push(this.modalController.onHide.subscribe(() => {
            this.isShow = false;
        }));
    }
    ngOnDestroy() {
        this.subs.forEach(item => item.unsubscribe());
    }
    animationEnd() {
        this.template = null;
    }
};
ModalBaseComponent.ctorParameters = () => [
    { type: ModalController },
    { type: Router, decorators: [{ type: Optional }] }
];
ModalBaseComponent = __decorate([
    Component({
        selector: 'ui-modal-base',
        template: "<ui-dialog-base [show]=\"isShow\" (uiHide)=\"animationEnd()\">\n  <ng-container [ngTemplateOutlet]=\"template\"></ng-container>\n</ui-dialog-base>"
    }),
    __param(1, Optional()),
    __metadata("design:paramtypes", [ModalController,
        Router])
], ModalBaseComponent);

let ModalFooterComponent = class ModalFooterComponent {
};
ModalFooterComponent = __decorate([
    Component({
        selector: 'ui-modal-footer',
        template: "<ng-content></ng-content>"
    })
], ModalFooterComponent);

let ModalHeaderComponent = class ModalHeaderComponent {
};
ModalHeaderComponent = __decorate([
    Component({
        selector: 'ui-modal-header',
        template: "<ng-content></ng-content>"
    })
], ModalHeaderComponent);

let ModalInnerComponent = class ModalInnerComponent {
};
ModalInnerComponent = __decorate([
    Component({
        selector: 'ui-modal-inner',
        template: "<ng-content></ng-content>"
    })
], ModalInnerComponent);

let ToastComponent = class ToastComponent {
    constructor(toastService) {
        this.toastService = toastService;
        this.messageList = [];
        this.fadeIn = false;
        this.fadeOut = false;
        this.timer = null;
    }
    ngOnInit() {
        this.toastService.notify.pipe(delay(0)).subscribe((config) => {
            this.fadeOut = false;
            this.fadeIn = true;
            const _config = {};
            _config.rawConfig = config;
            switch (config.type) {
                case ToastType.Default:
                    _config.type = 'ui-default';
                    break;
                case ToastType.Primary:
                    _config.type = 'ui-primary';
                    break;
                case ToastType.Info:
                    _config.type = 'ui-info';
                    break;
                case ToastType.Success:
                    _config.type = 'ui-success';
                    break;
                case ToastType.Warning:
                    _config.type = 'ui-warning';
                    break;
                case ToastType.Danger:
                    _config.type = 'ui-danger';
                    break;
                default:
                    _config.type = 'ui-default';
            }
            _config.content = config.content || '';
            _config.time = config.time || 2000;
            this.messageList.push(_config);
            this.timer = setTimeout(() => {
                for (let i = 0; i < this.messageList.length; i++) {
                    if (this.messageList[i] === _config) {
                        this.messageList.splice(i, 1);
                        return;
                    }
                }
                this.fadeIn = false;
                this.fadeOut = true;
            }, _config.time);
        });
    }
    ngOnDestroy() {
        clearTimeout(this.timer);
    }
};
ToastComponent.ctorParameters = () => [
    { type: ToastController }
];
__decorate([
    HostBinding('class.ui-toast-fadein'),
    __metadata("design:type", Boolean)
], ToastComponent.prototype, "fadeIn", void 0);
__decorate([
    HostBinding('class.ui-toast-fadeout'),
    __metadata("design:type", Boolean)
], ToastComponent.prototype, "fadeOut", void 0);
ToastComponent = __decorate([
    Component({
        selector: 'ui-toast',
        template: "<div *ngFor=\"let config of messageList;\"\n     class=\"ui-toast\"\n     [ngClass]=\"config.type || 'ui-default'\">\n  {{ config.content }}\n</div>",
        animations: [
            trigger('toastState', [state('*', style({
                    opacity: 1
                })), transition(':leave', animate('.15s', style({
                    opacity: 0,
                }))), transition(':enter', [style({
                        opacity: 0,
                    }), animate('.15s', style({
                        opacity: 1,
                    }))])])
        ]
    }),
    __metadata("design:paramtypes", [ToastController])
], ToastComponent);

let SegmentService = class SegmentService {
    constructor() {
        this.checkedSource = new Subject();
        this.onChecked = this.checkedSource.asObservable();
    }
    checked(option) {
        this.checkedSource.next(option);
    }
};
SegmentService = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], SegmentService);

function inputAttrToBoolean$1(value) {
    return value === '' || !!value;
}
let SegmentButtonComponent = class SegmentButtonComponent {
    constructor(elementRef, segmentService) {
        this.elementRef = elementRef;
        this.segmentService = segmentService;
        this.uiChecked = new EventEmitter();
        this.value = '';
        this._disabled = false;
        this._selected = false;
    }
    set disabled(isDisabled) {
        this._disabled = inputAttrToBoolean$1(isDisabled);
    }
    get disabled() {
        return this._disabled;
    }
    set selected(isSelected) {
        this._selected = inputAttrToBoolean$1(isSelected);
    }
    get selected() {
        return this._selected;
    }
    click() {
        if (!this.disabled) {
            this.segmentService.checked(this);
            this.uiChecked.emit(this);
        }
    }
    ngAfterViewInit() {
        this.nativeElement = this.elementRef.nativeElement;
    }
};
SegmentButtonComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: SegmentService }
];
__decorate([
    Output(),
    __metadata("design:type", Object)
], SegmentButtonComponent.prototype, "uiChecked", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], SegmentButtonComponent.prototype, "value", void 0);
__decorate([
    Input(),
    HostBinding('class.ui-disabled'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], SegmentButtonComponent.prototype, "disabled", null);
__decorate([
    Input(),
    HostBinding('class.ui-selected'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], SegmentButtonComponent.prototype, "selected", null);
__decorate([
    HostListener('click'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SegmentButtonComponent.prototype, "click", null);
SegmentButtonComponent = __decorate([
    Component({
        selector: 'ui-segment-button',
        template: "<ng-content></ng-content>"
    }),
    __metadata("design:paramtypes", [ElementRef,
        SegmentService])
], SegmentButtonComponent);

var SegmentComponent_1;
let SegmentComponent = SegmentComponent_1 = class SegmentComponent {
    constructor(segmentService) {
        this.segmentService = segmentService;
        this.uiChange = new EventEmitter();
        this.selectedIndex = 0;
    }
    ngAfterContentInit() {
        let defaultOption;
        this.options.forEach((option, index) => {
            if (option.selected) {
                defaultOption = option;
                this.selectedIndex = index;
            }
        });
        if (!defaultOption) {
            defaultOption = this.options.toArray()[this.selectedIndex];
        }
        if (!defaultOption) {
            defaultOption = this.options.first;
            this.selectedIndex = 0;
        }
        if (defaultOption) {
            this.value = defaultOption.value;
            defaultOption.selected = true;
            this.defaultOption = defaultOption;
        }
        this.sub = this.segmentService.onChecked.subscribe((option) => {
            this.options.forEach((op, index) => {
                if (op === option) {
                    op.selected = true;
                    this.value = option.value;
                    this.selectedIndex = index;
                    if (this.onChange) {
                        this.onChange(this.value);
                    }
                    if (this.onTouched) {
                        this.onTouched(this.value);
                    }
                    this.uiChange.emit(this.value);
                }
                else {
                    op.selected = false;
                }
            });
        });
    }
    ngOnDestroy() {
        this.sub.unsubscribe();
    }
    writeValue(value) {
        this.value = value;
        if (this.options) {
            let selectedOption;
            this.options.forEach((item, index) => {
                item.selected = false;
                if (item.value === value || `${item.value}` === value || item.value === `${value}`) {
                    selectedOption = item;
                    this.selectedIndex = index;
                }
            });
            if (selectedOption) {
                selectedOption.selected = true;
            }
        }
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
};
SegmentComponent.ctorParameters = () => [
    { type: SegmentService }
];
__decorate([
    ContentChildren(SegmentButtonComponent),
    __metadata("design:type", QueryList)
], SegmentComponent.prototype, "options", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], SegmentComponent.prototype, "uiChange", void 0);
__decorate([
    Input(),
    __metadata("design:type", Number)
], SegmentComponent.prototype, "selectedIndex", void 0);
SegmentComponent = SegmentComponent_1 = __decorate([
    Component({
        selector: 'ui-segment',
        template: "<ng-content></ng-content>",
        providers: [{
                provide: NG_VALUE_ACCESSOR,
                useExisting: SegmentComponent_1,
                multi: true
            }, SegmentService]
    }),
    __metadata("design:paramtypes", [SegmentService])
], SegmentComponent);

let SlideItemComponent = class SlideItemComponent {
    constructor() {
        this.state = '';
        this.animateProgress = 0;
    }
    get isIn() {
        return this.state === 'in';
    }
    get isOut() {
        return this.state === 'out';
    }
    get transform() {
        return `translateX(${this.animateProgress * 100}%)`;
    }
};
__decorate([
    HostBinding('class.ui-in'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], SlideItemComponent.prototype, "isIn", null);
__decorate([
    HostBinding('class.ui-out'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], SlideItemComponent.prototype, "isOut", null);
__decorate([
    HostBinding('style.transform'),
    HostBinding('style.webkitTransform'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], SlideItemComponent.prototype, "transform", null);
SlideItemComponent = __decorate([
    Component({
        selector: 'ui-slide-item',
        template: "<ng-content></ng-content>\n"
    })
], SlideItemComponent);

let SlideComponent = class SlideComponent {
    constructor(elementRef, scale, appController) {
        this.elementRef = elementRef;
        this.scale = scale;
        this.appController = appController;
        this.uiPlayed = new EventEmitter();
        this.initIndex = 0;
        this.speed = 2000;
        this.steps = 35;
        this.progress = 0;
        this.oldProgress = 0;
        this.timer = null;
        this.subs = [];
        this._autoPlay = true;
    }
    set autoPlay(v) {
        this._autoPlay = v;
        if (v) {
            this.play();
        }
    }
    get autoPlay() {
        return this._autoPlay;
    }
    get index() {
        return Math.ceil(this.progress - 0.5) % this.items.length;
    }
    ngOnInit() {
        this.progress = this.initIndex;
    }
    ngAfterViewInit() {
        this.containerWidth = this.elementRef.nativeElement.offsetWidth;
        this.subs.push(this.appController.onResize.subscribe(() => {
            this.containerWidth = this.elementRef.nativeElement.offsetWidth;
        }));
        setTimeout(() => {
            this.updateChildrenStyle(this.progress);
            this.play();
        });
        this.subs.push(this.items.changes.pipe(delay(0)).subscribe(() => {
            this.updateChildrenStyle(0);
            this.play();
        }));
    }
    ngOnDestroy() {
        clearTimeout(this.timer);
        this.subs.forEach(item => item.unsubscribe());
    }
    touch(event) {
        clearTimeout(this.timer);
        cancelAnimationFrame(this.animateId);
        event.srcEvent.stopPropagation();
        const firstDirection = event.firstDirection;
        const durationTime = event.durationTime;
        const element = this.elementRef.nativeElement;
        this.containerWidth = element.offsetWidth;
        const len = this.items.length;
        const distanceX = event.distanceX;
        const isFirstMoving = event.first;
        const autoUpdateStyle = () => {
            let min = 0;
            const max = 20;
            const p = this.progress;
            let target;
            if (durationTime < 200 && Math.abs(distanceX) > 20 * this.scale) {
                target = distanceX > 0 ? Math.floor(p) : Math.floor(p) + 1;
            }
            else {
                target = p % 1 > 0.5 ? Math.ceil(p) : Math.floor(p);
            }
            const distance = target - p;
            const updateStyle = () => {
                min++;
                this.progress = (p + Easing.Cubic.Out(min / max) * distance) % len;
                this.updateChildrenStyle(this.progress);
                if (min < max) {
                    this.animateId = requestAnimationFrame(updateStyle);
                }
                else {
                    this.uiPlayed.emit(this.progress);
                    this.play();
                }
            };
            this.animateId = requestAnimationFrame(updateStyle);
        };
        if (this.items.length < 2) {
            return;
        }
        if (firstDirection === 'up' || firstDirection === 'down') {
            this.touchAction = 'auto';
            if (event.type === 'touchend') {
                autoUpdateStyle();
            }
            return;
        }
        this.touchAction = 'pan-x';
        if (isFirstMoving) {
            this.oldProgress = this.progress;
        }
        this.progress = this.oldProgress - (distanceX) / this.containerWidth;
        if (this.progress < 0) {
            this.progress += len;
        }
        this.updateChildrenStyle(this.progress);
        if (event.type === 'touchend') {
            this.touchAction = undefined;
            autoUpdateStyle();
        }
    }
    play() {
        clearTimeout(this.timer);
        if (!this.autoPlay) {
            return;
        }
        if (this.items.length < 2) {
            return;
        }
        this.timer = setTimeout(() => {
            this.animate();
        }, this.speed);
    }
    animate() {
        const progress = this.progress;
        let i = 0;
        const animateFn = () => {
            i++;
            this.progress = (progress + Easing.Cubic.Out(i / this.steps)) % this.items.length;
            this.updateChildrenStyle(this.progress);
            if (i < this.steps) {
                this.animateId = requestAnimationFrame(animateFn);
            }
            else {
                this.uiPlayed.emit(this.progress);
                this.play();
            }
        };
        this.animateId = requestAnimationFrame(animateFn);
    }
    updateChildrenStyle(progress) {
        const currentIndex = Math.floor(progress);
        const p = progress % 1;
        const len = this.items.length;
        this.items.forEach((item, index) => {
            if (index === currentIndex) {
                item.state = 'in';
                item.animateProgress = -p;
            }
            else if (index === (currentIndex + 1) % len) {
                item.state = 'out';
                item.animateProgress = -p + 1;
            }
            else {
                item.animateProgress = -1;
                item.state = '';
            }
        });
    }
};
SlideComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: Number, decorators: [{ type: Inject, args: [UI_SCREEN_SCALE,] }] },
    { type: AppController }
];
__decorate([
    ContentChildren(SlideItemComponent),
    __metadata("design:type", QueryList)
], SlideComponent.prototype, "items", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], SlideComponent.prototype, "uiPlayed", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], SlideComponent.prototype, "initIndex", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], SlideComponent.prototype, "speed", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], SlideComponent.prototype, "steps", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [Boolean])
], SlideComponent.prototype, "autoPlay", null);
SlideComponent = __decorate([
    Component({
        selector: 'ui-slide',
        template: "<div class=\"ui-slide-group\" (uiPan)=\"touch($event)\" [style.touchAction]=\"touchAction\">\n  <ng-content></ng-content>\n</div>\n<div class=\"ui-slide-buttons\" *ngIf=\"items.length > 1\">\n  <div class=\"ui-slide-btn\" *ngFor=\"let item of items; let i = index\" [ngClass]=\"{'ui-active': i === index}\"></div>\n</div>"
    }),
    __param(1, Inject(UI_SCREEN_SCALE)),
    __metadata("design:paramtypes", [ElementRef, Number, AppController])
], SlideComponent);

class TouchManager {
    init(points) {
        let isLock = false;
        this.touchSubscription = fromEvent(this.elementRef.nativeElement, 'touchstart')
            .pipe(filter(() => !isLock), debounceTime(points === 1 ? 0 : 100), filter(ev => {
            return ev.touches.length === points;
        }), tap(() => {
            isLock = true;
        }))
            .subscribe(touchStartEvent => {
            this.touchStart(touchStartEvent);
            const unSubMove = fromEvent(this.elementRef.nativeElement, 'touchmove')
                .subscribe(touchMoveEvent => {
                this.touchMove(touchMoveEvent, unListen);
            });
            const unSubEnd = merge(fromEvent(this.elementRef.nativeElement, 'touchend'), fromEvent(this.elementRef.nativeElement, 'touchcancel'))
                .subscribe(ev => {
                unListen();
                this.touchEnd(ev);
            });
            const unListen = function () {
                isLock = false;
                unSubMove.unsubscribe();
                unSubEnd.unsubscribe();
            };
        });
    }
    destroy() {
        if (this.touchSubscription) {
            this.touchSubscription.unsubscribe();
        }
    }
}

let PanDirective = class PanDirective extends TouchManager {
    constructor(elementRef) {
        super();
        this.elementRef = elementRef;
        this.uiPan = new EventEmitter();
        this.cumulativeDistanceX = 0;
        this.cumulativeDistanceY = 0;
        this.distanceX = 0;
        this.distanceY = 0;
        this.oldMoveX = 0;
        this.oldMoveY = 0;
        this.direction = 'origin';
        this.isFirst = true;
    }
    ngOnInit() {
        super.init(1);
    }
    ngOnDestroy() {
        super.destroy();
    }
    touchStart(event) {
        const startPoint = event.touches[0];
        this.startX = startPoint.clientX;
        this.startY = startPoint.clientY;
        this.moveX = this.startX;
        this.moveY = this.startY;
        this.distanceX = 0;
        this.distanceY = 0;
        this.oldMoveX = 0;
        this.oldMoveY = 0;
        this.direction = 'origin';
        this.isFirst = true;
        this.startTime = event.timeStamp;
    }
    touchMove(event, unListen) {
        const point = event.touches[0];
        this.oldMoveX = this.moveX;
        this.oldMoveY = this.moveY;
        this.moveX = point.clientX;
        this.moveY = point.clientY;
        this.distanceX = this.moveX - this.startX;
        this.distanceY = this.moveY - this.startY;
        if (this.isFirst) {
            if (Math.abs(this.distanceX) > Math.abs(this.distanceY)) {
                this.direction = this.distanceX > 0 ? 'right' : 'left';
            }
            else {
                this.direction = this.distanceY > 0 ? 'down' : 'up';
            }
        }
        this.cumulativeDistanceX += this.moveX - this.oldMoveX;
        this.cumulativeDistanceY += this.moveY - this.oldMoveY;
        this.uiPan.emit({
            durationTime: event.timeStamp - this.startTime,
            first: this.isFirst,
            eventName: 'uiPan',
            firstDirection: this.direction,
            startX: this.startX,
            startY: this.startY,
            moveX: this.moveX,
            moveY: this.moveY,
            distanceX: this.distanceX,
            distanceY: this.distanceY,
            cumulativeDistanceX: this.cumulativeDistanceX,
            cumulativeDistanceY: this.cumulativeDistanceY,
            srcEvent: event,
            type: 'touchmove',
            stop: unListen,
            resetCumulative: () => {
                this.reset();
            }
        });
        this.isFirst = false;
    }
    touchEnd(event) {
        if (this.isFirst) {
            return;
        }
        this.uiPan.emit({
            durationTime: event.timeStamp - this.startTime,
            first: false,
            eventName: 'uiPan',
            firstDirection: this.direction,
            startX: this.startX,
            startY: this.startY,
            moveX: this.moveX,
            moveY: this.moveY,
            distanceX: this.distanceX,
            distanceY: this.distanceY,
            cumulativeDistanceX: this.cumulativeDistanceX,
            cumulativeDistanceY: this.cumulativeDistanceY,
            srcEvent: event,
            type: 'touchend',
            /*tslint:disable*/
            stop: function noop() {
            },
            /*tslint:enable*/
            resetCumulative: () => {
                this.reset();
            }
        });
    }
    reset() {
        this.cumulativeDistanceY = 0;
        this.cumulativeDistanceX = 0;
    }
};
PanDirective.ctorParameters = () => [
    { type: ElementRef }
];
__decorate([
    Output(),
    __metadata("design:type", Object)
], PanDirective.prototype, "uiPan", void 0);
PanDirective = __decorate([
    Directive({
        selector: '[uiPan]'
    }),
    __metadata("design:paramtypes", [ElementRef])
], PanDirective);

let PinchDirective = class PinchDirective extends TouchManager {
    constructor(elementRef) {
        super();
        this.elementRef = elementRef;
        this.uiPinch = new EventEmitter();
        this.cumulativeScale = 1;
        this.scale = 1;
        this.oldScale = 1;
        this.distanceX = 0;
        this.distanceY = 0;
        this.cumulativeDistanceX = 0;
        this.cumulativeDistanceY = 0;
        this.oldMoveX = 0;
        this.oldMoveY = 0;
        this.isFirst = true;
    }
    ngOnInit() {
        super.init(2);
    }
    ngOnDestroy() {
        super.destroy();
    }
    touchStart(event) {
        const p1 = event.touches[0];
        const p2 = event.touches[1];
        const centerPoint = this.getCenterPoint(p1, p2);
        this.startX = centerPoint.x;
        this.startY = centerPoint.y;
        this.moveX = this.startX;
        this.moveY = this.startY;
        this.startDistance = this.getChordLength(p1, p2);
        this.scale = 1;
        this.isFirst = true;
        this.startTime = event.timeStamp;
        event.preventDefault();
    }
    touchMove(event, unListen) {
        const p1 = event.touches[0];
        const p2 = event.touches[1];
        const centerPoint = this.getCenterPoint(p1, p2);
        this.oldMoveX = this.moveX;
        this.oldMoveY = this.moveY;
        this.moveX = centerPoint.x;
        this.moveY = centerPoint.y;
        this.distanceX = this.moveX - this.startX;
        this.distanceY = this.moveY - this.startY;
        this.cumulativeDistanceX += this.moveX - this.oldMoveX;
        this.cumulativeDistanceY += this.moveY - this.oldMoveY;
        const moveArea = this.getChordLength(p1, p2);
        this.oldScale = this.scale;
        this.scale = moveArea / this.startDistance;
        this.uiPinch.emit({
            durationTime: event.timeStamp - this.startTime,
            first: this.isFirst,
            eventName: 'uiPinch',
            startX: this.startX,
            startY: this.startY,
            moveX: this.moveX,
            moveY: this.moveY,
            distanceX: this.distanceX,
            distanceY: this.distanceY,
            cumulativeDistanceX: this.cumulativeDistanceX,
            cumulativeDistanceY: this.cumulativeDistanceY,
            stop: unListen,
            srcEvent: event,
            type: 'touchmove',
            resetCumulative: () => {
                this.cumulativeScale = 1;
                this.cumulativeDistanceX = 0;
                this.cumulativeDistanceY = 0;
            },
            scale: this.scale,
            cumulativeScale: this.cumulativeScale * this.scale
        });
        this.isFirst = false;
    }
    touchEnd(event) {
        if (this.isFirst) {
            return;
        }
        this.cumulativeScale = this.cumulativeScale * this.scale;
        this.uiPinch.emit({
            durationTime: event.timeStamp - this.startTime,
            first: false,
            eventName: 'uiPinch',
            startX: this.startX,
            startY: this.startY,
            moveX: this.moveX,
            moveY: this.moveY,
            distanceX: this.distanceX,
            distanceY: this.distanceY,
            cumulativeDistanceX: this.cumulativeDistanceX,
            cumulativeDistanceY: this.cumulativeDistanceY,
            /*tslint:disable*/
            stop: function noop() {
            },
            /*tslint:enable*/
            srcEvent: event,
            type: 'touchend',
            resetCumulative: () => {
                this.cumulativeScale = 1;
            },
            scale: this.scale,
            cumulativeScale: this.cumulativeScale
        });
    }
    getChordLength(p1, p2) {
        const x = Math.abs(p1.clientX - p2.clientX);
        const y = Math.abs(p1.clientY - p2.clientY);
        return Math.sqrt(x * x + y * y);
    }
    getCenterPoint(p1, p2) {
        return {
            x: (p1.clientX + p2.clientX) / 2,
            y: (p1.clientY + p2.clientY) / 2
        };
    }
};
PinchDirective.ctorParameters = () => [
    { type: ElementRef }
];
__decorate([
    Output(),
    __metadata("design:type", Object)
], PinchDirective.prototype, "uiPinch", void 0);
PinchDirective = __decorate([
    Directive({
        selector: '[uiPinch]'
    }),
    __metadata("design:paramtypes", [ElementRef])
], PinchDirective);

let UIModalModule = class UIModalModule {
};
UIModalModule = __decorate([
    NgModule({
        imports: [
            CommonModule
        ],
        declarations: [
            AlertComponent,
            DialogComponent,
            DialogBaseComponent,
            MaskComponent,
            ModalComponent,
            ModalBaseComponent,
            ModalFooterComponent,
            ModalHeaderComponent,
            ModalInnerComponent,
            ToastComponent
        ],
        exports: [
            AlertComponent,
            DialogComponent,
            DialogBaseComponent,
            MaskComponent,
            ModalComponent,
            ModalBaseComponent,
            ModalFooterComponent,
            ModalHeaderComponent,
            ModalInnerComponent,
            ToastComponent
        ]
    })
], UIModalModule);

let UIOtherModule = class UIOtherModule {
};
UIOtherModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            UIModalModule
        ],
        declarations: [
            ActionSheetComponent,
            ContentLoadingComponent,
            FixedBarComponent,
            LoadingComponent,
            StopPropagationDirective
        ],
        exports: [
            ActionSheetComponent,
            ContentLoadingComponent,
            FixedBarComponent,
            LoadingComponent,
            StopPropagationDirective
        ]
    })
], UIOtherModule);

let UIRouterModule = class UIRouterModule {
};
UIRouterModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            UIOtherModule
        ],
        declarations: [
            BackComponent,
            RouterComponent,
            TabComponent,
            TabBarComponent,
            TabBarItemComponent,
            TabViewComponent,
            TabViewItemComponent,
            ViewComponent,
            ComponentHostDirective
        ],
        exports: [
            BackComponent,
            RouterComponent,
            TabComponent,
            TabBarComponent,
            TabBarItemComponent,
            TabViewComponent,
            TabViewItemComponent,
            ViewComponent
        ]
    })
], UIRouterModule);

let UITouchModule = class UITouchModule {
};
UITouchModule = __decorate([
    NgModule({
        declarations: [
            PanDirective,
            PinchDirective,
        ],
        exports: [
            PanDirective,
            PinchDirective,
        ]
    })
], UITouchModule);

let UISlideModule = class UISlideModule {
};
UISlideModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            UITouchModule
        ],
        declarations: [
            SlideComponent,
            SlideItemComponent
        ],
        exports: [
            SlideComponent,
            SlideItemComponent
        ]
    })
], UISlideModule);

let UIImageViewerModule = class UIImageViewerModule {
};
UIImageViewerModule = __decorate([
    NgModule({
        imports: [
            UISlideModule,
            CommonModule,
            UITouchModule
        ],
        declarations: [
            ImageViewerComponent,
            ImageViewableDirective
        ],
        exports: [
            ImageViewerComponent,
            ImageViewableDirective
        ]
    })
], UIImageViewerModule);

let AppComponent = class AppComponent {
    constructor(document, env, scale, routeCacheController, location, appController, listActivatedService) {
        this.document = document;
        this.env = env;
        this.scale = scale;
        this.routeCacheController = routeCacheController;
        this.location = location;
        this.appController = appController;
        this.listActivatedService = listActivatedService;
        /**
         * 根据不同的屏幕宽度，设置 html 元素的 font-size
         * 主要用于 rem 布局，默认为 10
         */
        this.baseFontSize = 10;
        /**
         * 屏幕 X 轴默认逻辑像素
         */
        this.screenWidth = 375;
        /**
         * angular 路由的 name
         */
        this.name = PRIMARY_OUTLET;
        this.showMask = false;
        this.subs = [];
    }
    // 根据不同环境，需要把页面头部增高相对尺寸，以显示时间，电池电量等信息，这里通过样式来控制
    get isIphone() {
        return this.env === BrowserENV.iphone;
    }
    get isIphoneX() {
        return this.env === BrowserENV.iphoneX;
    }
    get isAndroid() {
        return this.env === BrowserENV.android;
    }
    get isDefault() {
        return this.env === BrowserENV.default;
    }
    get defaultDocWidth() {
        return this.screenWidth * this.scale;
    }
    ngOnInit() {
        // 根据当前应用是否在在路由动画转场中时，切换遮罩的显示与隐藏，防止多次点击
        this.subs.push(this.appController.transition.subscribe(b => {
            this.showMask = b;
        }));
        // 当屏幕大小改变时，变更环境变量
        this.subs.push(this.appController.onResize.subscribe((env) => {
            this.env = env;
        }));
        // 当路由地址变更时，设置路由缓存
        this.subs.push(this.location.subscribe(() => {
            this.routeCacheController.isCacheCurrentView(false);
        }));
        this.htmlElement = this.document.querySelector('html');
        this.resize();
    }
    ngOnDestroy() {
        this.subs.forEach(item => item.unsubscribe());
    }
    resize() {
        // 通过当前视口的宽度，动态设置 css rem 值的大小
        if (!this.htmlElement) {
            return;
        }
        const docWidth = this.htmlElement.getBoundingClientRect().width;
        const scale = docWidth / this.defaultDocWidth;
        this.htmlElement.style.fontSize = `${scale * this.baseFontSize}px`;
        this.appController.resize(getDeviceType());
    }
    documentTouchStart() {
        // 当页面有 list 组件时，如果其中某一项已打开 sliding，当用户点击其它地方时，需要发布全局事件，来收起 sliding
        this.listActivatedService.publish();
    }
};
AppComponent.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [UI_BROWSER_ENV,] }] },
    { type: Number, decorators: [{ type: Inject, args: [UI_SCREEN_SCALE,] }] },
    { type: RouteCacheController },
    { type: Location },
    { type: AppController },
    { type: ListActivatedService }
];
__decorate([
    Input(),
    __metadata("design:type", Number)
], AppComponent.prototype, "baseFontSize", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], AppComponent.prototype, "screenWidth", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], AppComponent.prototype, "name", void 0);
__decorate([
    HostBinding('class.ui-iphone'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], AppComponent.prototype, "isIphone", null);
__decorate([
    HostBinding('class.ui-iphone-x'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], AppComponent.prototype, "isIphoneX", null);
__decorate([
    HostBinding('class.ui-android'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], AppComponent.prototype, "isAndroid", null);
__decorate([
    HostBinding('class.ui-default'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], AppComponent.prototype, "isDefault", null);
__decorate([
    HostListener('window:resize'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AppComponent.prototype, "resize", null);
__decorate([
    HostListener('document:touchstart'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AppComponent.prototype, "documentTouchStart", null);
AppComponent = __decorate([
    Component({
        selector: 'ui-app',
        template: "<ng-content></ng-content>\n<ui-router [name]=\"name\"></ui-router>\n<ui-image-viewer></ui-image-viewer>\n<ui-modal-base></ui-modal-base>\n<ui-alert></ui-alert>\n<ui-toast></ui-toast>\n<ui-dialog></ui-dialog>\n<div class=\"ui-app-mask\" [class.ui-show]=\"showMask\"></div>"
    }),
    __param(0, Inject(DOCUMENT)),
    __param(1, Inject(UI_BROWSER_ENV)),
    __param(2, Inject(UI_SCREEN_SCALE)),
    __metadata("design:paramtypes", [Object, String, Number, RouteCacheController,
        Location,
        AppController,
        ListActivatedService])
], AppComponent);

let UIAppModule = class UIAppModule {
};
UIAppModule = __decorate([
    NgModule({
        imports: [
            UIRouterModule,
            UIModalModule,
            CommonModule,
            UIImageViewerModule
        ],
        declarations: [
            AppComponent
        ],
        exports: [
            AppComponent
        ]
    })
], UIAppModule);

let UICollectionModule = class UICollectionModule {
};
UICollectionModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            UITouchModule
        ],
        declarations: [
            CollectionComponent,
            CollectionItemComponent
        ],
        exports: [
            CollectionComponent,
            CollectionItemComponent
        ]
    })
], UICollectionModule);

let UIFlexModule = class UIFlexModule {
};
UIFlexModule = __decorate([
    NgModule({
        declarations: [
            FlexComponent,
            FlexItemComponent
        ],
        exports: [
            FlexComponent,
            FlexItemComponent
        ]
    })
], UIFlexModule);

let UIFormsModule = class UIFormsModule {
};
UIFormsModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            UIOtherModule,
            FormsModule,
            UIFlexModule
        ],
        declarations: [
            // 组件
            ButtonComponent,
            CheckboxComponent,
            DateComponent,
            OptionComponent,
            RadioComponent,
            PickerComponent,
            PickerColumnComponent,
            RangeComponent,
            SelectComponent,
            SwitchComponent,
            // 指令
            CheckboxRequiredValidator,
            SwitchRequiredValidator,
            RadioRequiredValidator,
            SelectRequiredValidator,
            DateRequiredValidator,
            SubmitDirective
        ],
        exports: [
            // 组件
            ButtonComponent,
            CheckboxComponent,
            DateComponent,
            OptionComponent,
            PickerComponent,
            PickerColumnComponent,
            RadioComponent,
            RangeComponent,
            SelectComponent,
            SwitchComponent,
            // 指令
            CheckboxRequiredValidator,
            SwitchRequiredValidator,
            RadioRequiredValidator,
            SelectRequiredValidator,
            DateRequiredValidator,
            SubmitDirective
        ],
        providers: [
            RadioStateService, {
                provide: UI_SELECT_ARROW_CLASSNAME,
                useValue: 'ui-caret'
            }
        ]
    })
], UIFormsModule);

let UILayoutModule = class UILayoutModule {
};
UILayoutModule = __decorate([
    NgModule({
        declarations: [
            ButtonsComponent,
            ContentComponent,
            FooterComponent,
            HeaderComponent,
            NavBarComponent,
            PageComponent,
            TitleComponent
        ],
        exports: [
            ButtonsComponent,
            ContentComponent,
            FooterComponent,
            HeaderComponent,
            NavBarComponent,
            PageComponent,
            TitleComponent
        ]
    })
], UILayoutModule);

let UIListModule = class UIListModule {
};
UIListModule = __decorate([
    NgModule({
        declarations: [
            ListComponent,
            ListHeaderComponent,
            ListInnerComponent,
            ListItemComponent,
            ListOptionComponent,
            ListSlidingComponent,
            ListThumbnailComponent
        ],
        exports: [
            ListComponent,
            ListHeaderComponent,
            ListInnerComponent,
            ListItemComponent,
            ListOptionComponent,
            ListSlidingComponent,
            ListThumbnailComponent
        ]
    })
], UIListModule);

let UIScrollModule = class UIScrollModule {
};
UIScrollModule = __decorate([
    NgModule({
        imports: [
            CommonModule
        ],
        declarations: [
            RefresherComponent,
            ScrollComponent,
            ScrollTabComponent,
            ScrollTabButtonComponent
        ],
        exports: [
            RefresherComponent,
            ScrollComponent,
            ScrollTabComponent,
            ScrollTabButtonComponent
        ]
    })
], UIScrollModule);

let UISegmentModule = class UISegmentModule {
};
UISegmentModule = __decorate([
    NgModule({
        declarations: [
            SegmentComponent,
            SegmentButtonComponent
        ],
        exports: [
            SegmentComponent,
            SegmentButtonComponent
        ]
    })
], UISegmentModule);

var UINativeModule_1;
let UINativeModule = UINativeModule_1 = class UINativeModule {
    static forRoot() {
        return {
            ngModule: UINativeModule_1,
            providers: [{
                    provide: RouteReuseStrategy,
                    useClass: UIRouteReuseStrategy
                }, {
                    provide: UI_BACK_ICON_CLASSNAME,
                    useValue: 'ui-icon-arrow-back'
                }, {
                    provide: UI_ROUTER_ANIMATION_STEPS,
                    useValue: navigator.userAgent.indexOf('web-2.0') > -1 ? 1 : 30
                }, {
                    provide: UI_DO_REFRESH_DISTANCE,
                    useValue: 40
                }, {
                    provide: UI_DO_LOAD_DISTANCE,
                    useValue: 600
                }, {
                    provide: UI_BROWSER_ENV,
                    useFactory: getDeviceType
                }, {
                    provide: UI_SCREEN_SCALE,
                    useValue: 1
                },
                AppController,
                RouteCacheController,
                AlertController,
                DialogController,
                ListActivatedService,
                ModalController,
                ToastController,
                ViewStateService,
                ImageViewerController
            ]
        };
    }
};
UINativeModule = UINativeModule_1 = __decorate([
    NgModule({
        exports: [
            UIAppModule,
            UICollectionModule,
            UIFlexModule,
            UIFormsModule,
            UIImageViewerModule,
            UILayoutModule,
            UIListModule,
            UIModalModule,
            UIOtherModule,
            UIRouterModule,
            UIScrollModule,
            UISegmentModule,
            UISlideModule,
            UITouchModule
        ]
    })
], UINativeModule);

/*
 * Public API Surface of ui-native
 */

/**
 * Generated bundle index. Do not edit.
 */

export { ActionSheetComponent, ActionSheetService, AlertComponent, AlertController, AppController, BackComponent, BrowserENV, ButtonComponent, ButtonsComponent, CheckboxComponent, CheckboxRequiredValidator, CollectionComponent, CollectionItemComponent, ComponentHostDirective, ContentComponent, ContentLoadingComponent, ContentLoadingController, DateComponent, DateRequiredValidator, DialogBaseComponent, DialogComponent, DialogController, FixedBarComponent, FlexComponent, FlexItemComponent, FooterComponent, HeaderComponent, ImageViewableDirective, ImageViewerComponent, ImageViewerController, ListActivatedService, ListComponent, ListEventService, ListHeaderComponent, ListInnerComponent, ListItemComponent, ListOptionComponent, ListSlidingComponent, ListThumbnailComponent, LoadingComponent, MaskComponent, ModalBaseComponent, ModalComponent, ModalController, ModalFooterComponent, ModalHeaderComponent, ModalInnerComponent, NavBarComponent, OptionComponent, PageComponent, PanDirective, PickerColumnComponent, PickerComponent, PickerService, PinchDirective, PullDownRefreshController, PullUpLoadController, RadioComponent, RadioRequiredValidator, RangeComponent, RefresherComponent, RouteCacheController, RouterComponent, RouterService, ScrollComponent, ScrollTabButtonComponent, ScrollTabComponent, SegmentButtonComponent, SegmentComponent, SelectComponent, SelectRequiredValidator, SlideComponent, SlideItemComponent, StopPropagationDirective, SubmitDirective, SwitchComponent, SwitchRequiredValidator, TabBarComponent, TabBarItemComponent, TabComponent, TabController, TabViewComponent, TabViewItemComponent, TitleComponent, ToastComponent, ToastController, ToastType, UINativeModule, UIRouteReuseStrategy, UI_BACK_ICON_CLASSNAME, UI_BROWSER_ENV, UI_DO_LOAD_DISTANCE, UI_DO_REFRESH_DISTANCE, UI_ROUTER_ANIMATION_STEPS, UI_SCREEN_SCALE, UI_SELECT_ARROW_CLASSNAME, UI_VIEW_INIT_STATE, UI_VIEW_INIT_TOUCH_BACK, ViewComponent, ViewState, ViewStateService, getDeviceType, inputAttrToBoolean, SelectService as ɵa, RadioStateService as ɵb, TabViewItemService as ɵc, ScrollTabService as ɵd, SegmentService as ɵe, TouchManager as ɵf, UIAppModule as ɵg, UIRouterModule as ɵh, UIOtherModule as ɵi, UIModalModule as ɵj, UIImageViewerModule as ɵk, UISlideModule as ɵl, UITouchModule as ɵm, AppComponent as ɵn, UICollectionModule as ɵo, UIFlexModule as ɵp, UIFormsModule as ɵq, UILayoutModule as ɵr, UIListModule as ɵs, UIScrollModule as ɵt, UISegmentModule as ɵu };
//# sourceMappingURL=cloudbae-frontend-ui-native.js.map
